@@ -1,6 +1,7 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
 // Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
+// Copyright Dirk Lemstra 2014
 //
 // Implementation of Drawable (Graphic objects)
 //
@@ -551,6 +552,26 @@ Magick::DrawableBase* Magick::DrawableCompositeImage::copy() const
   return new DrawableCompositeImage(*this);
 }
 
+Magick::DrawableDensity::DrawableDensity(const std::string &density_)
+  : _density(density_)
+{
+}
+
+Magick::DrawableDensity::~DrawableDensity(void)
+{
+}
+
+void Magick::DrawableDensity::operator()(
+  MagickCore::DrawingWand *context_) const
+{
+  DrawSetDensity(context_,_density.c_str());
+}
+
+Magick::DrawableBase* Magick::DrawableDensity::copy() const
+{
+  return(new DrawableDensity(*this));
+}
+
 // Ellipse
 Magick::DrawableEllipse::~DrawableEllipse( void )
 {
@@ -1853,8 +1874,9 @@ void Magick::PathSmoothCurvetoAbs::operator()
       double x2 = p->x();
       double y2 = p->y();
       p++;
-      if(p != _coordinates.end() )
-        DrawPathCurveToSmoothAbsolute( context_, x2, y2, p->x(), p->y() );
+      if (p == _coordinates.end() )
+        break;
+      DrawPathCurveToSmoothAbsolute( context_, x2, y2, p->x(), p->y() );
     }
 }
 Magick::VPathBase* Magick::PathSmoothCurvetoAbs::copy() const
@@ -1889,8 +1911,9 @@ void Magick::PathSmoothCurvetoRel::operator()
       double x2 = p->x();
       double y2 = p->y();
       p++;
-      if(p != _coordinates.end() )
-        DrawPathCurveToSmoothRelative( context_, x2, y2, p->x(), p->y() );
+      if (p == _coordinates.end() )
+        break;
+      DrawPathCurveToSmoothRelative( context_, x2, y2, p->x(), p->y() );
     }
 }
 Magick::VPathBase* Magick::PathSmoothCurvetoRel::copy() const
