@@ -1,6 +1,7 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
 // Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002
+// Copyright Dirk Lemstra 2013-2015
 //
 // Inclusion of ImageMagick headers (with namespace magic)
 
@@ -112,10 +113,16 @@ namespace MagickCore
 #          pragma comment(lib, "CORE_DB_bzlib_.lib")
 #        endif
 #        pragma comment(lib, "CORE_DB_coders_.lib")
+#        if defined(MAGICKCORE_OPENEXR_DELEGATE)
+#          pragma comment(lib, "CORE_DB_exr_.lib")
+#        endif
 #        if defined(MAGICKCORE_LQR_DELEGATE)
 #          pragma comment(lib, "CORE_DB_ffi_.lib")
 #        endif
 #        pragma comment(lib, "CORE_DB_filters_.lib")
+#        if defined(MAGICKCORE_FLIF_DELEGATE)
+#          pragma comment(lib, "CORE_DB_flif_.lib")
+#        endif
 #        if defined(MAGICKCORE_LQR_DELEGATE)
 #          pragma comment(lib, "CORE_DB_glib_.lib")
 #          pragma comment(lib, "winmm.lib")
@@ -174,10 +181,16 @@ namespace MagickCore
 #          pragma comment(lib, "CORE_RL_bzlib_.lib")
 #        endif
 #        pragma comment(lib, "CORE_RL_coders_.lib")
+#        if defined(MAGICKCORE_OPENEXR_DELEGATE)
+#          pragma comment(lib, "CORE_RL_exr_.lib")
+#        endif
 #        if defined(MAGICKCORE_LQR_DELEGATE)
 #          pragma comment(lib, "CORE_RL_ffi_.lib")
 #        endif
 #        pragma comment(lib, "CORE_RL_filters_.lib")
+#        if defined(MAGICKCORE_FLIF_DELEGATE)
+#          pragma comment(lib, "CORE_RL_flif_.lib")
+#        endif
 #        if defined(MAGICKCORE_LQR_DELEGATE)
 #          pragma comment(lib, "CORE_RL_glib_.lib")
 #          pragma comment(lib, "winmm.lib")
@@ -356,7 +369,10 @@ namespace Magick
 
   // Command options
   using MagickCore::CommandOption;
+  using MagickCore::MagickDirectionOptions;
+  using MagickCore::MagickGravityOptions;
   using MagickCore::MagickKernelOptions;
+  using MagickCore::MagickStyleOptions;
 
   // Composition operations
   using MagickCore::CompositeOperator;
@@ -751,6 +767,14 @@ namespace Magick
   using MagickCore::AccelerateEvent;
   using MagickCore::AllEvents;
 
+  // Magick functions
+  using MagickCore::MagickFunction;
+  using MagickCore::UndefinedFunction;
+  using MagickCore::ArcsinFunction;
+  using MagickCore::ArctanFunction;
+  using MagickCore::PolynomialFunction;
+  using MagickCore::SinusoidFunction;
+
   // Metric types
   using MagickCore::MetricType;
   using MagickCore::UndefinedMetric;
@@ -924,6 +948,7 @@ namespace Magick
   using MagickCore::ShepardsColorInterpolate;
   using MagickCore::VoronoiColorInterpolate;
   using MagickCore::InverseColorInterpolate;
+  using MagickCore::ManhattanColorInterpolate;
 
   // Statistic type
   using MagickCore::MedianStatistic;
@@ -988,6 +1013,7 @@ namespace Magick
   //
   using MagickCore::AcquireExceptionInfo;
   using MagickCore::AcquireCacheView;
+  using MagickCore::AcquireDrawingWand;
   using MagickCore::AcquireImage;
   using MagickCore::AcquireKernelInfo;
   using MagickCore::AcquireMagickMemory;
@@ -1045,6 +1071,7 @@ namespace Magick
   using MagickCore::CoderError;
   using MagickCore::CoderFatalError;
   using MagickCore::CoderWarning;
+  using MagickCore::CopyImagePixels;
   using MagickCore::ColorDecisionListImage;
   using MagickCore::ColorizeImage;
   using MagickCore::ColorMatrixImage;
@@ -1056,6 +1083,7 @@ namespace Magick
   using MagickCore::ConfigureError;
   using MagickCore::ConfigureFatalError;
   using MagickCore::ConfigureWarning;
+  using MagickCore::ConnectedComponentsImage;
   using MagickCore::ConstituteImage;
   using MagickCore::ContrastImage;
   using MagickCore::ContrastStretchImageChannel;
@@ -1094,7 +1122,6 @@ namespace Magick
   using MagickCore::DisplayImages;
   using MagickCore::DistortImage;
   using MagickCore::DrawAffine;
-  using MagickCore::DrawAllocateWand;
   using MagickCore::DrawAnnotation;
   using MagickCore::DrawArc;
   using MagickCore::DrawBezier;
@@ -1215,6 +1242,7 @@ namespace Magick
   using MagickCore::GetCacheViewAuthenticIndexQueue;
   using MagickCore::GetCacheViewAuthenticPixels;
   using MagickCore::GetCacheViewVirtualPixels;
+  using MagickCore::GetClientName;
   using MagickCore::GetColorTuple;
   using MagickCore::GetDrawInfo;
   using MagickCore::GetGeometry;
@@ -1275,16 +1303,20 @@ namespace Magick
   using MagickCore::IsEventLogging;
   using MagickCore::IsGeometry;
   using MagickCore::IsImagesEqual;
+  using MagickCore::IsOpaqueImage;
   using MagickCore::KernelInfo;
   using MagickCore::KuwaharaImage;
   using MagickCore::LessValue;
   using MagickCore::LevelImage;
   using MagickCore::LevelImageChannel;
   using MagickCore::LevelColorsImageChannel;
+  using MagickCore::LevelizeImage;
+  using MagickCore::LevelizeImageChannel;
   using MagickCore::LinearStretchImage;
   using MagickCore::LinkedListInfo;
   using MagickCore::LiquidRescaleImage;
   using MagickCore::LocaleCompare;
+  using MagickCore::LocalContrastImage;
   using MagickCore::LockSemaphoreInfo;
   using MagickCore::LogMagickEvent;
   using MagickCore::MagickCoreTerminus;
@@ -1317,6 +1349,7 @@ namespace Magick
   using MagickCore::NoiseType;
   using MagickCore::NormalizeImage;
   using MagickCore::NoValue;
+  using MagickCore::OffsetInfo;
   using MagickCore::OilPaintImage;
   using MagickCore::OpaquePaintImage;
   using MagickCore::OrderedPosterizeImage;
@@ -1381,7 +1414,6 @@ namespace Magick
   using MagickCore::SelectiveBlurImageChannel;
   using MagickCore::SeparateImageChannel;
   using MagickCore::SepiaToneImage;
-  using MagickCore::SetClientName;
   using MagickCore::SetGeometry;
   using MagickCore::SetImageAlphaChannel;
   using MagickCore::SetImageArtifact;
@@ -1455,6 +1487,7 @@ namespace Magick
   using MagickCore::VignetteImage;
   using MagickCore::CacheView;
   using MagickCore::WaveImage;
+  using MagickCore::WaveletDenoiseImage;
   using MagickCore::WhiteThresholdImage;
   using MagickCore::WhiteThresholdImageChannel;
   using MagickCore::WidthValue;
@@ -1480,8 +1513,8 @@ namespace Magick
   MagickCore::ExceptionInfo \
     *exceptionInfo; \
   exceptionInfo=MagickCore::AcquireExceptionInfo()
-#define ThrowPPException \
-  throwException(exceptionInfo); \
+#define ThrowPPException(quiet) \
+  throwException(exceptionInfo,quiet); \
   (void) MagickCore::DestroyExceptionInfo(exceptionInfo)
 
 #endif // Magick_Include_header
