@@ -1,6 +1,7 @@
 // This may look like C code, but it is really -*- C++ -*-
 //
 // Copyright Bob Friesenhahn, 1999, 2000, 2001, 2002, 2003
+// Copyright Dirk Lemstra 2014-2015
 //
 // Implementation of Options
 //
@@ -28,7 +29,8 @@ Magick::Options::Options(void)
       sizeof(ImageInfo)))),
     _quantizeInfo(static_cast<QuantizeInfo*>(AcquireMagickMemory(
       sizeof(QuantizeInfo)))),
-    _drawInfo(static_cast<DrawInfo*>(AcquireMagickMemory(sizeof(DrawInfo))))
+    _drawInfo(static_cast<DrawInfo*>(AcquireMagickMemory(sizeof(DrawInfo)))),
+    _quiet(false)
 {
   // Initialize image info with defaults
   GetImageInfo(_imageInfo);
@@ -43,7 +45,8 @@ Magick::Options::Options(void)
 Magick::Options::Options(const Magick::Options& options_)
   : _imageInfo(CloneImageInfo(options_._imageInfo)),
     _quantizeInfo(CloneQuantizeInfo(options_._quantizeInfo)),
-    _drawInfo(CloneDrawInfo(_imageInfo,options_._drawInfo))
+    _drawInfo(CloneDrawInfo(_imageInfo,options_._drawInfo)),
+    _quiet(false)
 {
 }
 
@@ -239,6 +242,7 @@ void Magick::Options::fillColor(const Magick::Color &fillColor_)
   _drawInfo->fill=fillColor_;
   if (fillColor_ == Magick::Color())
     fillPattern((const MagickCore::Image*) NULL);
+  setOption("fill",fillColor_);
 }
 
 Magick::Color Magick::Options::fillColor(void) const
@@ -259,7 +263,7 @@ void Magick::Options::fillPattern(const MagickCore::Image *fillPattern_)
       _drawInfo->fill_pattern=CloneImage(const_cast<MagickCore::Image*>(
         fillPattern_),0,0,static_cast<MagickBooleanType>(MagickTrue),
         exceptionInfo);
-      ThrowPPException;
+      ThrowPPException(_quiet);
     }
 }
 
@@ -300,6 +304,28 @@ std::string Magick::Options::font(void) const
   return(std::string());
 }
 
+void Magick::Options::fontFamily(const std::string &family_)
+{
+  if (family_.length() == 0)
+    {
+      _drawInfo->family=(char *) RelinquishMagickMemory(_drawInfo->font);
+      DestroyString(RemoveImageOption(imageInfo(),"family"));
+    }
+  else
+    {
+      Magick::CloneString(&_drawInfo->family,family_);
+      (void) SetImageOption(imageInfo(),"family",family_.c_str());
+    }
+}
+
+std::string Magick::Options::fontFamily(void) const
+{
+  if (_drawInfo->family)
+    return(std::string(_drawInfo->family));
+  
+  return(std::string());
+}
+
 void Magick::Options::fontPointsize(double pointSize_)
 {
   _imageInfo->pointsize=pointSize_;
@@ -311,6 +337,30 @@ double Magick::Options::fontPointsize(void) const
   return(_imageInfo->pointsize);
 }
 
+void Magick::Options::fontStyle(const StyleType style_)
+{
+  _drawInfo->style=style_;
+  (void) SetImageOption(_imageInfo,"style",CommandOptionToMnemonic(
+    MagickStyleOptions,(ssize_t) style_));
+}
+
+Magick::StyleType Magick::Options::fontStyle(void) const
+{
+  return(_drawInfo->style);
+}
+
+void Magick::Options::fontWeight(const size_t weight_)
+{
+  _drawInfo->weight=weight_;
+  setOption("weight",(double) weight_);
+}
+
+size_t Magick::Options::fontWeight(void) const
+{
+  return(_drawInfo->weight);
+}
+
+
 std::string Magick::Options::format(void) const
 {
   const MagickInfo
@@ -319,7 +369,7 @@ std::string Magick::Options::format(void) const
   GetPPException;
   if (*_imageInfo->magick != '\0')
     magick_info=GetMagickInfo(_imageInfo->magick,exceptionInfo);
-  ThrowPPException;
+  ThrowPPException(_quiet);
   
   if ((magick_info != 0) && (*magick_info->description != '\0'))
     return(std::string(magick_info->description));
@@ -339,19 +389,25 @@ Magick::InterlaceType Magick::Options::interlaceType(void) const
 
 void Magick::Options::magick(const std::string &magick_)
 {
+  if (magick_.empty())
+  {
+    _imageInfo->magick[0] = '\0';
+    return;
+  }
+
   FormatLocaleString(_imageInfo->filename,MaxTextExtent,"%.1024s:",
     magick_.c_str());
   GetPPException;
   SetImageInfo(_imageInfo,1,exceptionInfo);
-  ThrowPPException;
-  if (*_imageInfo->magick == '\0')
+  ThrowPPException(_quiet);
+  if ( _imageInfo->magick[0] == '\0' )
     throwExceptionExplicit(OptionWarning,"Unrecognized image format",
       magick_.c_str());
 }
 
 std::string Magick::Options::magick(void) const
 {
-  if (*_imageInfo->magick)
+  if ( _imageInfo->magick[0] != '\0' )
     return(std::string(_imageInfo->magick));
  
   return(std::string());
@@ -454,6 +510,16 @@ size_t Magick::Options::quantizeTreeDepth(void) const
   return(_quantizeInfo->tree_depth);
 }
 
+void Magick::Options::quiet(const bool quiet_)
+{
+  _quiet=quiet_;
+}
+
+bool Magick::Options::quiet(void) const
+{
+   return(_quiet);
+}
+
 void Magick::Options::resolutionUnits(Magick::ResolutionType resolutionUnits_)
 {
   _imageInfo->units=resolutionUnits_;
@@ -461,7 +527,7 @@ void Magick::Options::resolutionUnits(Magick::ResolutionType resolutionUnits_)
 
 Magick::ResolutionType Magick::Options::resolutionUnits(void) const
 {
-  return(static_cast<Magick::ResolutionType>(_imageInfo->units));
+  return(_imageInfo->units);
 }
 
 void Magick::Options::samplingFactor(const std::string &samplingFactor_)
@@ -511,6 +577,9 @@ bool Magick::Options::strokeAntiAlias(void) const
 void Magick::Options::strokeColor(const Magick::Color &strokeColor_)
 {
   _drawInfo->stroke=strokeColor_;
+  if (strokeColor_ == Magick::Color())
+    strokePattern((const MagickCore::Image*) NULL);
+  setOption("stroke",strokeColor_);
 }
 
 Magick::Color Magick::Options::strokeColor(void) const
@@ -535,6 +604,7 @@ void Magick::Options::strokeDashArray(const double *strokeDashArray_)
         sizeof(double)));
       // Copy elements
       memcpy(_drawInfo->dash_pattern,strokeDashArray_,(x+1)*sizeof(double));
+      _drawInfo->dash_pattern[x]=0.0;
     }
 }
 
@@ -593,7 +663,7 @@ void Magick::Options::strokePattern(const MagickCore::Image *strokePattern_)
       GetPPException;
       _drawInfo->stroke_pattern=CloneImage(const_cast<MagickCore::Image*>(
         strokePattern_),0,0,MagickTrue,exceptionInfo);
-      ThrowPPException;
+      ThrowPPException(_quiet);
     }
 }
 
@@ -635,6 +705,8 @@ size_t Magick::Options::subRange(void) const
 void Magick::Options::textDirection(DirectionType direction_)
 {
   _drawInfo->direction=direction_;
+  (void) SetImageOption(_imageInfo,"direction",CommandOptionToMnemonic(
+    MagickDirectionOptions,(ssize_t) direction_));
 }
 
 Magick::DirectionType Magick::Options::textDirection() const
@@ -645,6 +717,7 @@ Magick::DirectionType Magick::Options::textDirection() const
 void Magick::Options::textEncoding(const std::string &encoding_)
 {
   CloneString(&_drawInfo->encoding,encoding_.c_str());
+  (void) SetImageOption(imageInfo(),"encoding",encoding_.c_str());
 }
 
 std::string Magick::Options::textEncoding(void) const
@@ -658,6 +731,8 @@ std::string Magick::Options::textEncoding(void) const
 void Magick::Options::textGravity(GravityType gravity_)
 {
   _drawInfo->gravity=gravity_;
+  (void) SetImageOption(_imageInfo,"gravity",CommandOptionToMnemonic(
+    MagickGravityOptions,(ssize_t) gravity_));
 }
 
 Magick::GravityType Magick::Options::textGravity() const
@@ -668,6 +743,7 @@ Magick::GravityType Magick::Options::textGravity() const
 void Magick::Options::textInterlineSpacing(double spacing_)
 {
   _drawInfo->interline_spacing=spacing_;
+  setOption("interline-spacing",spacing_);
 }
 
 double Magick::Options::textInterlineSpacing(void) const
@@ -678,6 +754,7 @@ double Magick::Options::textInterlineSpacing(void) const
 void Magick::Options::textInterwordSpacing(double spacing_)
 {
   _drawInfo->interword_spacing=spacing_;
+  setOption("interword-spacing",spacing_);
 }
 
 double Magick::Options::textInterwordSpacing(void) const
@@ -688,6 +765,7 @@ double Magick::Options::textInterwordSpacing(void) const
 void Magick::Options::textKerning(double kerning_)
 {
   _drawInfo->kerning=kerning_;
+  setOption("kerning",kerning_);
 }
 
 double Magick::Options::textKerning(void) const
@@ -695,6 +773,18 @@ double Magick::Options::textKerning(void) const
   return(_drawInfo->kerning);
 }
 
+void Magick::Options::textUnderColor(const Magick::Color &undercolor_)
+{
+  _drawInfo->undercolor=undercolor_;
+  setOption("undercolor",undercolor_);
+}
+
+Magick::Color Magick::Options::textUnderColor(void) const
+{
+  return(_drawInfo->undercolor);
+}
+
+
 void Magick::Options::tileName(const std::string &tileName_)
 {
   if (tileName_.length() == 0)
@@ -927,11 +1017,30 @@ MagickCore::QuantizeInfo *Magick::Options::quantizeInfo( void )
 Magick::Options::Options(const MagickCore::ImageInfo *imageInfo_,
   const MagickCore::QuantizeInfo *quantizeInfo_,
   const MagickCore::DrawInfo *drawInfo_)
-: _imageInfo(0),
-  _quantizeInfo(0),
-  _drawInfo(0)
+: _imageInfo((MagickCore::ImageInfo* ) NULL),
+  _quantizeInfo((MagickCore::QuantizeInfo* ) NULL),
+  _drawInfo((MagickCore::DrawInfo* ) NULL),
+  _quiet(false)
 {
   _imageInfo=CloneImageInfo(imageInfo_);
   _quantizeInfo=CloneQuantizeInfo(quantizeInfo_);
   _drawInfo=CloneDrawInfo(imageInfo_,drawInfo_);
 }
+
+void Magick::Options::setOption(const char *name,const Color &value_)
+{
+  std::string
+    option;
+
+  option=value_;
+  (void) SetImageOption(imageInfo(),name,option.c_str());
+}
+
+void Magick::Options::setOption(const char *name,const double value_)
+{
+  char
+    option[MaxTextExtent];
+
+  (void) FormatLocaleString(option,MaxTextExtent,"%.20g",value_);
+  (void) SetImageOption(_imageInfo,name,option);
+}
