@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -482,7 +482,7 @@ static MagickBooleanType WriteCALSImage(const ImageInfo *image_info,
   count=WriteCALSRecord(image,"txtfilid: NONE");
   count=WriteCALSRecord(image,"figid: NONE");
   count=WriteCALSRecord(image,"srcgph: NONE");
-  count=WriteCALSRecord(image,"docls: NONE");
+  count=WriteCALSRecord(image,"doccls: NONE");
   count=WriteCALSRecord(image,"rtype: 1");
   orient_x=0;
   orient_y=0;
