@@ -17,7 +17,7 @@
 %                                March 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -66,6 +66,11 @@
 #include "magick/string_.h"
 #include "magick/string-private.h"
 
+/*
+  Define declaration.
+*/
+#define MaxNumberImageElements  8
+
 /*
   Typedef declaration.
 */
@@ -230,7 +235,7 @@ typedef struct _DPXImageInfo
     lines_per_element;
 
   DPXImageElement
-    image_element[8];
+    image_element[MaxNumberImageElements];
 
   unsigned char
     reserve[52];
@@ -436,8 +441,14 @@ static size_t GetBytesPerRow(const size_t columns,
     }
     case 16:
     {
-      bytes_per_row=2*(((size_t) samples_per_pixel*columns*bits_per_pixel+8)/
-        16);
+      if (pad == MagickFalse)
+        {
+          bytes_per_row=2*(((size_t) samples_per_pixel*columns*bits_per_pixel+
+            15)/16);
+          break;
+        }
+      bytes_per_row=4*(((size_t) samples_per_pixel*columns*bits_per_pixel+31)/
+        32);
       break;
     }
     case 32:
@@ -741,10 +752,13 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     dpx.file.creator);
   if (*dpx.file.creator == '\0')
     {
-      (void) FormatImageProperty(image,"dpx:file.creator","%.100s",
-        GetMagickVersion((size_t *) NULL));
-      (void) FormatImageProperty(image,"software","%.100s",
-        GetMagickVersion((size_t *) NULL));
+      char
+        *url;
+
+      url=GetMagickHomeURL();
+      (void) FormatImageProperty(image,"dpx:file.creator","%.100s",url);
+      (void) FormatImageProperty(image,"software","%.100s",url);
+      url=DestroyString(url);
     }
   else
     {
@@ -780,6 +794,8 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     Read DPX image header.
   */
   dpx.image.orientation=ReadBlobShort(image);
+  if (dpx.image.orientation > 7)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   offset+=2;
   if (dpx.image.orientation != (unsigned short) ~0)
     (void) FormatImageProperty(image,"dpx:image.orientation","%d",
@@ -797,6 +813,8 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     case 7: image->orientation=RightBottomOrientation; break;
   }
   dpx.image.number_elements=ReadBlobShort(image);
+  if (dpx.image.number_elements > MaxNumberImageElements)
+    ThrowReaderException(CorruptImageError,"ImproperImageHeader");
   offset+=2;
   dpx.image.pixels_per_line=ReadBlobLong(image);
   offset+=4;
@@ -1117,6 +1135,12 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
+  status=SetImageExtent(image,image->columns,image->rows);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
   for (n=0; n < (ssize_t) dpx.image.number_elements; n++)
   {
     /*
@@ -1214,6 +1238,9 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       MagickTrue : MagickFalse);
     for (y=0; y < (ssize_t) image->rows; y++)
     {
+      const unsigned char
+        *pixels;
+
       MagickBooleanType
         sync;
 
@@ -1227,29 +1254,24 @@ static Image *ReadDPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
         count,
         offset;
 
-      unsigned char
-        *pixels;
-
       if (status == MagickFalse)
         continue;
-      pixels=GetQuantumPixels(quantum_info);
-      {
-        count=ReadBlob(image,extent,pixels);
-        if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
-            (image->previous == (Image *) NULL))
-          {
-            MagickBooleanType
-              proceed;
-
-            proceed=SetImageProgress(image,LoadImageTag,(MagickOffsetType) row,
-              image->rows);
-            if (proceed == MagickFalse)
-              status=MagickFalse;
-          }
-        offset=row++;
-      }
+      pixels=(const unsigned char *) ReadBlobStream(image,extent,
+        GetQuantumPixels(quantum_info),&count);
       if (count != (ssize_t) extent)
         status=MagickFalse;
+      if ((image->progress_monitor != (MagickProgressMonitor) NULL) &&
+          (image->previous == (Image *) NULL))
+        {
+          MagickBooleanType
+            proceed;
+
+          proceed=SetImageProgress(image,LoadImageTag,(MagickOffsetType) row,
+            image->rows);
+          if (proceed == MagickFalse)
+            status=MagickFalse;
+        }
+      offset=row++;
       q=QueueAuthenticPixels(image,0,offset,image->columns,1,exception);
       if (q == (PixelPacket *) NULL)
         {
@@ -1400,11 +1422,27 @@ static unsigned int StringToTimeCode(const char *key)
   return(value);
 }
 
-static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
+static inline const char *GetDPXProperty(const Image *image,
+  const char *property)
 {
   const char
     *value;
 
+  value=GetImageArtifact(image,property);
+  if (value != (const char *) NULL)
+    return(value);
+  return(GetImageProperty(image,property));
+}
+
+static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,
+  Image *image)
+{
+  char
+    *url;
+
+  const char
+    *value;
+
   const StringInfo
     *profile;
 
@@ -1521,7 +1559,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
       dpx.file.user_size=(((dpx.file.user_size+0x2000-1)/0x2000)*0x2000);
     }
   offset+=WriteBlobLong(image,dpx.file.user_size);
-  value=GetImageArtifact(image,"dpx:file.filename");
+  value=GetDPXProperty(image,"dpx:file.filename");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.file.filename,value,sizeof(dpx.file.filename)-1);
   offset+=WriteBlob(image,sizeof(dpx.file.filename),(unsigned char *)
@@ -1531,19 +1569,20 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     dpx.file.timestamp);
   offset+=WriteBlob(image,sizeof(dpx.file.timestamp),(unsigned char *)
     dpx.file.timestamp);
-  (void) strncpy(dpx.file.creator,GetMagickVersion((size_t *) NULL),
-    sizeof(dpx.file.creator)-1);
-  value=GetImageArtifact(image,"dpx:file.creator");
+  url=GetMagickHomeURL();
+  (void) strncpy(dpx.file.creator,url,sizeof(dpx.file.creator)-1);
+  url=DestroyString(url);
+  value=GetDPXProperty(image,"dpx:file.creator");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.file.creator,value,sizeof(dpx.file.creator)-1);
   offset+=WriteBlob(image,sizeof(dpx.file.creator),(unsigned char *)
     dpx.file.creator);
-  value=GetImageArtifact(image,"dpx:file.project");
+  value=GetDPXProperty(image,"dpx:file.project");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.file.project,value,sizeof(dpx.file.project)-1);
   offset+=WriteBlob(image,sizeof(dpx.file.project),(unsigned char *)
     dpx.file.project);
-  value=GetImageArtifact(image,"dpx:file.copyright");
+  value=GetDPXProperty(image,"dpx:file.copyright");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.file.copyright,value,sizeof(dpx.file.copyright)-1);
   offset+=WriteBlob(image,sizeof(dpx.file.copyright),(unsigned char *)
@@ -1607,7 +1646,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
             dpx.image.image_element[i].descriptor=RGBAComponentType;
           if ((image_info->type != TrueColorType) &&
               (image->matte == MagickFalse) &&
-              (IsGrayImage(image,&image->exception) != MagickFalse))
+              (SetImageGray(image,&image->exception) != MagickFalse))
             dpx.image.image_element[i].descriptor=LumaComponentType;
           break;
         }
@@ -1661,36 +1700,36 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
       (void) DeleteImageProperty(image,"dpx:orientation.y_size");
     }
   dpx.orientation.x_offset=0U;
-  value=GetImageArtifact(image,"dpx:orientation.x_offset");
+  value=GetDPXProperty(image,"dpx:orientation.x_offset");
   if (value != (const char *) NULL)
     dpx.orientation.x_offset=(unsigned int) StringToUnsignedLong(value);
   offset+=WriteBlobLong(image,dpx.orientation.x_offset);
   dpx.orientation.y_offset=0U;
-  value=GetImageArtifact(image,"dpx:orientation.y_offset");
+  value=GetDPXProperty(image,"dpx:orientation.y_offset");
   if (value != (const char *) NULL)
     dpx.orientation.y_offset=(unsigned int) StringToUnsignedLong(value);
   offset+=WriteBlobLong(image,dpx.orientation.y_offset);
   dpx.orientation.x_center=0.0f;
-  value=GetImageArtifact(image,"dpx:orientation.x_center");
+  value=GetDPXProperty(image,"dpx:orientation.x_center");
   if (value != (const char *) NULL)
     dpx.orientation.x_center=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.orientation.x_center);
   dpx.orientation.y_center=0.0f;
-  value=GetImageArtifact(image,"dpx:orientation.y_center");
+  value=GetDPXProperty(image,"dpx:orientation.y_center");
   if (value != (const char *) NULL)
     dpx.orientation.y_center=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.orientation.y_center);
   dpx.orientation.x_size=0U;
-  value=GetImageArtifact(image,"dpx:orientation.x_size");
+  value=GetDPXProperty(image,"dpx:orientation.x_size");
   if (value != (const char *) NULL)
     dpx.orientation.x_size=(unsigned int) StringToUnsignedLong(value);
   offset+=WriteBlobLong(image,dpx.orientation.x_size);
   dpx.orientation.y_size=0U;
-  value=GetImageArtifact(image,"dpx:orientation.y_size");
+  value=GetDPXProperty(image,"dpx:orientation.y_size");
   if (value != (const char *) NULL)
     dpx.orientation.y_size=(unsigned int) StringToUnsignedLong(value);
   offset+=WriteBlobLong(image,dpx.orientation.y_size);
-  value=GetImageArtifact(image,"dpx:orientation.filename");
+  value=GetDPXProperty(image,"dpx:orientation.filename");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.orientation.filename,value,
       sizeof(dpx.orientation.filename)-1);
@@ -1698,13 +1737,13 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     dpx.orientation.filename);
   offset+=WriteBlob(image,sizeof(dpx.orientation.timestamp),(unsigned char *)
     dpx.orientation.timestamp);
-  value=GetImageArtifact(image,"dpx:orientation.device");
+  value=GetDPXProperty(image,"dpx:orientation.device");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.orientation.device,value,
       sizeof(dpx.orientation.device)-1);
   offset+=WriteBlob(image,sizeof(dpx.orientation.device),(unsigned char *)
     dpx.orientation.device);
-  value=GetImageArtifact(image,"dpx:orientation.serial");
+  value=GetDPXProperty(image,"dpx:orientation.serial");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.orientation.serial,value,
       sizeof(dpx.orientation.serial)-1);
@@ -1712,7 +1751,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     dpx.orientation.serial);
   for (i=0; i < 4; i++)
     dpx.orientation.border[i]=0;
-  value=GetImageArtifact(image,"dpx:orientation.border");
+  value=GetDPXProperty(image,"dpx:orientation.border");
   if (value != (const char *) NULL)
     {
       flags=ParseGeometry(value,&geometry_info);
@@ -1727,7 +1766,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     offset+=WriteBlobShort(image,dpx.orientation.border[i]);
   for (i=0; i < 2; i++)
     dpx.orientation.aspect_ratio[i]=0U;
-  value=GetImageArtifact(image,"dpx:orientation.aspect_ratio");
+  value=GetDPXProperty(image,"dpx:orientation.aspect_ratio");
   if (value != (const char *) NULL)
     {
       flags=ParseGeometry(value,&geometry_info);
@@ -1744,72 +1783,72 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
     Write film header.
   */
   (void) ResetMagickMemory(dpx.film.id,0,sizeof(dpx.film.id));
-  value=GetImageArtifact(image,"dpx:film.id");
+  value=GetDPXProperty(image,"dpx:film.id");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.film.id,value,sizeof(dpx.film.id)-1);
   offset+=WriteBlob(image,sizeof(dpx.film.id),(unsigned char *) dpx.film.id);
   (void) ResetMagickMemory(dpx.film.type,0,sizeof(dpx.film.type));
-  value=GetImageArtifact(image,"dpx:film.type");
+  value=GetDPXProperty(image,"dpx:film.type");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.film.type,value,sizeof(dpx.film.type)-1);
   offset+=WriteBlob(image,sizeof(dpx.film.type),(unsigned char *)
     dpx.film.type);
   (void) ResetMagickMemory(dpx.film.offset,0,sizeof(dpx.film.offset));
-  value=GetImageArtifact(image,"dpx:film.offset");
+  value=GetDPXProperty(image,"dpx:film.offset");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.film.offset,value,sizeof(dpx.film.offset)-1);
   offset+=WriteBlob(image,sizeof(dpx.film.offset),(unsigned char *)
     dpx.film.offset);
   (void) ResetMagickMemory(dpx.film.prefix,0,sizeof(dpx.film.prefix));
-  value=GetImageArtifact(image,"dpx:film.prefix");
+  value=GetDPXProperty(image,"dpx:film.prefix");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.film.prefix,value,sizeof(dpx.film.prefix)-1);
   offset+=WriteBlob(image,sizeof(dpx.film.prefix),(unsigned char *)
     dpx.film.prefix);
   (void) ResetMagickMemory(dpx.film.count,0,sizeof(dpx.film.count));
-  value=GetImageArtifact(image,"dpx:film.count");
+  value=GetDPXProperty(image,"dpx:film.count");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.film.count,value,sizeof(dpx.film.count)-1);
   offset+=WriteBlob(image,sizeof(dpx.film.count),(unsigned char *)
     dpx.film.count);
   (void) ResetMagickMemory(dpx.film.format,0,sizeof(dpx.film.format));
-  value=GetImageArtifact(image,"dpx:film.format");
+  value=GetDPXProperty(image,"dpx:film.format");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.film.format,value,sizeof(dpx.film.format)-1);
   offset+=WriteBlob(image,sizeof(dpx.film.format),(unsigned char *)
     dpx.film.format);
   dpx.film.frame_position=0U;
-  value=GetImageArtifact(image,"dpx:film.frame_position");
+  value=GetDPXProperty(image,"dpx:film.frame_position");
   if (value != (const char *) NULL)
     dpx.film.frame_position=(unsigned int) StringToUnsignedLong(value);
   offset+=WriteBlobLong(image,dpx.film.frame_position);
   dpx.film.sequence_extent=0U;
-  value=GetImageArtifact(image,"dpx:film.sequence_extent");
+  value=GetDPXProperty(image,"dpx:film.sequence_extent");
   if (value != (const char *) NULL)
     dpx.film.sequence_extent=(unsigned int) StringToUnsignedLong(value);
   offset+=WriteBlobLong(image,dpx.film.sequence_extent);
   dpx.film.held_count=0U;
-  value=GetImageArtifact(image,"dpx:film.held_count");
+  value=GetDPXProperty(image,"dpx:film.held_count");
   if (value != (const char *) NULL)
     dpx.film.held_count=(unsigned int) StringToUnsignedLong(value);
   offset+=WriteBlobLong(image,dpx.film.held_count);
   dpx.film.frame_rate=0.0f;
-  value=GetImageArtifact(image,"dpx:film.frame_rate");
+  value=GetDPXProperty(image,"dpx:film.frame_rate");
   if (value != (const char *) NULL)
     dpx.film.frame_rate=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.film.frame_rate);
   dpx.film.shutter_angle=0.0f;
-  value=GetImageArtifact(image,"dpx:film.shutter_angle");
+  value=GetDPXProperty(image,"dpx:film.shutter_angle");
   if (value != (const char *) NULL)
     dpx.film.shutter_angle=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.film.shutter_angle);
   (void) ResetMagickMemory(dpx.film.frame_id,0,sizeof(dpx.film.frame_id));
-  value=GetImageArtifact(image,"dpx:film.frame_id");
+  value=GetDPXProperty(image,"dpx:film.frame_id");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.film.frame_id,value,sizeof(dpx.film.frame_id)-1);
   offset+=WriteBlob(image,sizeof(dpx.film.frame_id),(unsigned char *)
     dpx.film.frame_id);
-  value=GetImageArtifact(image,"dpx:film.slate");
+  value=GetDPXProperty(image,"dpx:film.slate");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.film.slate,value,sizeof(dpx.film.slate)-1);
   offset+=WriteBlob(image,sizeof(dpx.film.slate),(unsigned char *)
@@ -1819,82 +1858,81 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   /*
     Write television header.
   */
-  value=GetImageArtifact(image,"dpx:television.time.code");
+  value=GetDPXProperty(image,"dpx:television.time.code");
   if (value != (const char *) NULL)
     dpx.television.time_code=StringToTimeCode(value);
   offset+=WriteBlobLong(image,dpx.television.time_code);
-  value=GetImageArtifact(image,"dpx:television.user.bits");
+  value=GetDPXProperty(image,"dpx:television.user.bits");
   if (value != (const char *) NULL)
     dpx.television.user_bits=StringToTimeCode(value);
   offset+=WriteBlobLong(image,dpx.television.user_bits);
-  value=GetImageArtifact(image,"dpx:television.interlace");
+  value=GetDPXProperty(image,"dpx:television.interlace");
   if (value != (const char *) NULL)
     dpx.television.interlace=(unsigned char) StringToLong(value);
   offset+=WriteBlobByte(image,dpx.television.interlace);
-  value=GetImageArtifact(image,"dpx:television.field_number");
+  value=GetDPXProperty(image,"dpx:television.field_number");
   if (value != (const char *) NULL)
     dpx.television.field_number=(unsigned char) StringToLong(value);
   offset+=WriteBlobByte(image,dpx.television.field_number);
   dpx.television.video_signal=0;
-  value=GetImageArtifact(image,"dpx:television.video_signal");
+  value=GetDPXProperty(image,"dpx:television.video_signal");
   if (value != (const char *) NULL)
     dpx.television.video_signal=(unsigned char) StringToLong(value);
   offset+=WriteBlobByte(image,dpx.television.video_signal);
   dpx.television.padding=0;
-  value=GetImageArtifact(image,"dpx:television.padding");
+  value=GetDPXProperty(image,"dpx:television.padding");
   if (value != (const char *) NULL)
     dpx.television.padding=(unsigned char) StringToLong(value);
   offset+=WriteBlobByte(image,dpx.television.padding);
   dpx.television.horizontal_sample_rate=0.0f;
-  value=GetImageArtifact(image,
-    "dpx:television.horizontal_sample_rate");
+  value=GetDPXProperty(image,"dpx:television.horizontal_sample_rate");
   if (value != (const char *) NULL)
     dpx.television.horizontal_sample_rate=StringToDouble(value,
       (char **) NULL);
   offset+=WriteBlobFloat(image,dpx.television.horizontal_sample_rate);
   dpx.television.vertical_sample_rate=0.0f;
-  value=GetImageArtifact(image,"dpx:television.vertical_sample_rate");
+  value=GetDPXProperty(image,"dpx:television.vertical_sample_rate");
   if (value != (const char *) NULL)
     dpx.television.vertical_sample_rate=StringToDouble(value,
       (char **) NULL);
   offset+=WriteBlobFloat(image,dpx.television.vertical_sample_rate);
   dpx.television.frame_rate=0.0f;
-  value=GetImageArtifact(image,"dpx:television.frame_rate");
+  value=GetDPXProperty(image,"dpx:television.frame_rate");
   if (value != (const char *) NULL)
     dpx.television.frame_rate=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.television.frame_rate);
   dpx.television.time_offset=0.0f;
-  value=GetImageArtifact(image,"dpx:television.time_offset");
+  value=GetDPXProperty(image,"dpx:television.time_offset");
   if (value != (const char *) NULL)
     dpx.television.time_offset=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.television.time_offset);
   dpx.television.gamma=0.0f;
-  value=GetImageArtifact(image,"dpx:television.gamma");
+  value=GetDPXProperty(image,"dpx:television.gamma");
   if (value != (const char *) NULL)
     dpx.television.gamma=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.television.gamma);
   dpx.television.black_level=0.0f;
-  value=GetImageArtifact(image,"dpx:television.black_level");
+  value=GetDPXProperty(image,"dpx:television.black_level");
   if (value != (const char *) NULL)
     dpx.television.black_level=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.television.black_level);
   dpx.television.black_gain=0.0f;
-  value=GetImageArtifact(image,"dpx:television.black_gain");
+  value=GetDPXProperty(image,"dpx:television.black_gain");
   if (value != (const char *) NULL)
     dpx.television.black_gain=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.television.black_gain);
   dpx.television.break_point=0.0f;
-  value=GetImageArtifact(image,"dpx:television.break_point");
+  value=GetDPXProperty(image,"dpx:television.break_point");
   if (value != (const char *) NULL)
     dpx.television.break_point=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.television.break_point);
   dpx.television.white_level=0.0f;
-  value=GetImageArtifact(image,"dpx:television.white_level");
+  value=GetDPXProperty(image,"dpx:television.white_level");
   if (value != (const char *) NULL)
     dpx.television.white_level=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.television.white_level);
   dpx.television.integration_times=0.0f;
-  value=GetImageArtifact(image,"dpx:television.integration_times");
+  value=GetDPXProperty(image,"dpx:television.integration_times");
   if (value != (const char *) NULL)
     dpx.television.integration_times=StringToDouble(value,(char **) NULL);
   offset+=WriteBlobFloat(image,dpx.television.integration_times);
@@ -1903,7 +1941,7 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
   /*
     Write user header.
   */
-  value=GetImageArtifact(image,"dpx:user.id");
+  value=GetDPXProperty(image,"dpx:user.id");
   if (value != (const char *) NULL)
     (void) strncpy(dpx.user.id,value,sizeof(dpx.user.id)-1);
   offset+=WriteBlob(image,sizeof(dpx.user.id),(unsigned char *) dpx.user.id);
@@ -1940,12 +1978,14 @@ static MagickBooleanType WriteDPXImage(const ImageInfo *image_info,Image *image)
         quantum_type=CbYCrYQuantum;
     }
   extent=GetBytesPerRow(image->columns,image->matte != MagickFalse ? 4UL : 3UL,
-    image->depth,MagickTrue);
+    image->depth,dpx.image.image_element[0].packing == 0 ? MagickFalse :
+    MagickTrue);
   if ((image_info->type != TrueColorType) && (image->matte == MagickFalse) &&
-      (IsGrayImage(image,&image->exception) != MagickFalse))
+      (SetImageGray(image,&image->exception) != MagickFalse))
     {
       quantum_type=GrayQuantum;
-      extent=GetBytesPerRow(image->columns,1UL,image->depth,MagickTrue);
+      extent=GetBytesPerRow(image->columns,1UL,image->depth,
+        dpx.image.image_element[0].packing == 0 ? MagickFalse : MagickTrue);
     }
   pixels=GetQuantumPixels(quantum_info);
   for (y=0; y < (ssize_t) image->rows; y++)
