@@ -17,7 +17,7 @@
 %                                 April 2007                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -50,6 +50,7 @@
 #include "magick/list.h"
 #include "magick/magick.h"
 #include "magick/memory_.h"
+#include "magick/option.h"
 #include "magick/pixel-accessor.h"
 #include "magick/property.h"
 #include "magick/quantum-private.h"
@@ -61,6 +62,18 @@
 #if defined(MAGICKCORE_OPENEXR_DELEGATE)
 #include <ImfCRgbaFile.h>
 
+/*
+  Typedef declaractions.
+*/
+typedef struct _ExrWindow
+{
+  int
+    max_x,
+    max_y,
+    min_x,
+    min_y;
+} ExrWindow;
+
 /*
   Forward declarations.
 */
@@ -131,6 +144,10 @@ static MagickBooleanType IsEXR(const unsigned char *magick,const size_t length)
 */
 static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+  ExrWindow
+    data_window,
+    display_window;
+
   const ImfHeader
     *hdr_info;
 
@@ -146,12 +163,6 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
   ImfRgba
     *scanline;
 
-  int
-    max_x,
-    max_y,
-    min_x,
-    min_y;
-
   MagickBooleanType
     status;
 
@@ -162,6 +173,7 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
     *q;
 
   ssize_t
+    columns,
     y;
 
   /*
@@ -192,15 +204,19 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
     {
       ThrowFileException(exception,BlobError,"UnableToOpenBlob",
         ImfErrorMessage());
+      if (LocaleCompare(image_info->filename,read_info->filename) != 0)
+        (void) RelinquishUniqueFileResource(read_info->filename);
       read_info=DestroyImageInfo(read_info);
       return((Image *) NULL);
     }
   hdr_info=ImfInputHeader(file);
-  ImfHeaderDisplayWindow(hdr_info,&min_x,&min_y,&max_x,&max_y);
-  image->columns=max_x-min_x+1UL;
-  image->rows=max_y-min_y+1UL;
+  ImfHeaderDisplayWindow(hdr_info,&display_window.min_x,&display_window.min_y,
+    &display_window.max_x,&display_window.max_y);
+  image->columns=display_window.max_x-display_window.min_x+1UL;
+  image->rows=display_window.max_y-display_window.min_y+1UL;
   image->matte=MagickTrue;
   SetImageColorspace(image,RGBColorspace);
+  image->gamma=1.0;
   if (image_info->ping != MagickFalse)
     {
       (void) ImfCloseInputFile(file);
@@ -210,31 +226,80 @@ static Image *ReadEXRImage(const ImageInfo *image_info,ExceptionInfo *exception)
       (void) CloseBlob(image);
       return(GetFirstImageInList(image));
     }
-  scanline=(ImfRgba *) AcquireQuantumMemory(image->columns,sizeof(*scanline));
-  if (scanline == (ImfRgba *) NULL)
+  status=SetImageExtent(image,image->columns,image->rows);
+  if (status == MagickFalse)
     {
-      (void) ImfCloseInputFile(file);
-      ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
+  ImfHeaderDataWindow(hdr_info,&data_window.min_x,&data_window.min_y,
+    &data_window.max_x,&data_window.max_y);
+  columns=(ssize_t) data_window.max_x-data_window.min_x+1UL;
+  if ((display_window.min_x > data_window.max_x) ||
+      (display_window.min_x+(int) image->columns <= data_window.min_x))
+    scanline=(ImfRgba *) NULL;
+  else
+    {
+      scanline=(ImfRgba *) AcquireQuantumMemory(columns,sizeof(*scanline));
+      if (scanline == (ImfRgba *) NULL)
+        {
+          (void) ImfCloseInputFile(file);
+          if (LocaleCompare(image_info->filename,read_info->filename) != 0)
+            (void) RelinquishUniqueFileResource(read_info->filename);
+          read_info=DestroyImageInfo(read_info);
+          ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
+        }
     }
   for (y=0; y < (ssize_t) image->rows; y++)
   {
+    int
+      yy;
+
     q=QueueAuthenticPixels(image,0,y,image->columns,1,exception);
     if (q == (PixelPacket *) NULL)
       break;
-    ResetMagickMemory(scanline,0,image->columns*sizeof(*scanline));
-    ImfInputSetFrameBuffer(file,scanline-min_x-image->columns*(min_y+y),1,
-      image->columns);
-    ImfInputReadPixels(file,min_y+y,min_y+y);
+    yy=display_window.min_y+y;
+    if ((yy < data_window.min_y) || (yy > data_window.max_y) ||
+        (scanline == (ImfRgba *) NULL))
+    {
+      for (x=0; x < (ssize_t) image->columns; x++)
+      {
+        SetPixelRed(q,image->background_color.red);
+        SetPixelGreen(q,image->background_color.green);
+        SetPixelBlue(q,image->background_color.blue);
+        SetPixelOpacity(q,image->background_color.opacity);
+        q++;
+      }
+      continue;
+    }
+    ResetMagickMemory(scanline,0,columns*sizeof(*scanline));
+    ImfInputSetFrameBuffer(file,scanline-data_window.min_x-columns*yy,1,
+      columns);
+    ImfInputReadPixels(file,yy,yy);
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      SetPixelRed(q,ClampToQuantum((MagickRealType) QuantumRange*
-        ImfHalfToFloat(scanline[x].r)));
-      SetPixelGreen(q,ClampToQuantum((MagickRealType) QuantumRange*
-        ImfHalfToFloat(scanline[x].g)));
-      SetPixelBlue(q,ClampToQuantum((MagickRealType) QuantumRange*
-        ImfHalfToFloat(scanline[x].b)));
-      SetPixelAlpha(q,ClampToQuantum((MagickRealType) QuantumRange*
-        ImfHalfToFloat(scanline[x].a)));
+      int
+        xx;
+
+      xx=display_window.min_x+((int) x-data_window.min_x);
+      if ((xx < 0) || (display_window.min_x+(int) x > data_window.max_x))
+        {
+          SetPixelRed(q,image->background_color.red);
+          SetPixelGreen(q,image->background_color.green);
+          SetPixelBlue(q,image->background_color.blue);
+          SetPixelOpacity(q,image->background_color.opacity);
+        }
+      else
+        {
+          SetPixelRed(q,ClampToQuantum((MagickRealType) QuantumRange*
+            ImfHalfToFloat(scanline[xx].r)));
+          SetPixelGreen(q,ClampToQuantum((MagickRealType) QuantumRange*
+            ImfHalfToFloat(scanline[xx].g)));
+          SetPixelBlue(q,ClampToQuantum((MagickRealType) QuantumRange*
+            ImfHalfToFloat(scanline[xx].b)));
+          SetPixelAlpha(q,ClampToQuantum((MagickRealType) QuantumRange*
+            ImfHalfToFloat(scanline[xx].a)));
+        }
       q++;
     }
     if (SyncAuthenticPixels(image,exception) == MagickFalse)
@@ -344,6 +409,10 @@ ModuleExport void UnregisterEXRImage(void)
 */
 static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
 {
+  const char
+    *sampling_factor,
+    *value;
+
   ImageInfo
     *write_info;
 
@@ -360,7 +429,9 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
     *scanline;
 
   int
-    compression;
+    channels,
+    compression,
+    factors[3];
 
   MagickBooleanType
     status;
@@ -411,21 +482,101 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
   if (write_info->compression == B44ACompression)
     compression=IMF_B44A_COMPRESSION;
 #endif
+  channels=0;
+  value=GetImageOption(image_info,"exr:color-type");
+  if (value != (const char *) NULL)
+    {
+      if (LocaleCompare(value,"RGB") == 0)
+        channels=IMF_WRITE_RGB;
+      else if (LocaleCompare(value,"RGBA") == 0)
+        channels=IMF_WRITE_RGBA;
+      else if (LocaleCompare(value,"YC") == 0)
+        channels=IMF_WRITE_YC;
+      else if (LocaleCompare(value,"YCA") == 0)
+        channels=IMF_WRITE_YCA;
+      else if (LocaleCompare(value,"Y") == 0)
+        channels=IMF_WRITE_Y;
+      else if (LocaleCompare(value,"YA") == 0)
+        channels=IMF_WRITE_YA;
+      else if (LocaleCompare(value,"R") == 0)
+        channels=IMF_WRITE_R;
+      else if (LocaleCompare(value,"G") == 0)
+        channels=IMF_WRITE_G;
+      else if (LocaleCompare(value,"B") == 0)
+        channels=IMF_WRITE_B;
+      else if (LocaleCompare(value,"A") == 0)
+        channels=IMF_WRITE_A;
+      else
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          CoderWarning,"ignoring invalid defined exr:color-type","=%s",value);
+   }
+  sampling_factor=(const char *) NULL;
+  factors[0]=0;
+  if (image_info->sampling_factor != (char *) NULL)
+    sampling_factor=image_info->sampling_factor;
+  if (sampling_factor != NULL)
+    {
+      /*
+        Sampling factors, valid values are 1x1 or 2x2.
+      */
+      if (sscanf(sampling_factor,"%d:%d:%d",factors,factors+1,factors+2) == 3)
+        {
+          if ((factors[0] == factors[1]) && (factors[1] == factors[2]))
+            factors[0]=1;
+          else
+            if ((factors[0] == (2*factors[1])) && (factors[2] == 0))
+              factors[0]=2;
+        }
+      else
+        if (sscanf(sampling_factor,"%dx%d",factors,factors+1) == 2)
+          {
+            if (factors[0] != factors[1])
+              factors[0]=0;
+          }
+      if ((factors[0] != 1) && (factors[0] != 2))
+        (void) ThrowMagickException(&image->exception,GetMagickModule(),
+          CoderWarning,"ignoring sampling-factor","=%s",sampling_factor);
+      else if (channels != 0)
+        {
+          /*
+            Cross check given color type and subsampling.
+          */
+          factors[1]=((channels == IMF_WRITE_YCA) ||
+            (channels == IMF_WRITE_YC)) ? 2 : 1;
+          if (factors[0] != factors[1])
+            (void) ThrowMagickException(&image->exception,GetMagickModule(),
+              CoderWarning,"sampling-factor and color type mismatch","=%s",
+              sampling_factor);
+        }
+    }
+  if (channels == 0)
+    {
+      /*
+        If no color type given, select it now.
+      */
+      if (factors[0] == 2)
+        channels=image->matte ? IMF_WRITE_YCA : IMF_WRITE_YC;
+      else
+        channels=image->matte ? IMF_WRITE_RGBA : IMF_WRITE_RGB;
+    }
   ImfHeaderSetCompression(hdr_info,compression);
   ImfHeaderSetLineOrder(hdr_info,IMF_INCREASING_Y);
-  file=ImfOpenOutputFile(write_info->filename,hdr_info,IMF_WRITE_RGBA);
+  file=ImfOpenOutputFile(write_info->filename,hdr_info,channels);
   ImfDeleteHeader(hdr_info);
   if (file == (ImfOutputFile *) NULL)
     {
+      (void) RelinquishUniqueFileResource(write_info->filename);
+      write_info=DestroyImageInfo(write_info);
       ThrowFileException(&image->exception,BlobError,"UnableToOpenBlob",
         ImfErrorMessage());
-      write_info=DestroyImageInfo(write_info);
       return(MagickFalse);
     }
   scanline=(ImfRgba *) AcquireQuantumMemory(image->columns,sizeof(*scanline));
   if (scanline == (ImfRgba *) NULL)
     {
       (void) ImfCloseOutputFile(file);
+      (void) RelinquishUniqueFileResource(write_info->filename);
+      write_info=DestroyImageInfo(write_info);
       ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
     }
   ResetMagickMemory(scanline,0,image->columns*sizeof(*scanline));
@@ -445,8 +596,7 @@ static MagickBooleanType WriteEXRImage(const ImageInfo *image_info,Image *image)
       if (image->matte == MagickFalse)
         ImfFloatToHalf(1.0,&half_quantum);
       else
-        ImfFloatToHalf(1.0-QuantumScale*GetPixelOpacity(p),
-          &half_quantum);
+        ImfFloatToHalf(1.0-QuantumScale*GetPixelOpacity(p),&half_quantum);
       scanline[x].a=half_quantum;
       p++;
     }
