@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -60,6 +60,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
+#include "magick/option.h"
 #include "magick/pixel.h"
 #include "magick/pixel-accessor.h"
 #include "magick/property.h"
@@ -112,6 +113,9 @@ static MagickBooleanType
 */
 static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
 {
+  const char
+    *option;
+
   FPXColorspace
     colorspace;
 
@@ -240,7 +244,10 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       FPX_ClearSystem();
       ThrowReaderException(CorruptImageError,"ImageTypeNotSupported");
     }
-  if (image_info->view == (char *) NULL)
+  option=image_info->view;
+  if (option == (const char *) NULL)
+    option=GetImageOption(image_info,"fpx:view");
+  if (option == (const char *) NULL)
     {
       float
         aspect_ratio;
@@ -352,6 +359,12 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
       FPX_ClearSystem();
       return(GetFirstImageInList(image));
     }
+  status=SetImageExtent(image,image->columns,image->rows);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
   /*
     Allocate memory for the image and pixel buffer.
   */
@@ -400,9 +413,9 @@ static Image *ReadFPXImage(const ImageInfo *image_info,ExceptionInfo *exception)
     if ((y % tile_height) == 0)
       {
         /*
-          Read FPX image tile (with or without viewing affine)..
+          Read FPX image tile (with or without viewing affine).
         */
-        if (image_info->view != (char *) NULL)
+        if (option != (const char *) NULL)
           fpx_status=FPX_ReadImageRectangle(flashpix,0,y,image->columns,y+
             tile_height-1,scene,&fpx_info);
         else
@@ -757,7 +770,8 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
 
   const char
     *comment,
-    *label;
+    *label,
+    *option;
 
   FPXCompressionOption
     compression;
@@ -830,7 +844,7 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
   if (image->matte != MagickFalse)
     colorspace.numberOfComponents=4;
   if ((image_info->type != TrueColorType) &&
-      IsGrayImage(image,&image->exception))
+      SetImageGray(image,&image->exception))
     {
       colorspace.numberOfComponents=1;
       colorspace.theComponents[0].myColor=MONOCHROME;
@@ -942,13 +956,13 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
       image->columns*fpx_info.components[i].columnStride;
     fpx_info.components[i].theData=pixels+i;
   }
-  fpx_info.components[0].myColorType.myColor=fpx_info.numberOfComponents != 1
-    ? NIFRGB_R : MONOCHROME;
+  fpx_info.components[0].myColorType.myColor=fpx_info.numberOfComponents != 1 ?
+    NIFRGB_R : MONOCHROME;
   fpx_info.components[1].myColorType.myColor=NIFRGB_G;
   fpx_info.components[2].myColorType.myColor=NIFRGB_B;
   fpx_info.components[3].myColorType.myColor=ALPHA;
   /*
-    Write image pixelss.
+    Write image pixels.
   */
   quantum_type=RGBQuantum;
   if (image->matte != MagickFalse)
@@ -975,7 +989,10 @@ static MagickBooleanType WriteFPXImage(const ImageInfo *image_info,Image *image)
       break;
   }
   quantum_info=DestroyQuantumInfo(quantum_info);
-  if (image_info->view != (char *) NULL)
+  option=image_info->view;
+  if (option == (const char *) NULL)
+    option=GetImageOption(image_info,"fpx:view");
+  if (option != (const char *) NULL)
     {
       FPXAffineMatrix
         affine;
