@@ -17,7 +17,7 @@
 %                                 June 2001                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -327,7 +327,9 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   option=GetImageOption(image_info,"jp2:reduce-factor");
   if (option != (const char *) NULL)
     parameters.cp_reduce=StringToInteger(option);
-  option=GetImageOption(image_info,"jp2:layer-number");
+  option=GetImageOption(image_info,"jp2:quality-layers");
+  if (option == (const char *) NULL)
+    option=GetImageOption(image_info,"jp2:layer-number");
   if (option != (const char *) NULL)
     parameters.cp_layer=StringToInteger(option);
   if (opj_setup_decoder(jp2_codec,&parameters) == 0)
@@ -354,9 +356,10 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
       /*
         Extract an area from the image.
       */
-      jp2_status=opj_set_decode_area(jp2_codec,jp2_image,image->extract_info.x,
-        image->extract_info.y,image->extract_info.x+(ssize_t) image->columns,
-        image->extract_info.y+(ssize_t) image->rows);
+      jp2_status=opj_set_decode_area(jp2_codec,jp2_image,
+        (OPJ_INT32) image->extract_info.x,(OPJ_INT32) image->extract_info.y,
+        (OPJ_INT32) image->extract_info.x+(ssize_t) image->columns,
+        (OPJ_INT32) image->extract_info.y+(ssize_t) image->rows);
       if (jp2_status == 0)
         {
           opj_stream_destroy(jp2_stream);
@@ -365,16 +368,16 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
           ThrowReaderException(DelegateError,"UnableToDecodeImageFile");
         }
     }
-  if (image_info->number_scenes != 0)
+  if ((image_info->number_scenes != 0) && (image_info->scene != 0))
     jp2_status=opj_get_decoded_tile(jp2_codec,jp2_stream,jp2_image,
-      (unsigned int) image_info->scene);
+      (unsigned int) image_info->scene-1);
   else
-   if (image->ping == MagickFalse)
-     {
-       jp2_status=opj_decode(jp2_codec,jp2_stream,jp2_image);
-       if (jp2_status != 0)
-         jp2_status=opj_end_decompress(jp2_codec,jp2_stream);
-     }
+    if (image->ping == MagickFalse)
+      {
+        jp2_status=opj_decode(jp2_codec,jp2_stream,jp2_image);
+        if (jp2_status != 0)
+          jp2_status=opj_end_decompress(jp2_codec,jp2_stream);
+      }
   if (jp2_status == 0)
     {
       opj_stream_destroy(jp2_stream);
@@ -398,6 +401,12 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   image->columns=(size_t) jp2_image->comps[0].w;
   image->rows=(size_t) jp2_image->comps[0].h;
   image->depth=jp2_image->comps[0].prec;
+  status=SetImageExtent(image,image->columns,image->rows);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
   image->compression=JPEG2000Compression;
   if (jp2_image->numcomps <= 2)
     {
@@ -430,7 +439,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     register ssize_t
       x;
@@ -480,7 +489,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
            }
            case 3:
            {
-             q->opacity=ClampToQuantum(pixel);
+             q->opacity=ClampToQuantum(QuantumRange-pixel);
              break;
            }
         }
@@ -500,6 +509,7 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   opj_destroy_codec(jp2_codec);
   opj_image_destroy(jp2_image);
   opj_destroy_cstr_index(&codestream_index);
+  (void) CloseBlob(image);
   return(GetFirstImageInList(image));
 }
 #endif
@@ -547,7 +557,6 @@ ModuleExport size_t RegisterJP2Image(void)
   entry->magick=(IsImageFormatHandler *) IsJP2;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->thread_support=NoThreadSupport;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
@@ -562,7 +571,6 @@ ModuleExport size_t RegisterJP2Image(void)
   entry->magick=(IsImageFormatHandler *) IsJ2K;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->thread_support=NoThreadSupport;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
@@ -577,7 +585,6 @@ ModuleExport size_t RegisterJP2Image(void)
   entry->magick=(IsImageFormatHandler *) IsJ2K;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->thread_support=NoThreadSupport;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
@@ -592,7 +599,6 @@ ModuleExport size_t RegisterJP2Image(void)
   entry->magick=(IsImageFormatHandler *) IsJP2;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->thread_support=NoThreadSupport;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
@@ -607,7 +613,6 @@ ModuleExport size_t RegisterJP2Image(void)
   entry->magick=(IsImageFormatHandler *) IsJP2;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->thread_support=NoThreadSupport;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
@@ -622,7 +627,6 @@ ModuleExport size_t RegisterJP2Image(void)
   entry->magick=(IsImageFormatHandler *) IsJP2;
   entry->adjoin=MagickFalse;
   entry->seekable_stream=MagickTrue;
-  entry->thread_support=NoThreadSupport;
 #if defined(MAGICKCORE_LIBOPENJP2_DELEGATE)
   entry->decoder=(DecodeImageHandler *) ReadJP2Image;
   entry->encoder=(EncodeImageHandler *) WriteJP2Image;
@@ -793,7 +797,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   ssize_t
     y;
 
-  size_t
+  unsigned int
     channels;
 
   /*
@@ -813,7 +817,8 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   */
   opj_set_default_encoder_parameters(&parameters);
   for (i=1; i < 6; i++)
-    if (((1U << (i+2)) > image->columns) && ((1U << (i+2)) > image->rows))
+    if (((ssize_t) (1 << (i+2)) > image->columns) &&
+        ((ssize_t) (1 << (i+2)) > image->rows))
       break;
   parameters.numresolution=i;
   option=GetImageOption(image_info,"jp2:number-resolutions");
@@ -822,9 +827,9 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   parameters.tcp_numlayers=1;
   parameters.tcp_rates[0]=0;  /* lossless */
   parameters.cp_disto_alloc=1;
-  if (image->quality != 0)
+  if ((image_info->quality != 0) && (image_info->quality != 100))
     {
-      parameters.tcp_distoratio[0]=(double) image->quality;
+      parameters.tcp_distoratio[0]=(double) image_info->quality;
       parameters.cp_fixed_quality=OPJ_TRUE;
     }
   if (image_info->extract != (char *) NULL)
@@ -924,12 +929,13 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
     }
   else
     {
-      (void) TransformImageColorspace(image,sRGBColorspace);
       if (IsGrayColorspace(image->colorspace) != MagickFalse)
         {
           channels=1;
           jp2_colorspace=OPJ_CLRSPC_GRAY;
         }
+      else
+        (void) TransformImageColorspace(image,sRGBColorspace);
       if (image->matte != MagickFalse)
         channels++;
     }
@@ -965,6 +971,11 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
       ((image->columns == 2048) || (image->rows == 1080) ||
        (image->columns == 4096) || (image->rows == 2160)))
     CinemaProfileCompliance(jp2_image,&parameters);
+  if (channels == 4)
+    jp2_image->comps[3].alpha=1;
+  else
+   if ((channels == 2) && (jp2_colorspace == OPJ_CLRSPC_GRAY))
+     jp2_image->comps[1].alpha=1;
   /*
     Convert to JP2 pixels.
   */
