@@ -17,7 +17,7 @@
 %                                 July 1999                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -245,14 +245,25 @@ ModuleExport size_t RegisterMPEGImage(void)
   entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
   entry->magick=(IsImageFormatHandler *) IsAVI;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Microsoft Audio/Visual Interleaved");
   entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
+  entry=SetMagickInfo("MKV");
+  entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
+  entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
+  entry->magick=(IsImageFormatHandler *) IsMPEG;
+  entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
+  entry->description=ConstantString("Multimedia Container");
+  entry->module=ConstantString("MPEG");
+  (void) RegisterMagickInfo(entry);
   entry=SetMagickInfo("MOV");
   entry->decoder=(DecodeImageHandler *) ReadMPEGImage;
   entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
   entry->magick=(IsImageFormatHandler *) IsMPEG;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("MPEG Video Stream");
   entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
@@ -261,6 +272,7 @@ ModuleExport size_t RegisterMPEGImage(void)
   entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
   entry->magick=(IsImageFormatHandler *) IsMPEG;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("MPEG Video Stream");
   entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
@@ -269,6 +281,7 @@ ModuleExport size_t RegisterMPEGImage(void)
   entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
   entry->magick=(IsImageFormatHandler *) IsMPEG;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("MPEG Video Stream");
   entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
@@ -277,6 +290,7 @@ ModuleExport size_t RegisterMPEGImage(void)
   entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
   entry->magick=(IsImageFormatHandler *) IsMPEG;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("MPEG-4 Video Stream");
   entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
@@ -285,6 +299,7 @@ ModuleExport size_t RegisterMPEGImage(void)
   entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
   entry->magick=(IsImageFormatHandler *) IsMPEG;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("MPEG Video Stream");
   entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
@@ -293,6 +308,7 @@ ModuleExport size_t RegisterMPEGImage(void)
   entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
   entry->magick=(IsImageFormatHandler *) IsMPEG;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Raw MPEG-4 Video");
   entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
@@ -301,6 +317,7 @@ ModuleExport size_t RegisterMPEGImage(void)
   entry->encoder=(EncodeImageHandler *) WriteMPEGImage;
   entry->magick=(IsImageFormatHandler *) IsMPEG;
   entry->blob_support=MagickFalse;
+  entry->seekable_stream=MagickTrue;
   entry->description=ConstantString("Windows Media Video");
   entry->module=ConstantString("MPEG");
   (void) RegisterMagickInfo(entry);
@@ -329,12 +346,13 @@ ModuleExport size_t RegisterMPEGImage(void)
 ModuleExport void UnregisterMPEGImage(void)
 {
   (void) UnregisterMagickInfo("WMV");
+  (void) UnregisterMagickInfo("MOV");
   (void) UnregisterMagickInfo("M4V");
   (void) UnregisterMagickInfo("M2V");
   (void) UnregisterMagickInfo("MP4");
   (void) UnregisterMagickInfo("MPG");
   (void) UnregisterMagickInfo("MPEG");
-  (void) UnregisterMagickInfo("MOV");
+  (void) UnregisterMagickInfo("MKV");
   (void) UnregisterMagickInfo("AVI");
 }
 
@@ -365,21 +383,6 @@ ModuleExport void UnregisterMPEGImage(void)
 %    o image:  The image.
 %
 */
-
-static inline double MagickMax(const double x,const double y)
-{
-  if (x > y)
-    return(x);
-  return(y);
-}
-
-static inline double MagickMin(const double x,const double y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 static MagickBooleanType CopyDelegateFile(const char *source,
   const char *destination)
 {
@@ -412,7 +415,7 @@ static MagickBooleanType CopyDelegateFile(const char *source,
   assert(source != (const char *) NULL);
   assert(destination != (char *) NULL);
   status=GetPathAttributes(destination,&attributes);
-  if ((status != MagickFalse) && (attributes.st_size != 0))
+  if ((status != MagickFalse) && (attributes.st_size > 0))
     return(MagickTrue);
   /*
     Copy source file to destination.
@@ -427,7 +430,7 @@ static MagickBooleanType CopyDelegateFile(const char *source,
       return(MagickFalse);
     }
   quantum=(size_t) MagickMaxBufferExtent;
-  if ((fstat(source_file,&attributes) == 0) && (attributes.st_size != 0))
+  if ((fstat(source_file,&attributes) == 0) && (attributes.st_size > 0))
     quantum=(size_t) MagickMin((double) attributes.st_size,
       MagickMaxBufferExtent);
   buffer=(unsigned char *) AcquireQuantumMemory(quantum,sizeof(*buffer));
@@ -518,6 +521,7 @@ static MagickBooleanType WriteMPEGImage(const ImageInfo *image_info,
     basename);
   count=0;
   write_info=CloneImageInfo(image_info);
+  *write_info->magick='\0';
   for (p=coalesce_image; p != (Image *) NULL; p=GetNextImageInList(p))
   {
     char
