@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -188,6 +188,9 @@ static MagickBooleanType InvokePostscriptDelegate(
   gs_main_instance
     *interpreter;
 
+  gsapi_revision_t
+    revision;
+
   int
     argc,
     code;
@@ -215,12 +218,16 @@ static MagickBooleanType InvokePostscriptDelegate(
   ghost_info_struct.set_stdio=(int (*)(gs_main_instance *,int(*)(void *,char *,
     int),int(*)(void *,const char *,int),int(*)(void *, const char *, int)))
     gsapi_set_stdio;
+  ghost_info_struct.revision=(int (*)(gsapi_revision_t *,int)) gsapi_revision;
 #endif
   if (ghost_info == (GhostInfo *) NULL)
     ExecuteGhostscriptCommand(command,status);
+  if ((ghost_info->revision)(&revision,sizeof(revision)) != 0)
+    revision.revision=0;
   if (verbose != MagickFalse)
     {
-      (void) fputs("[ghostscript library]",stdout);
+      (void) fprintf(stdout,"[ghostscript library %.2f]",(double)
+        revision.revision/100.0);
       SetArgsStart(command,args_start);
       (void) fputs(args_start,stdout);
     }
@@ -231,7 +238,10 @@ static MagickBooleanType InvokePostscriptDelegate(
   code=0;
   argv=StringToArgv(command,&argc);
   if (argv == (char **) NULL)
-    return(MagickFalse);
+    {
+      (ghost_info->delete_instance)(interpreter);
+      return(MagickFalse);
+    }
   (void) (ghost_info->set_stdio)(interpreter,(int(MagickDLLCall *)(void *,
     char *,int)) NULL,PostscriptDelegateMessage,PostscriptDelegateMessage);
   status=(ghost_info->init_with_args)(interpreter,argc-1,argv+1);
@@ -248,12 +258,14 @@ static MagickBooleanType InvokePostscriptDelegate(
       SetArgsStart(command,args_start);
       if (status == -101) /* quit */
         (void) FormatLocaleString(message,MaxTextExtent,
-          "[ghostscript library]%s: %s",args_start,errors);
+          "[ghostscript library %.2f]%s: %s",(double)revision.revision / 100,
+          args_start,errors);
       else
         {
           (void) ThrowMagickException(exception,GetMagickModule(),
             DelegateError,"PostscriptDelegateFailed",
-            "`[ghostscript library]%s': %s",args_start,errors);
+            "`[ghostscript library %.2f]%s': %s",
+            (double)revision.revision / 100,args_start,errors);
           if (errors != (char *) NULL)
             errors=DestroyString(errors);
           (void) LogMagickEvent(CoderEvent,GetMagickModule(),
@@ -407,12 +419,12 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
 
   char
     command[MaxTextExtent],
-    density[MaxTextExtent],
+    *density,
     filename[MaxTextExtent],
     geometry[MaxTextExtent],
     input_filename[MaxTextExtent],
     message[MaxTextExtent],
-    options[MaxTextExtent],
+    *options,
     postscript_filename[MaxTextExtent];
 
   const char
@@ -643,11 +655,14 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
           continue;
         length=extent;
         profile=BlobToStringInfo((const void *) NULL,length);
-        p=GetStringInfoDatum(profile);
-        for (i=0; i < (ssize_t) length; i++)
-          *p++=(unsigned char) ProfileInteger(image,hex_digits);
-        (void) SetImageProfile(image,"8bim",profile);
-        profile=DestroyStringInfo(profile);
+        if (profile != (StringInfo *) NULL)
+          {
+            p=GetStringInfoDatum(profile);
+            for (i=0; i < (ssize_t) length; i++)
+              *p++=(unsigned char) ProfileInteger(image,hex_digits);
+            (void) SetImageProfile(image,"8bim",profile);
+            profile=DestroyStringInfo(profile);
+          }
         continue;
       }
     if (LocaleNCompare(BeginXMPPacket,command,strlen(BeginXMPPacket)) == 0)
@@ -854,7 +869,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
       image=DestroyImageList(image);
       return((Image *) NULL);
     }
-  *options='\0';
+  density=AcquireString("");
+  options=AcquireString("");
   (void) FormatLocaleString(density,MaxTextExtent,"%gx%g",resolution.x,
     resolution.y);
   (void) FormatLocaleString(options,MaxTextExtent,"-g%.20gx%.20g ",(double)
@@ -892,6 +908,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
     read_info->antialias != MagickFalse ? 4 : 1,
     read_info->antialias != MagickFalse ? 4 : 1,density,options,filename,
     postscript_filename,input_filename);
+  options=DestroyString(options);
+  density=DestroyString(density);
   *message='\0';
   status=InvokePostscriptDelegate(read_info->verbose,command,message,exception);
   (void) InterpretImageFilename(image_info,image,filename,1,
@@ -973,6 +991,8 @@ static Image *ReadPSImage(const ImageInfo *image_info,ExceptionInfo *exception)
   do
   {
     (void) CopyMagickString(postscript_image->filename,filename,MaxTextExtent);
+    (void) CopyMagickString(postscript_image->magick,image->magick,
+      MaxTextExtent);
     if (columns != 0)
       postscript_image->magick_columns=columns;
     if (rows != 0)
@@ -1137,14 +1157,7 @@ ModuleExport void UnregisterPSImage(void)
 %
 */
 
-static inline size_t MagickMin(const size_t x,const size_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
-static inline unsigned char *PopHexPixel(const char **hex_digits,
+static inline unsigned char *PopHexPixel(const char *const *hex_digits,
   const size_t pixel,unsigned char *pixels)
 {
   register const char
@@ -1177,7 +1190,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
 }
 
   static const char
-    *hex_digits[] =
+    *const hex_digits[] =
     {
       "00", "01", "02", "03", "04", "05", "06", "07", "08", "09", "0A", "0B",
       "0C", "0D", "0E", "0F", "10", "11", "12", "13", "14", "15", "16", "17",
@@ -1200,9 +1213,9 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
       "D8", "D9", "DA", "DB", "DC", "DD", "DE", "DF", "E0", "E1", "E2", "E3",
       "E4", "E5", "E6", "E7", "E8", "E9", "EA", "EB", "EC", "ED", "EE", "EF",
       "F0", "F1", "F2", "F3", "F4", "F5", "F6", "F7", "F8", "F9", "FA", "FB",
-      "FC", "FD", "FE", "FF",  (char *) NULL
+      "FC", "FD", "FE", "FF",  (const char *) NULL
     },
-    *PostscriptProlog[]=
+    *const PostscriptProlog[]=
     {
       "%%BeginProlog",
       "%",
@@ -1441,10 +1454,9 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
       "  token pop /y exch def pop",
       "  currentfile buffer readline pop",
       "  token pop /pointsize exch def pop",
-      "  /Times-Roman findfont pointsize scalefont setfont",
-      (char *) NULL
+      (const char *) NULL
     },
-    *PostscriptEpilog[]=
+    *const PostscriptEpilog[]=
     {
       "  x y scale",
       "  currentfile buffer readline pop",
@@ -1456,7 +1468,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
       "  token pop /compression exch def pop",
       "  class 0 gt { PseudoClassImage } { DirectClassImage } ifelse",
       "  grestore",
-      (char *) NULL
+      (const char *) NULL
     };
 
   char
@@ -1469,7 +1481,7 @@ static MagickBooleanType WritePSImage(const ImageInfo *image_info,Image *image)
     compression;
 
   const char
-    **s,
+    *const *s,
     *value;
 
   const StringInfo
@@ -1816,13 +1828,18 @@ RestoreMSCWarning
         }
         value=GetImageProperty(image,"label");
         if (value != (const char *) NULL)
-          for (j=(ssize_t) MultilineCensus(value)-1; j >= 0; j--)
           {
-            (void) WriteBlobString(image,"  /label 512 string def\n");
-            (void) WriteBlobString(image,"  currentfile label readline pop\n");
-            (void) FormatLocaleString(buffer,MaxTextExtent,
-              "  0 y %g add moveto label show pop\n",j*pointsize+12);
-            (void) WriteBlobString(image,buffer);
+            (void) WriteBlobString(image,
+              "  /Times-Roman findfont pointsize scalefont setfont\n");
+            for (j=(ssize_t) MultilineCensus(value)-1; j >= 0; j--)
+            {
+              (void) WriteBlobString(image,"  /label 512 string def\n");
+              (void) WriteBlobString(image,
+                "  currentfile label readline pop\n");
+              (void) FormatLocaleString(buffer,MaxTextExtent,
+                "  0 y %g add moveto label show pop\n",j*pointsize+12);
+              (void) WriteBlobString(image,buffer);
+            }
           }
         for (s=PostscriptEpilog; *s != (char *) NULL; s++)
         {
@@ -1882,9 +1899,9 @@ RestoreMSCWarning
     index=(IndexPacket) 0;
     x=0;
     if ((image_info->type != TrueColorType) &&
-        (IsGrayImage(image,&image->exception) != MagickFalse))
+        (SetImageGray(image,&image->exception) != MagickFalse))
       {
-        if (IsMonochromeImage(image,&image->exception) == MagickFalse)
+        if (SetImageMonochrome(image,&image->exception) == MagickFalse)
           {
             Quantum
               pixel;
