@@ -18,7 +18,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -335,7 +335,7 @@ static MagickBooleanType SerializeImageChannel(const ImageInfo *image_info,
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   status=MagickTrue;
-  pack=IsMonochromeImage(image,&image->exception) == MagickFalse ? 1UL : 8UL;
+  pack=SetImageMonochrome(image,&image->exception) == MagickFalse ? 1UL : 8UL;
   padded_columns=((image->columns+pack-1)/pack)*pack;
   *length=(size_t) padded_columns*image->rows/pack;
   *pixel_info=AcquireVirtualMemory(*length,sizeof(*q));
@@ -474,11 +474,15 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
     Note BeginData DSC comment for update later.
   */
   start=TellBlob(image);
+  if (start < 0)
+    ThrowWriterException(CorruptImageError,"ImproperImageHeader");
   (void) FormatLocaleString(buffer,MaxTextExtent,
     "%%%%BeginData:%13ld %s Bytes\n",0L,compression == NoCompression ?
     "ASCII" : "BINARY");
   (void) WriteBlobString(image,buffer);
   stop=TellBlob(image);
+  if (stop < 0)
+    ThrowWriterException(CorruptImageError,"ImproperImageHeader");
   /*
     Only lossless compressions for the mask.
   */
@@ -487,38 +491,38 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
     case NoCompression:
     default:
     {
-      (void) FormatLocaleString(buffer,MaxTextExtent,
-        "currentfile %.20g %.20g "PS3_NoCompression" ByteStreamDecodeFilter\n",
-        (double) image->columns,(double) image->rows);
+      (void) FormatLocaleString(buffer,MaxTextExtent,"currentfile %.20g %.20g "
+        PS3_NoCompression " ByteStreamDecodeFilter\n",(double) image->columns,
+        (double) image->rows);
       break;
     }
     case FaxCompression:
     case Group4Compression:
     {
-      (void) FormatLocaleString(buffer,MaxTextExtent,
-        "currentfile %.20g %.20g "PS3_FaxCompression" ByteStreamDecodeFilter\n",
-        (double) image->columns,(double) image->rows);
+      (void) FormatLocaleString(buffer,MaxTextExtent,"currentfile %.20g %.20g "
+        PS3_FaxCompression " ByteStreamDecodeFilter\n",(double) image->columns,
+        (double) image->rows);
       break;
     }
     case LZWCompression:
     {
-      (void) FormatLocaleString(buffer,MaxTextExtent,
-        "currentfile %.20g %.20g "PS3_LZWCompression" ByteStreamDecodeFilter\n",
-        (double) image->columns,(double) image->rows);
+      (void) FormatLocaleString(buffer,MaxTextExtent,"currentfile %.20g %.20g "
+        PS3_LZWCompression " ByteStreamDecodeFilter\n",(double) image->columns,
+        (double) image->rows);
       break;
     }
     case RLECompression:
     {
-      (void) FormatLocaleString(buffer,MaxTextExtent,
-        "currentfile %.20g %.20g "PS3_RLECompression" ByteStreamDecodeFilter\n",
-        (double) image->columns,(double) image->rows);
+      (void) FormatLocaleString(buffer,MaxTextExtent,"currentfile %.20g %.20g "
+        PS3_RLECompression " ByteStreamDecodeFilter\n",(double) image->columns,
+        (double) image->rows);
       break;
     }
     case ZipCompression:
     {
-      (void) FormatLocaleString(buffer,MaxTextExtent,
-        "currentfile %.20g %.20g "PS3_ZipCompression" ByteStreamDecodeFilter\n",
-        (double) image->columns,(double) image->rows);
+      (void) FormatLocaleString(buffer,MaxTextExtent,"currentfile %.20g %.20g "
+        PS3_ZipCompression " ByteStreamDecodeFilter\n",(double) image->columns,
+        (double) image->rows);
       break;
     }
   }
@@ -589,6 +593,7 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
       status=SerializeImageChannel(image_info,mask_image,&pixel_info,&length);
       if (status == MagickFalse)
         break;
+      pixels=(unsigned char *) GetVirtualMemoryBlob(pixel_info);
       status=ZLIBEncodeImage(image,length,pixels);
       pixel_info=RelinquishVirtualMemory(pixel_info);
       break;
@@ -598,6 +603,8 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
   (void) WriteBlobByte(image,'\n');
   length=(size_t) (TellBlob(image)-stop);
   stop=TellBlob(image);
+  if (stop < 0)
+    ThrowWriterException(CorruptImageError,"ImproperImageHeader");
   offset=SeekBlob(image,start,SEEK_SET);
   if (offset < 0)
     ThrowWriterException(CorruptImageError,"ImproperImageHeader");
@@ -616,32 +623,32 @@ static MagickBooleanType WritePS3MaskImage(const ImageInfo *image_info,
 static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
 {
   static const char
-    *PostscriptProlog[]=
+    *const PostscriptProlog[]=
     {
       "/ByteStreamDecodeFilter",
       "{",
       "  /z exch def",
       "  /r exch def",
       "  /c exch def",
-      "  z "PS3_NoCompression" eq { /ASCII85Decode filter } if",
-      "  z "PS3_FaxCompression" eq",
+      "  z " PS3_NoCompression " eq { /ASCII85Decode filter } if",
+      "  z " PS3_FaxCompression " eq",
       "  {",
       "    <<",
-      "      /K "CCITTParam,
+      "      /K " CCITTParam,
       "      /Columns c",
       "      /Rows r",
       "    >>",
       "    /CCITTFaxDecode filter",
       "  } if",
-      "  z "PS3_JPEGCompression" eq { /DCTDecode filter } if",
-      "  z "PS3_LZWCompression" eq { /LZWDecode filter } if",
-      "  z "PS3_RLECompression" eq { /RunLengthDecode filter } if",
-      "  z "PS3_ZipCompression" eq { /FlateDecode filter } if",
+      "  z " PS3_JPEGCompression " eq { /DCTDecode filter } if",
+      "  z " PS3_LZWCompression " eq { /LZWDecode filter } if",
+      "  z " PS3_RLECompression " eq { /RunLengthDecode filter } if",
+      "  z " PS3_ZipCompression " eq { /FlateDecode filter } if",
       "} bind def",
       "",
       "/DirectClassImageDict",
       "{",
-      "  colorspace "PS3_RGBColorspace" eq",
+      "  colorspace " PS3_RGBColorspace " eq",
       "  {",
       "    /DeviceRGB setcolorspace",
       "    <<",
@@ -666,7 +673,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "      /MultipleDataSources false",
       "      /ImageMatrix [columns 0 0 rows neg 0 rows]",
       "      /Decode",
-      "        compression "PS3_JPEGCompression" eq",
+      "        compression " PS3_JPEGCompression " eq",
       "        { [1 0 1 0 1 0 1 0] }",
       "        { [0 1 0 1 0 1 0 1] }",
       "        ifelse",
@@ -699,7 +706,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "  {",
       "    % RGB colormap.",
       "    /colormap colors 3 mul string def",
-      "    compression "PS3_NoCompression" eq",
+      "    compression " PS3_NoCompression " eq",
       "    { currentfile /ASCII85Decode filter colormap readstring pop pop }",
       "    { currentfile colormap readstring pop pop }",
       "    ifelse",
@@ -719,7 +726,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "",
       "/NonMaskedImageDict",
       "{",
-      "  class "PS3_PseudoClass" eq",
+      "  class " PS3_PseudoClass " eq",
       "  { PseudoClassImageDict }",
       "  { DirectClassImageDict }",
       "  ifelse",
@@ -763,9 +770,9 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "  token pop /y exch def pop",
       "  currentfile buffer readline pop",
       "  token pop /pointsize exch def pop",
-      (char *) NULL
+      (const char *) NULL
     },
-    *PostscriptEpilog[]=
+    *const PostscriptEpilog[]=
     {
       "  x y scale",
       "  % Clipping path.",
@@ -804,7 +811,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "  grestore",
       "  sp { showpage } if",
       "} bind def",
-      (char *) NULL
+      (const char *) NULL
     };
 
   char
@@ -817,8 +824,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     compression;
 
   const char
+    *const *q,
     *option,
-    **q,
     *value;
 
   double
@@ -893,7 +900,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     case FaxCompression:
     case Group4Compression:
     {
-      if ((IsMonochromeImage(image,&image->exception) == MagickFalse) ||
+      if ((SetImageMonochrome(image,&image->exception) == MagickFalse) ||
           (image->matte != MagickFalse))
         compression=RLECompression;
       break;
@@ -1037,7 +1044,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
               (void) WriteBlobString(image,
                 "%%DocumentProcessColors: Cyan Magenta Yellow Black\n");
             else
-              if (IsGrayImage(image,&image->exception) != MagickFalse)
+              if (SetImageGray(image,&image->exception) != MagickFalse)
                 (void) WriteBlobString(image,
                   "%%DocumentProcessColors: Black\n");
           }
@@ -1123,7 +1130,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       (void) WriteBlobString(image,
         "%%PageProcessColors: Cyan Magenta Yellow Black\n");
     else
-      if (IsGrayImage(image,&image->exception) != MagickFalse)
+      if (SetImageGray(image,&image->exception) != MagickFalse)
         (void) WriteBlobString(image,"%%PageProcessColors: Black\n");
     /*
       Adjust document bounding box to bound page bounding box.
@@ -1177,11 +1184,15 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       Remember position of BeginData comment so we can update it.
     */
     start=TellBlob(image);
+    if (start < 0)
+      ThrowWriterException(CorruptImageError,"ImproperImageHeader");
     (void) FormatLocaleString(buffer,MaxTextExtent,
       "%%%%BeginData:%13ld %s Bytes\n",0L,
       compression == NoCompression ? "ASCII" : "BINARY");
     (void) WriteBlobString(image,buffer);
     stop=TellBlob(image);
+    if (stop < 0)
+      ThrowWriterException(CorruptImageError,"ImproperImageHeader");
     (void) WriteBlobString(image,"DisplayImage\n");
     /*
       Translate, scale, and font point size.
@@ -1248,7 +1259,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     */
     option=GetImageOption(image_info,"ps3:imagemask");
     (void) WriteBlobString(image,((option != (const char *) NULL) &&
-      (IsMonochromeImage(image,&image->exception) != MagickFalse)) ?
+      (SetImageMonochrome(image,&image->exception) != MagickFalse)) ?
       "true\n" : "false\n");
     /*
       Output pixel data.
@@ -1260,8 +1271,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
         (image_info->type != ColorSeparationType) &&
         (image_info->type != ColorSeparationMatteType) &&
         (image->colorspace != CMYKColorspace) &&
-        ((IsGrayImage(image,&image->exception) != MagickFalse) ||
-         (IsMonochromeImage(image,&image->exception) != MagickFalse)))
+        ((SetImageGray(image,&image->exception) != MagickFalse) ||
+         (SetImageMonochrome(image,&image->exception) != MagickFalse)))
       {
         /*
           Gray images.
@@ -1310,7 +1321,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
           1 bit or 8 bit components?
         */
         (void) FormatLocaleString(buffer,MaxTextExtent,"%d\n",
-          IsMonochromeImage(image,&image->exception) != MagickFalse ? 1 : 8);
+          SetImageMonochrome(image,&image->exception) != MagickFalse ? 1 : 8);
         (void) WriteBlobString(image,buffer);
         /*
           Image data.
@@ -1468,11 +1479,6 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
                 (void) WriteBlobString(image,PS3_NoCompression"\n");
                 break;
               }
-              case JPEGCompression:
-              {
-                (void) WriteBlobString(image,PS3_JPEGCompression"\n");
-                break;
-              }
               case RLECompression:
               {
                 (void) WriteBlobString(image,PS3_RLECompression"\n");
@@ -1544,12 +1550,6 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
                 status=MagickTrue;
                 break;
               }
-              case JPEGCompression:
-              {
-                status=InjectImageBlob(image_info,image,image,"jpeg",
-                  &image->exception);
-                break;
-              }
               case RLECompression:
               {
                 status=PackbitsEncodeImage(image,length,pixels);
@@ -1579,6 +1579,8 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
     */
     length=(size_t) (TellBlob(image)-stop);
     stop=TellBlob(image);
+    if (stop < 0)
+      ThrowWriterException(CorruptImageError,"ImproperImageHeader");
     offset=SeekBlob(image,start,SEEK_SET);
     if (offset < 0)
       ThrowWriterException(CorruptImageError,"ImproperImageHeader");
@@ -1586,7 +1588,7 @@ static MagickBooleanType WritePS3Image(const ImageInfo *image_info,Image *image)
       "%%%%BeginData:%13ld %s Bytes\n",(long) length,
       compression == NoCompression ? "ASCII" : "BINARY");
     (void) WriteBlobString(image,buffer);
-    offset=SeekBlob(image,stop,SEEK_SET);
+    (void) SeekBlob(image,stop,SEEK_SET);
     (void) WriteBlobString(image,"%%EndData\n");
     /*
       End private dictionary if this an EPS.
