@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -40,6 +40,7 @@
   Include declarations.
 */
 #include "magick/studio.h"
+#include "magick/artifact.h"
 #include "magick/attribute.h"
 #include "magick/blob.h"
 #include "magick/blob-private.h"
@@ -58,6 +59,7 @@
 #include "magick/memory_.h"
 #include "magick/monitor.h"
 #include "magick/monitor-private.h"
+#include "magick/option.h"
 #include "magick/pixel-accessor.h"
 #include "magick/property.h"
 #include "magick/quantum-private.h"
@@ -247,14 +249,10 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
   if ((tga_info.image_type != TGAColormap) &&
       (tga_info.image_type != TGARLEColormap))
     image->depth=(size_t) ((tga_info.bits_per_pixel <= 8) ? 8 :
-      (tga_info.bits_per_pixel <= 16) ? 5 :
-      (tga_info.bits_per_pixel == 24) ? 8 :
-      (tga_info.bits_per_pixel == 32) ? 8 : 8);
+      (tga_info.bits_per_pixel <= 16) ? 5 : 8);
   else
     image->depth=(size_t) ((tga_info.colormap_size <= 8) ? 8 :
-      (tga_info.colormap_size <= 16) ? 5 :
-      (tga_info.colormap_size == 24) ? 8 :
-      (tga_info.colormap_size == 32) ? 8 : 8);
+      (tga_info.colormap_size <= 16) ? 5 : 8);
   if ((tga_info.image_type == TGAColormap) ||
       (tga_info.image_type == TGAMonochrome) ||
       (tga_info.image_type == TGARLEColormap) ||
@@ -262,12 +260,13 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
     image->storage_class=PseudoClass;
   image->compression=NoCompression;
   if ((tga_info.image_type == TGARLEColormap) ||
-      (tga_info.image_type == TGARLEMonochrome))
+      (tga_info.image_type == TGARLEMonochrome) ||
+      (tga_info.image_type == TGARLERGB))
     image->compression=RLECompression;
   if (image->storage_class == PseudoClass)
     {
       if (tga_info.colormap_type != 0)
-        image->colors=tga_info.colormap_length;
+        image->colors=tga_info.colormap_index+tga_info.colormap_length;
       else
         {
           size_t
@@ -297,16 +296,36 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
           sizeof(*comment));
       if (comment == (char *) NULL)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      count=ReadBlob(image,tga_info.id_length,(unsigned char *) comment);
+      (void) ReadBlob(image,tga_info.id_length,(unsigned char *) comment);
       comment[tga_info.id_length]='\0';
       (void) SetImageProperty(image,"comment",comment);
       comment=DestroyString(comment);
     }
+  if (tga_info.attributes & (1UL << 4))
+    {
+      if (tga_info.attributes & (1UL << 5))
+        SetImageArtifact(image,"tga:image-origin","TopRight");
+      else
+        SetImageArtifact(image,"tga:image-origin","BottomRight");
+    }
+  else
+    {
+      if (tga_info.attributes & (1UL << 5))
+        SetImageArtifact(image,"tga:image-origin","TopLeft");
+      else
+        SetImageArtifact(image,"tga:image-origin","BottomLeft");
+    }
   if (image_info->ping != MagickFalse)
     {
       (void) CloseBlob(image);
       return(image);
     }
+  status=SetImageExtent(image,image->columns,image->rows);
+  if (status == MagickFalse)
+    {
+      InheritException(exception,&image->exception);
+      return(DestroyImageList(image));
+    }
   (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
   pixel.opacity=(Quantum) OpaqueOpacity;
   if (tga_info.colormap_type != 0)
@@ -314,9 +333,13 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
       /*
         Read TGA raster colormap.
       */
+      if (image->colors < tga_info.colormap_index)
+        image->colors=tga_info.colormap_index;
       if (AcquireImageColormap(image,image->colors) == MagickFalse)
         ThrowReaderException(ResourceLimitError,"MemoryAllocationFailed");
-      for (i=0; i < (ssize_t) image->colors; i++)
+      for (i=0; i < (ssize_t) tga_info.colormap_index; i++)
+        image->colormap[i]=pixel;
+      for ( ; i < (ssize_t) image->colors; i++)
       {
         switch (tga_info.colormap_size)
         {
@@ -408,7 +431,7 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
           else
             {
               count=ReadBlob(image,1,&runlength);
-              if (count == 0)
+              if (count != 1)
                 ThrowReaderException(CorruptImageError,"UnableToReadImageData");
               flag=runlength & 0x80;
               if (flag != 0)
@@ -456,8 +479,8 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
               (1UL*(j & 0xe0) >> 5),range);
             pixel.blue=ScaleAnyToQuantum(1UL*(j & 0x1f),range);
             if (image->matte != MagickFalse)
-              pixel.opacity=(k & 0x80) == 0 ? (Quantum) OpaqueOpacity :
-                (Quantum) TransparentOpacity;
+              pixel.opacity=(k & 0x80) == 0 ? (Quantum) TransparentOpacity :
+                (Quantum) OpaqueOpacity;
             if (image->storage_class == PseudoClass)
               index=ConstrainColormapIndex(image,((size_t) k << 8)+j);
             break;
@@ -500,10 +523,12 @@ static Image *ReadTGAImage(const ImageInfo *image_info,
         SetPixelOpacity(q,pixel.opacity);
       q++;
     }
-    if (((tga_info.attributes & 0xc0) >> 6) == 4)
-      offset+=4;
-    else
-      if (((tga_info.attributes & 0xc0) >> 6) == 2)
+    /*
+      if (((unsigned char) (tga_info.attributes & 0xc0) >> 6) == 4)
+        offset+=4;
+      else
+    */
+      if (((unsigned char) (tga_info.attributes & 0xc0) >> 6) == 2)
         offset+=2;
       else
         offset++;
@@ -640,16 +665,9 @@ ModuleExport void UnregisterTGAImage(void)
 %    o image:  The image.
 %
 */
-
-static inline size_t MagickMin(const size_t x,const size_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 static inline void WriteTGAPixel(Image *image,TGAImageType image_type,
-  const IndexPacket *indexes,const PixelPacket *p)
+  const IndexPacket *indexes,const PixelPacket *p,const QuantumAny range,
+  const double midpoint)
 {
   if (image_type == TGAColormap || image_type == TGARLEColormap)
     (void) WriteBlobByte(image,(unsigned char) GetPixelIndex(indexes));
@@ -659,18 +677,34 @@ static inline void WriteTGAPixel(Image *image,TGAImageType image_type,
         (void) WriteBlobByte(image,ScaleQuantumToChar(ClampToQuantum(
           GetPixelLuma(image,p))));
       else
-        {
-          (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelBlue(p)));
-          (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelGreen(p)));
-          (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelRed(p)));
-          if (image->matte != MagickFalse)
-            (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelAlpha(p)));
-        }
+        if (image->depth == 5)
+          {
+            unsigned char
+              green,
+              value;
+
+              green=(unsigned char) ScaleQuantumToAny(GetPixelGreen(p),range);
+              value=((unsigned char) ScaleQuantumToAny(GetPixelBlue(p),range)) |
+                ((green & 0x07) << 5);
+              (void) WriteBlobByte(image,value);
+              value=(unsigned char) ((((image->matte != MagickFalse) && 
+                ((double) GetPixelOpacity(p) < midpoint)) ? 0x80 : 0) |
+                ((unsigned char) ScaleQuantumToAny(GetPixelRed(p),range) << 2) |
+                ((green & 0x18) >> 3));
+              (void) WriteBlobByte(image,value);
+          }
+        else
+          {
+            (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelBlue(p)));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelGreen(p)));
+            (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelRed(p)));
+            if (image->matte != MagickFalse)
+              (void) WriteBlobByte(image,ScaleQuantumToChar(GetPixelAlpha(p)));
+          }
     }
 }
 
-static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
-  Image *image)
+static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,Image *image)
 {
   CompressionType
     compression;
@@ -678,9 +712,15 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
   const char
     *value;
 
+  const double
+    midpoint = QuantumRange/2.0;
+
   MagickBooleanType
     status;
 
+  QuantumAny
+    range;
+
   register const IndexPacket
     *indexes;
 
@@ -724,6 +764,7 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
   compression=image->compression;
   if (image_info->compression != UndefinedCompression)
     compression=image_info->compression;
+  range=GetQuantumRange(5UL);
   tga_info.id_length=0;
   value=GetImageProperty(image,"comment");
   if (value != (const char *) NULL)
@@ -742,7 +783,7 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
       (image_info->type != TrueColorMatteType) &&
       (image_info->type != PaletteType) &&
       (image->matte == MagickFalse) &&
-      (IsGrayImage(image,&image->exception) != MagickFalse))
+      (SetImageGray(image,&image->exception) != MagickFalse))
     tga_info.image_type=compression == RLECompression ? TGARLEMonochrome :
       TGAMonochrome;
   else
@@ -751,13 +792,21 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
         /*
           Full color TGA raster.
         */
-        tga_info.image_type=compression == RLECompression ? TGARLERGB :
-          TGARGB;
-        tga_info.bits_per_pixel=24;
-        if (image->matte != MagickFalse)
+        tga_info.image_type=compression == RLECompression ? TGARLERGB : TGARGB;
+        if (image_info->depth == 5)
           {
-            tga_info.bits_per_pixel=32;
-            tga_info.attributes=8;  /* # of alpha bits */
+            tga_info.bits_per_pixel=16;
+            if (image->matte != MagickFalse)
+              tga_info.attributes=1;  /* # of alpha bits */
+          }
+        else
+          {
+            tga_info.bits_per_pixel=24;
+            if (image->matte != MagickFalse)
+              {
+                tga_info.bits_per_pixel=32;
+                tga_info.attributes=8;  /* # of alpha bits */
+              }
           }
       }
     else
@@ -765,12 +814,28 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
         /*
           Colormapped TGA raster.
         */
-        tga_info.image_type=compression == RLECompression ? TGARLEColormap
-          : TGAColormap;
+        tga_info.image_type=compression == RLECompression ? TGARLEColormap :
+          TGAColormap;
         tga_info.colormap_type=1;
         tga_info.colormap_length=(unsigned short) image->colors;
-        tga_info.colormap_size=24;
+        if (image_info->depth == 5)
+          tga_info.colormap_size=16;
+        else
+          tga_info.colormap_size=24;
       }
+  value=GetImageArtifact(image,"tga:image-origin");
+  if (value != (const char *) NULL)
+    {
+      OrientationType
+        origin;
+
+      origin=(OrientationType) ParseCommandOption(MagickOrientationOptions,
+        MagickFalse,value);
+      if (origin == BottomRightOrientation || origin == TopRightOrientation)
+        tga_info.attributes|=(1UL << 4);
+      if (origin == TopLeftOrientation || origin == TopRightOrientation)
+        tga_info.attributes|=(1UL << 5);
+    }
   /*
     Write TGA header.
   */
@@ -791,24 +856,40 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
   if (tga_info.colormap_type != 0)
     {
       unsigned char
+        green,
         *targa_colormap;
 
       /*
         Dump colormap to file (blue, green, red byte order).
       */
       targa_colormap=(unsigned char *) AcquireQuantumMemory((size_t)
-        tga_info.colormap_length,3UL*sizeof(*targa_colormap));
+        tga_info.colormap_length,(tga_info.colormap_size/8)*sizeof(
+        *targa_colormap));
       if (targa_colormap == (unsigned char *) NULL)
         ThrowWriterException(ResourceLimitError,"MemoryAllocationFailed");
       q=targa_colormap;
       for (i=0; i < (ssize_t) image->colors; i++)
       {
-        *q++=ScaleQuantumToChar(image->colormap[i].blue);
-        *q++=ScaleQuantumToChar(image->colormap[i].green);
-        *q++=ScaleQuantumToChar(image->colormap[i].red);
+        if (image_info->depth == 5)
+          {
+            green=(unsigned char) ScaleQuantumToAny(image->colormap[i].green,
+              range);
+            *q++=((unsigned char) ScaleQuantumToAny(image->colormap[i].blue,
+              range)) | ((green & 0x07) << 5);
+            *q++=(((image->matte != MagickFalse) && (
+              (double) image->colormap[i].opacity < midpoint)) ? 0x80 : 0) |
+              ((unsigned char) ScaleQuantumToAny(image->colormap[i].red,
+              range) << 2) | ((green & 0x18) >> 3);
+          }
+        else
+          {
+            *q++=ScaleQuantumToChar(image->colormap[i].blue);
+            *q++=ScaleQuantumToChar(image->colormap[i].green);
+            *q++=ScaleQuantumToChar(image->colormap[i].red);
+          }
       }
-      (void) WriteBlob(image,(size_t) (3*tga_info.colormap_length),
-        targa_colormap);
+      (void) WriteBlob(image,(size_t) ((tga_info.colormap_size/8)*
+        tga_info.colormap_length),targa_colormap);
       targa_colormap=(unsigned char *) RelinquishMagickMemory(targa_colormap);
     }
   /*
@@ -835,21 +916,22 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
                 if (GetPixelIndex(indexes+i) != GetPixelIndex(indexes+(i-1)))
                   break;
               }
-            else if (tga_info.image_type == TGARLEMonochrome)
-              {
-                if (GetPixelLuma(image,p+i) != GetPixelLuma(image,p+(i-1)))
-                  break;
-              }
             else
-              {
-                if ((GetPixelBlue(p+i) != GetPixelBlue(p+(i-1))) ||
-                    (GetPixelGreen(p+i) != GetPixelGreen(p+(i-1))) ||
-                    (GetPixelRed(p+i) != GetPixelRed(p+(i-1))))
-                  break;
-                if ((image->matte != MagickFalse) &&
-                    (GetPixelAlpha(p+i) != GetPixelAlpha(p+(i-1))))
-                  break;
-              }
+              if (tga_info.image_type == TGARLEMonochrome)
+                {
+                  if (GetPixelLuma(image,p+i) != GetPixelLuma(image,p+(i-1)))
+                    break;
+                }
+              else
+                {
+                  if ((GetPixelBlue(p+i) != GetPixelBlue(p+(i-1))) ||
+                      (GetPixelGreen(p+i) != GetPixelGreen(p+(i-1))) ||
+                      (GetPixelRed(p+i) != GetPixelRed(p+(i-1))))
+                    break;
+                  if ((image->matte != MagickFalse) &&
+                      (GetPixelAlpha(p+i) != GetPixelAlpha(p+(i-1))))
+                    break;
+                }
             i++;
           }
           if (i < 3)
@@ -866,8 +948,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
                   (void) WriteBlobByte(image,(unsigned char) (--count));
                   while (count >= 0)
                   {
-                    WriteTGAPixel(image,tga_info.image_type,
-                      indexes-(count+1),p-(count+1));
+                    WriteTGAPixel(image,tga_info.image_type,indexes-(count+1),
+                      p-(count+1),range,midpoint);
                     count--;
                   }
                   count=0;
@@ -875,8 +957,8 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
             }
           if (i >= 3)
             {
-              WriteBlobByte(image,(unsigned char) ((i-1) | 0x80));
-              WriteTGAPixel(image,tga_info.image_type,indexes,p);
+              (void) WriteBlobByte(image,(unsigned char) ((i-1) | 0x80));
+              WriteTGAPixel(image,tga_info.image_type,indexes,p,range,midpoint);
               p+=i;
               indexes+=i;
             }
@@ -886,7 +968,7 @@ static MagickBooleanType WriteTGAImage(const ImageInfo *image_info,
     else
       {
         for (x=0; x < (ssize_t) image->columns; x++)
-          WriteTGAPixel(image,tga_info.image_type,indexes+x,p++);
+          WriteTGAPixel(image,tga_info.image_type,indexes+x,p++,range,midpoint);
       }
     if (image->previous == (Image *) NULL)
       {
