@@ -17,7 +17,7 @@
 %                                 March 2011                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -139,7 +139,8 @@ static MagickBooleanType IsWEBP(const unsigned char *magick,const size_t length)
 %
 */
 
-static inline uint32_t ReadWebPLSBWord(const unsigned char *restrict data)
+static inline uint32_t ReadWebPLSBWord(
+  const unsigned char *magick_restrict data)
 {
   register const unsigned char
     *p;
@@ -230,10 +231,10 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
     configure;
 
   WebPDecBuffer
-    *restrict webp_image = &configure.output;
+    *magick_restrict webp_image = &configure.output;
 
   WebPBitstreamFeatures
-    *restrict features = &configure.input;
+    *magick_restrict features = &configure.input;
 
   /*
     Open image file.
@@ -286,6 +287,12 @@ static Image *ReadWEBPImage(const ImageInfo *image_info,
           (void) CloseBlob(image);
           return(GetFirstImageInList(image));
         }
+      status=SetImageExtent(image,image->columns,image->rows);
+      if (status == MagickFalse)
+        {
+          InheritException(exception,&image->exception);
+          return(DestroyImageList(image));
+        }
       webp_status=WebPDecode(stream,length,&configure);
     }
   if (webp_status != VP8_STATUS_OK)
@@ -407,7 +414,7 @@ ModuleExport size_t RegisterWEBPImage(void)
     (WebPGetDecoderVersion() >> 0) & 0xff,WEBP_DECODER_ABI_VERSION);
 #endif
   entry->description=ConstantString("WebP Image Format");
-  entry->mime_type=ConstantString("image/x-webp");
+  entry->mime_type=ConstantString("image/webp");
   entry->adjoin=MagickFalse;
   entry->module=ConstantString("WEBP");
   entry->magick=(IsImageFormatHandler *) IsWEBP;
@@ -512,7 +519,7 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
     *pixel_info;
 
   register uint32_t
-    *restrict q;
+    *magick_restrict q;
 
   ssize_t
     y;
@@ -554,9 +561,8 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
   picture.use_argb=1;
   if (image->quality != UndefinedCompressionQuality)
     configure.quality=(float) image->quality;
-  else
-    if (image->quality >= 100)
-      configure.lossless=1;
+  if (image->quality >= 100)
+    configure.lossless=1;
   value=GetImageOption(image_info,"webp:lossless");
   if (value != (char *) NULL)
     configure.lossless=(int) ParseCommandOption(MagickBooleanOptions,
@@ -628,14 +634,14 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
   if (value != (char *) NULL)
     configure.partition_limit=StringToInteger(value);
 #if WEBP_DECODER_ABI_VERSION >= 0x0201
-  value=GetImageOption(image_info,"webp:low-memory");
-  if (value != (char *) NULL)
-    configure.low_memory=(int) ParseCommandOption(MagickBooleanOptions,
-      MagickFalse,value);
   value=GetImageOption(image_info,"webp:emulate-jpeg-size");
   if (value != (char *) NULL)
     configure.emulate_jpeg_size=(int) ParseCommandOption(MagickBooleanOptions,
       MagickFalse,value);
+  value=GetImageOption(image_info,"webp:low-memory");
+  if (value != (char *) NULL)
+    configure.low_memory=(int) ParseCommandOption(MagickBooleanOptions,
+      MagickFalse,value);
   value=GetImageOption(image_info,"webp:thread-level");
   if (value != (char *) NULL)
     configure.thread_level=StringToInteger(value);
@@ -658,7 +664,7 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       x;
@@ -749,6 +755,7 @@ static MagickBooleanType WriteWEBPImage(const ImageInfo *image_info,
       (void) ThrowMagickException(&image->exception,GetMagickModule(),
         CorruptImageError,(char *) message,"`%s'",image->filename);
     }
+  picture.argb=(uint32_t *) NULL;
   WebPPictureFree(&picture);
   pixel_info=RelinquishVirtualMemory(pixel_info);
   (void) CloseBlob(image);
