@@ -17,7 +17,7 @@
 %                                October 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -189,10 +189,10 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
       bounding_box;
 
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       x;
@@ -243,7 +243,7 @@ MagickExport RectangleInfo GetImageBoundingBox(const Image *image,
     }
   }
   image_view=DestroyCacheView(image_view);
-  if ((bounds.width == 0) || (bounds.height == 0))
+  if ((bounds.width == 0) && (bounds.height == 0))
     (void) ThrowMagickException(exception,GetMagickModule(),OptionWarning,
       "GeometryDoesNotContainImage","`%s'",image->filename);
   else
@@ -410,10 +410,10 @@ RestoreMSCWarning
           id = GetOpenMPThreadId();
 
         register const IndexPacket
-          *restrict indexes;
+          *magick_restrict indexes;
 
         register const PixelPacket
-          *restrict p;
+          *magick_restrict p;
 
         register ssize_t
           x;
@@ -486,10 +486,10 @@ RestoreMSCWarning
       id = GetOpenMPThreadId();
 
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       x;
@@ -574,14 +574,6 @@ RestoreMSCWarning
 %      a maximum of MAGICKCORE_QUANTUM_DEPTH.
 %
 */
-
-static inline double MagickMin(const double x,const double y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 MagickExport size_t GetImageQuantumDepth(const Image *image,
   const MagickBooleanType constrain)
 {
@@ -673,19 +665,17 @@ MagickExport ImageType GetImageType(const Image *image,ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%     I s G r a y I m a g e                                                   %
+%     I d e n t i f y I m a g e G r a y                                       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsGrayImage() returns MagickTrue if all the pixels in the image have the
-%  same red, green, and blue intensities.
+%  either 0 or QuantumRange. Otherwise undefined is returned.
 %
-%  The format of the IsGrayImage method is:
+%  The format of the IdentifyImageGray method is:
 %
-%      MagickBooleanType IsGrayImage(const Image *image,
-%        ExceptionInfo *exception)
+%      ImageType IdentifyImageGray(const Image *image,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -694,7 +684,7 @@ MagickExport ImageType GetImageType(const Image *image,ExceptionInfo *exception)
 %    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport MagickBooleanType IsGrayImage(const Image *image,
+MagickExport ImageType IdentifyImageGray(const Image *image,
   ExceptionInfo *exception)
 {
   CacheView
@@ -718,10 +708,9 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if ((image->type == BilevelType) || (image->type == GrayscaleType) ||
       (image->type == GrayscaleMatteType))
-    return(MagickTrue);
-  if ((IsGrayColorspace(image->colorspace) == MagickFalse) &&
-      (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse))
-    return(MagickFalse);
+    return(image->type);
+  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
+    return(UndefinedType);
   type=BilevelType;
   image_view=AcquireVirtualCacheView(image,exception);
   for (y=0; y < (ssize_t) image->rows; y++)
@@ -731,12 +720,12 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      if (IsGrayPixel(p) == MagickFalse)
+      if (IsPixelGray(p) == MagickFalse)
         {
           type=UndefinedType;
           break;
         }
-      if ((type == BilevelType) && (IsMonochromePixel(p) == MagickFalse))
+      if ((type == BilevelType) && (IsPixelMonochrome(p) == MagickFalse))
         type=GrayscaleType;
       p++;
     }
@@ -744,15 +733,9 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
       break;
   }
   image_view=DestroyCacheView(image_view);
-  if (type == UndefinedType)
-    return(MagickFalse);
-  ((Image *) image)->colorspace=GRAYColorspace;
-  if (SyncImagePixelCache((Image *) image,exception) == MagickFalse)
-    return(MagickFalse);
-  ((Image *) image)->type=type;
   if ((type == GrayscaleType) && (image->matte != MagickFalse))
-    ((Image *) image)->type=GrayscaleMatteType;
-  return(MagickTrue);
+    type=GrayscaleMatteType;
+  return(type);
 }
 
 /*
@@ -760,19 +743,19 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-%   I s M o n o c h r o m e I m a g e                                         %
+%   I d e n t i f y I m a g e M o n o c h r o m e                             %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  IsMonochromeImage() returns MagickTrue if all the pixels in the image have
-%  the same red, green, and blue intensities and the intensity is either
+%  IdentifyImageMonochrome() returns MagickTrue if all the pixels in the image
+%  have the same red, green, and blue intensities and the intensity is either
 %  0 or QuantumRange.
 %
-%  The format of the IsMonochromeImage method is:
+%  The format of the IdentifyImageMonochrome method is:
 %
-%      MagickBooleanType IsMonochromeImage(const Image *image,
+%      MagickBooleanType IdentifyImageMonochrome(const Image *image,
 %        ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
@@ -782,7 +765,7 @@ MagickExport MagickBooleanType IsGrayImage(const Image *image,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
+MagickExport MagickBooleanType IdentifyImageMonochrome(const Image *image,
   ExceptionInfo *exception)
 {
   CacheView
@@ -791,9 +774,6 @@ MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
   ImageType
     type;
 
-  MagickBooleanType
-    status;
-
   register ssize_t
     x;
 
@@ -809,8 +789,7 @@ MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
   if (image->type == BilevelType)
     return(MagickTrue);
-  if ((IsGrayColorspace(image->colorspace) == MagickFalse) &&
-      (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse))
+  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
     return(MagickFalse);
   type=BilevelType;
   image_view=AcquireVirtualCacheView(image,exception);
@@ -821,7 +800,7 @@ MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
       break;
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      if (IsMonochromePixel(p) == MagickFalse)
+      if (IsPixelMonochrome(p) == MagickFalse)
         {
           type=UndefinedType;
           break;
@@ -832,14 +811,145 @@ MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
       break;
   }
   image_view=DestroyCacheView(image_view);
-  if (type == UndefinedType)
-    return(MagickFalse);
-  ((Image *) image)->colorspace=GRAYColorspace;
-  status=SyncImagePixelCache((Image *) image,exception);
-  if (SyncImagePixelCache((Image *) image,exception) == MagickFalse)
-    return(status);
-  ((Image *) image)->type=type;
-  return(MagickTrue);
+  if (type == BilevelType)
+    return(MagickTrue);
+  return(MagickFalse);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   I d e n t i f y I m a g e T y p e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IdentifyImageType() returns the potential type of image:
+%
+%        Bilevel         Grayscale        GrayscaleMatte
+%        Palette         PaletteMatte     TrueColor
+%        TrueColorMatte  ColorSeparation  ColorSeparationMatte
+%
+%  To ensure the image type matches its potential, use SetImageType():
+%
+%    (void) SetImageType(image,IdentifyImageType(image,exception),exception);
+%
+%  The format of the IdentifyImageType method is:
+%
+%      ImageType IdentifyImageType(const Image *image,ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+MagickExport ImageType IdentifyImageType(const Image *image,
+  ExceptionInfo *exception)
+{
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  if (image->colorspace == CMYKColorspace)
+    {
+      if (image->matte == MagickFalse)
+        return(ColorSeparationType);
+      return(ColorSeparationMatteType);
+    }
+  if (IdentifyImageMonochrome(image,exception) != MagickFalse)
+    return(BilevelType);
+  if (IdentifyImageGray(image,exception) != UndefinedType)
+    {
+      if (image->matte != MagickFalse)
+        return(GrayscaleMatteType);
+      return(GrayscaleType);
+    }
+  if (IdentifyPaletteImage(image,exception) != MagickFalse)
+    {
+      if (image->matte != MagickFalse)
+        return(PaletteMatteType);
+      return(PaletteType);
+    }
+  if (image->matte != MagickFalse)
+    return(TrueColorMatteType);
+  return(TrueColorType);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%     I s G r a y I m a g e                                                   %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IsGrayImage() returns MagickTrue if the type of the image is grayscale or
+%  bi-level.
+%
+%  The format of the IsGrayImage method is:
+%
+%      MagickBooleanType IsGrayImage(const Image *image,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+MagickExport MagickBooleanType IsGrayImage(const Image *image,
+  ExceptionInfo *exception)
+{
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if ((image->type == BilevelType) || (image->type == GrayscaleType) ||
+      (image->type == GrayscaleMatteType))
+    return(MagickTrue);
+  return(MagickFalse);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   I s M o n o c h r o m e I m a g e                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IsMonochromeImage() returns MagickTrue if type of the image is bi-level.
+%
+%  The format of the IsMonochromeImage method is:
+%
+%      MagickBooleanType IsMonochromeImage(const Image *image,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows:
+%
+%    o image: the image.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+MagickExport MagickBooleanType IsMonochromeImage(const Image *image,
+  ExceptionInfo *exception)
+{
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->type == BilevelType)
+    return(MagickTrue);
+  return(MagickFalse);
 }
 
 /*
@@ -940,19 +1050,6 @@ MagickExport MagickBooleanType IsOpaqueImage(const Image *image,
 %
 */
 
-static inline Quantum ClampPixel(const MagickRealType value)
-{
-#if !defined(MAGICKCORE_HDRI_SUPPORT)
-  return((Quantum) value);
-#else
-  if (value < 0.0f)
-    return(0.0f);
-  if (value >= (MagickRealType) QuantumRange)
-    return((Quantum) QuantumRange);
-  return(value);
-#endif
-}
-
 MagickExport MagickBooleanType SetImageDepth(Image *image,
   const size_t depth)
 {
@@ -1045,7 +1142,7 @@ RestoreMSCWarning
           x;
 
         register PixelPacket
-          *restrict q;
+          *magick_restrict q;
 
         if (status == MagickFalse)
           continue;
@@ -1095,7 +1192,7 @@ RestoreMSCWarning
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -1189,31 +1286,29 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
   {
     case BilevelType:
     {
-      if (IsGrayImage(image,&image->exception) == MagickFalse)
-        status=TransformImageColorspace(image,GRAYColorspace);
-      (void) NormalizeImage(image);
-      if (IsMonochromeImage(image,&image->exception) == MagickFalse)
+      if (SetImageMonochrome(image,&image->exception) == MagickFalse)
         {
+          status=TransformImageColorspace(image,GRAYColorspace);
+          (void) NormalizeImage(image);
           quantize_info=AcquireQuantizeInfo(image_info);
           quantize_info->number_colors=2;
           quantize_info->colorspace=GRAYColorspace;
           status=QuantizeImage(quantize_info,image);
           quantize_info=DestroyQuantizeInfo(quantize_info);
         }
-      image->colors=2;
       image->matte=MagickFalse;
       break;
     }
     case GrayscaleType:
     {
-      if (IsGrayImage(image,&image->exception) == MagickFalse)
+      if (SetImageGray(image,&image->exception) == MagickFalse)
         status=TransformImageColorspace(image,GRAYColorspace);
       image->matte=MagickFalse;
       break;
     }
     case GrayscaleMatteType:
     {
-      if (IsGrayImage(image,&image->exception) == MagickFalse)
+      if (SetImageGray(image,&image->exception) == MagickFalse)
         status=TransformImageColorspace(image,GRAYColorspace);
       if (image->matte == MagickFalse)
         (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
@@ -1281,7 +1376,7 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
       if (image->colorspace != CMYKColorspace)
         {
           if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-            status=TransformImageColorspace(image,sRGBColorspace);
+            (void) TransformImageColorspace(image,sRGBColorspace);
           status=TransformImageColorspace(image,CMYKColorspace);
         }
       if (image->storage_class != DirectClass)
@@ -1294,7 +1389,7 @@ MagickExport MagickBooleanType SetImageType(Image *image,const ImageType type)
       if (image->colorspace != CMYKColorspace)
         {
           if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-            status=TransformImageColorspace(image,sRGBColorspace);
+            (void) TransformImageColorspace(image,sRGBColorspace);
           status=TransformImageColorspace(image,CMYKColorspace);
         }
       if (image->storage_class != DirectClass)
