@@ -17,7 +17,7 @@
 %                               December 2003                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -159,10 +159,10 @@ MagickExport Image *CombineImages(const Image *image,const ChannelType channel,
       *pixels;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     register ssize_t
       x;
@@ -398,7 +398,7 @@ MagickExport MagickBooleanType SeparateImageChannel(Image *image,
   if (SetImageStorageClass(image,DirectClass) == MagickFalse)
     return(MagickFalse);
   if (channel == GrayChannels)
-    image->matte=MagickTrue;
+    (void) SetImageAlphaChannel(image,OpaqueAlphaChannel);
   /*
     Separate image channels.
   */
@@ -413,10 +413,10 @@ MagickExport MagickBooleanType SeparateImageChannel(Image *image,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     register ssize_t
       x;
@@ -527,8 +527,10 @@ MagickExport MagickBooleanType SeparateImageChannel(Image *image,
   }
   image_view=DestroyCacheView(image_view);
   if (channel != GrayChannels)
-    image->matte=MagickFalse;
-  (void) SetImageColorspace(image,GRAYColorspace);
+    {
+      image->matte=MagickFalse;
+      (void) SetImageColorspace(image,GRAYColorspace);
+    }
   return(status);
 }
 
@@ -681,7 +683,7 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
       for (y=0; y < (ssize_t) image->rows; y++)
       {
         register PixelPacket
-          *restrict q;
+          *magick_restrict q;
 
         register ssize_t
           x;
@@ -698,12 +700,12 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
         for (x=0; x < (ssize_t) image->columns; x++)
         {
           double
-            alpha;
+            gamma;
 
-          alpha=QuantumScale*GetPixelAlpha(q);
-          SetPixelRed(q,ClampToQuantum(alpha*GetPixelRed(q)));
-          SetPixelGreen(q,ClampToQuantum(alpha*GetPixelGreen(q)));
-          SetPixelBlue(q,ClampToQuantum(alpha*GetPixelBlue(q)));
+          gamma=QuantumScale*GetPixelAlpha(q);
+          SetPixelRed(q,ClampToQuantum(gamma*GetPixelRed(q)));
+          SetPixelGreen(q,ClampToQuantum(gamma*GetPixelGreen(q)));
+          SetPixelBlue(q,ClampToQuantum(gamma*GetPixelBlue(q)));
           q++;
         }
         if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
@@ -752,10 +754,10 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
       for (y=0; y < (ssize_t) image->rows; y++)
       {
         register IndexPacket
-          *restrict indexes;
+          *magick_restrict indexes;
 
         register PixelPacket
-          *restrict q;
+          *magick_restrict q;
 
         register ssize_t
           x;
@@ -834,7 +836,7 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
       for (y=0; y < (ssize_t) image->rows; y++)
       {
         register PixelPacket
-          *restrict q;
+          *magick_restrict q;
 
         register ssize_t
           x;
@@ -851,13 +853,14 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
         for (x=0; x < (ssize_t) image->columns; x++)
         {
           double
-            alpha;
+            alpha,
+            gamma;
 
           alpha=QuantumScale*GetPixelAlpha(q);
-          alpha=PerceptibleReciprocal(alpha);
-          SetPixelRed(q,ClampToQuantum(alpha*GetPixelRed(q)));
-          SetPixelGreen(q,ClampToQuantum(alpha*GetPixelGreen(q)));
-          SetPixelBlue(q,ClampToQuantum(alpha*GetPixelBlue(q)));
+          gamma=PerceptibleReciprocal(alpha);
+          SetPixelRed(q,ClampToQuantum(gamma*GetPixelRed(q)));
+          SetPixelGreen(q,ClampToQuantum(gamma*GetPixelGreen(q)));
+          SetPixelBlue(q,ClampToQuantum(gamma*GetPixelBlue(q)));
           q++;
         }
         if (SyncCacheViewAuthenticPixels(image_view,exception) == MagickFalse)
@@ -897,6 +900,7 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
         NULL,&background);
       if (image->colorspace == CMYKColorspace)
         ConvertRGBToCMYK(&background);
+      (void) ResetMagickMemory(&pixel,0,sizeof(pixel));
       index=0;
       SetPixelPacket(image,&background,&pixel,&index);
       image_view=AcquireAuthenticCacheView(image,exception);
@@ -907,10 +911,10 @@ MagickExport MagickBooleanType SetImageAlphaChannel(Image *image,
       for (y=0; y < (ssize_t) image->rows; y++)
       {
         register IndexPacket
-          *restrict indexes;
+          *magick_restrict indexes;
 
         register PixelPacket
-          *restrict q;
+          *magick_restrict q;
 
         register ssize_t
           x;
