@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 
   MagickCore image channel methods.
 */
-#ifndef _MAGICKCORE_CHANNEL_H
-#define _MAGICKCORE_CHANNEL_H
+#ifndef MAGICKCORE_CHANNEL_H
+#define MAGICKCORE_CHANNEL_H
 
 #include "magick/image.h"
 
