@@ -17,7 +17,7 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -97,16 +97,21 @@ static const CoderMapInfo
     { "B", "RAW" },
     { "BRF", "BRAILLE" },
     { "BGRA", "BGR" },
+    { "BGRO", "BGR" },
     { "CMYKA", "CMYK" },
     { "C", "RAW" },
     { "CAL", "CALS" },
     { "CANVAS", "XC" },
+    { "CMYKA", "CMYK" },
     { "CR2", "DNG" },
     { "CRW", "DNG" },
     { "CUR", "ICON" },
+    { "DATA", "INLINE" },
     { "DCR", "DNG" },
     { "DCX", "PCX" },
     { "DFONT", "TTF" },
+    { "DXT1", "DDS" },
+    { "DXT5", "DDS" },
     { "EPDF", "PDF" },
     { "EPI", "PS" },
     { "EPS2", "PS2" },
@@ -123,11 +128,16 @@ static const CoderMapInfo
     { "FTP", "URL" },
     { "FTS", "FITS" },
     { "G3", "FAX" },
+    { "G4", "FAX" },
     { "GIF87", "GIF" },
     { "G", "RAW" },
     { "GRANITE", "MAGICK" },
     { "GROUP4", "TIFF" },
     { "GV", "DOT" },
+    { "HTM", "HTML" },
+    { "ICB", "TGA" },
+    { "ICO", "ICON" },
+    { "IIQ", "DNG" },
     { "K25", "DNG" },
     { "KDC", "DNG" },
     { "H", "MAGICK" },
@@ -143,6 +153,7 @@ static const CoderMapInfo
     { "IPTCTEXT", "META" },
     { "IPTCWTEXT", "META" },
     { "ISOBRL", "BRAILLE" },
+    { "ISOBRL6", "BRAILLE" },
     { "JBG", "JBIG" },
     { "JNG", "PNG" },
     { "JPC", "JP2" },
@@ -150,13 +161,22 @@ static const CoderMapInfo
     { "JPM", "JP2" },
     { "J2C", "JP2" },
     { "J2K", "JP2" },
+    { "JNG", "PNG" },
+    { "JPE", "JPEG" },
     { "JPG", "JPEG" },
+    { "JPM", "JP2" },
+    { "JPS", "JPEG" },
+    { "JPT", "JP2" },
     { "JPX", "JP2" },
     { "K", "RAW" },
+    { "K25", "DNG" },
+    { "KDC", "DNG" },
     { "LOGO", "MAGICK" },
+    { "M", "RAW" },
     { "M2V", "MPEG" },
     { "M4V", "MPEG" },
-    { "M", "RAW" },
+    { "MEF", "DNG" },
+    { "MKV", "MPEG" },
     { "MNG", "PNG" },
     { "MOV", "MPEG" },
     { "MP4", "MPEG" },
@@ -176,6 +196,7 @@ static const CoderMapInfo
     { "PAM", "PNM" },
     { "PBM", "PNM" },
     { "PCDS", "PCD" },
+    { "PCT", "PICT" },
     { "PDFA", "PDF" },
     { "PEF", "DNG" },
     { "PEF", "DNG" },
@@ -187,12 +208,16 @@ static const CoderMapInfo
     { "PICON", "XPM" },
     { "PJPEG", "JPEG" },
     { "PM", "XPM" },
+    { "PNG00", "PNG" },
     { "PNG24", "PNG" },
     { "PNG32", "PNG" },
+    { "PNG48", "PNG" },
+    { "PNG64", "PNG" },
     { "PNG8", "PNG" },
     { "PPM", "PNM" },
     { "PSB", "PSD" },
     { "PTIF", "TIFF" },
+    { "R", "RAW" },
     { "RADIAL-GRADIENT", "GRADIENT" },
     { "RAF", "DNG" },
     { "RAS", "SUN" },
@@ -200,7 +225,6 @@ static const CoderMapInfo
     { "RGBA", "RGB" },
     { "RGBO", "RGB" },
     { "RMF", "DNG" },
-    { "R", "RAW" },
     { "ROSE", "MAGICK" },
     { "RW2", "DNG" },
     { "SHTML", "HTML" },
@@ -214,6 +238,7 @@ static const CoderMapInfo
     { "TIF", "TIFF" },
     { "TTC", "TTF" },
     { "UBRL", "BRAILLE" },
+    { "UBRL6", "BRAILLE" },
     { "VDA", "TGA" },
     { "VST", "TGA" },
     { "WIZARD", "MAGICK" },
@@ -225,9 +250,6 @@ static const CoderMapInfo
     { "X3f", "DNG" },
     { "XMP", "META" },
     { "XTRNARRAY", "XTRN" },
-    { "XTRNBLOB", "XTRN" },
-    { "XTRNFILE", "XTRN" },
-    { "XTRNIMAGE", "XTRN" },
     { "XV", "VIFF" },
     { "Y", "RAW" },
     { "YCbCrA", "YCbCr" }
@@ -295,12 +317,6 @@ static void *DestroyCoderNode(void *coder_info)
 static SplayTreeInfo *AcquireCoderCache(const char *filename,
   ExceptionInfo *exception)
 {
-  const StringInfo
-    *option;
-
-  LinkedListInfo
-    *options;
-
   MagickStatusType
     status;
 
@@ -308,25 +324,35 @@ static SplayTreeInfo *AcquireCoderCache(const char *filename,
     i;
 
   SplayTreeInfo
-    *coder_cache;
+    *cache;
 
   /*
     Load external coder map.
   */
-  coder_cache=NewSplayTree(CompareSplayTreeString,RelinquishMagickMemory,
+  cache=NewSplayTree(CompareSplayTreeString,RelinquishMagickMemory,
     DestroyCoderNode);
-  if (coder_cache == (SplayTreeInfo *) NULL)
+  if (cache == (SplayTreeInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   status=MagickTrue;
-  options=GetConfigureOptions(filename,exception);
-  option=(const StringInfo *) GetNextValueInLinkedList(options);
-  while (option != (const StringInfo *) NULL)
+#if !defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
   {
-    status&=LoadCoderCache(coder_cache,(const char *)
-      GetStringInfoDatum(option),GetStringInfoPath(option),0,exception);
+    const StringInfo
+      *option;
+
+    LinkedListInfo
+      *options;
+
+    options=GetConfigureOptions(filename,exception);
     option=(const StringInfo *) GetNextValueInLinkedList(options);
+    while (option != (const StringInfo *) NULL)
+    {
+      status&=LoadCoderCache(cache,(const char *)
+        GetStringInfoDatum(option),GetStringInfoPath(option),0,exception);
+      option=(const StringInfo *) GetNextValueInLinkedList(options);
+    }
+    options=DestroyConfigureOptions(options);
   }
-  options=DestroyConfigureOptions(options);
+#endif
   /*
     Load built-in coder map.
   */
@@ -352,13 +378,13 @@ static SplayTreeInfo *AcquireCoderCache(const char *filename,
     coder_info->name=(char *) p->name;
     coder_info->exempt=MagickTrue;
     coder_info->signature=MagickSignature;
-    status&=AddValueToSplayTree(coder_cache,ConstantString(coder_info->magick),
+    status&=AddValueToSplayTree(cache,ConstantString(coder_info->magick),
       coder_info);
     if (status == MagickFalse)
       (void) ThrowMagickException(exception,GetMagickModule(),
         ResourceLimitError,"MemoryAllocationFailed","`%s'",coder_info->name);
   }
-  return(coder_cache);
+  return(cache);
 }
 
 /*
@@ -750,7 +776,7 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   L o a d C o d e r L i s t                                                 %
++   L o a d C o d e r C a c h e                                               %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -761,9 +787,8 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
 %
 %  The format of the LoadCoderCache coder is:
 %
-%      MagickBooleanType LoadCoderCache(SplayTreeInfo *coder_cache,
-%        const char *xml,const char *filename,const size_t depth,
-%        ExceptionInfo *exception)
+%      MagickBooleanType LoadCoderCache(SplayTreeInfo *cache,const char *xml,
+%        const char *filename,const size_t depth,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -776,9 +801,8 @@ MagickExport MagickBooleanType ListCoderInfo(FILE *file,
 %    o exception: return any errors or warnings in this structure.
 %
 */
-static MagickBooleanType LoadCoderCache(SplayTreeInfo *coder_cache,
-  const char *xml,const char *filename,const size_t depth,
-  ExceptionInfo *exception)
+static MagickBooleanType LoadCoderCache(SplayTreeInfo *cache,const char *xml,
+  const char *filename,const size_t depth,ExceptionInfo *exception)
 {
   char
     keyword[MaxTextExtent],
@@ -793,6 +817,9 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *coder_cache,
   MagickStatusType
     status;
 
+  size_t
+    extent;
+
   /*
     Load the coder map file.
   */
@@ -803,12 +830,13 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *coder_cache,
   status=MagickTrue;
   coder_info=(CoderInfo *) NULL;
   token=AcquireString(xml);
+  extent=strlen(token)+MaxTextExtent;
   for (q=(char *) xml; *q != '\0'; )
   {
     /*
       Interpret XML.
     */
-    GetMagickToken(q,&q,token);
+    GetNextToken(q,&q,extent,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
@@ -818,7 +846,7 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *coder_cache,
           Doctype element.
         */
         while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
-          GetMagickToken(q,&q,token);
+          GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
@@ -827,7 +855,7 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *coder_cache,
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetMagickToken(q,&q,token);
+          GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -838,10 +866,10 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *coder_cache,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetMagickToken(q,&q,token);
+          GetNextToken(q,&q,extent,token);
           if (*token != '=')
             continue;
-          GetMagickToken(q,&q,token);
+          GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > 200)
@@ -864,7 +892,7 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *coder_cache,
                   xml=FileToXML(path,~0UL);
                   if (xml != (char *) NULL)
                     {
-                      status&=LoadCoderCache(coder_cache,xml,path,depth+1,
+                      status&=LoadCoderCache(cache,xml,path,depth+1,
                         exception);
                       xml=(char *) RelinquishMagickMemory(xml);
                     }
@@ -889,9 +917,10 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *coder_cache,
       }
     if (coder_info == (CoderInfo *) NULL)
       continue;
-    if (LocaleCompare(keyword,"/>") == 0)
+    if ((LocaleCompare(keyword,"/>") == 0) ||
+        (LocaleCompare(keyword,"</policy>") == 0))
       {
-        status=AddValueToSplayTree(coder_cache,ConstantString(
+        status=AddValueToSplayTree(cache,ConstantString(
           coder_info->magick),coder_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
@@ -900,11 +929,11 @@ static MagickBooleanType LoadCoderCache(SplayTreeInfo *coder_cache,
         coder_info=(CoderInfo *) NULL;
         continue;
       }
-    GetMagickToken(q,(const char **) NULL,token);
+    GetNextToken(q,(const char **) NULL,extent,token);
     if (*token != '=')
       continue;
-    GetMagickToken(q,&q,token);
-    GetMagickToken(q,&q,token);
+    GetNextToken(q,&q,extent,token);
+    GetNextToken(q,&q,extent,token);
     switch (*keyword)
     {
       case 'M':
