@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
 
   You may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 
   MagickCore image composite private methods.
 */
-#ifndef _MAGICKCORE_COMPOSITE_PRIVATE_H
-#define _MAGICKCORE_COMPOSITE_PRIVATE_H
+#ifndef MAGICKCORE_COMPOSITE_PRIVATE_H
+#define MAGICKCORE_COMPOSITE_PRIVATE_H
 
 #include "magick/color.h"
 #include "magick/image.h"
