@@ -17,7 +17,7 @@
 %                               October 1998                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -428,7 +428,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
         if ((image_info->endian == UndefinedEndian) &&
             (GetMagickRawSupport(magick_info) != MagickFalse))
           {
-            size_t
+            unsigned long
               lsb_first;
 
             lsb_first=1;
@@ -495,6 +495,9 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
     }
   else
     {
+      MagickBooleanType
+        status;
+
       delegate_info=GetDelegateInfo(read_info->magick,(char *) NULL,exception);
       if (delegate_info == (const DelegateInfo *) NULL)
         {
@@ -520,13 +523,14 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       *read_info->filename='\0';
       if (GetDelegateThreadSupport(delegate_info) == MagickFalse)
         LockSemaphoreInfo(delegate_info->semaphore);
-      (void) InvokeDelegate(read_info,image,read_info->magick,(char *) NULL,
+      status=InvokeDelegate(read_info,image,read_info->magick,(char *) NULL,
         exception);
       if (GetDelegateThreadSupport(delegate_info) == MagickFalse)
         UnlockSemaphoreInfo(delegate_info->semaphore);
       image=DestroyImageList(image);
       read_info->temporary=MagickTrue;
-      (void) SetImageInfo(read_info,0,exception);
+      if (status != MagickFalse)
+        (void) SetImageInfo(read_info,0,exception);
       magick_info=GetMagickInfo(read_info->magick,exception);
       if ((magick_info == (const MagickInfo *) NULL) ||
           (GetImageDecoder(magick_info) == (DecodeImageHandler *) NULL))
@@ -582,9 +586,6 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
           image=GetFirstImageInList(clones);
         }
     }
-  if (GetBlobError(image) != MagickFalse)
-    ThrowFileException(exception,FileOpenError,
-      "AnErrorHasOccurredReadingFromFile",read_info->filename);
   for (next=image; next != (Image *) NULL; next=GetNextImageInList(next))
   {
     char
@@ -624,9 +625,11 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       {
         geometry_info.rho=next->x_resolution;
         geometry_info.sigma=1.0;
-        flags=ParseGeometry(value,&geometry_info);
+        (void) ParseGeometry(value,&geometry_info);
         if (geometry_info.sigma != 0)
           next->x_resolution=geometry_info.rho/geometry_info.sigma;
+        if (strchr(value,',') != (char *) NULL)
+          next->x_resolution=geometry_info.rho+geometry_info.sigma/1000.0;
         (void) DeleteImageProperty(next,"exif:XResolution");
       }
     value=GetImageProperty(next,"exif:YResolution");
@@ -634,9 +637,11 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       {
         geometry_info.rho=next->y_resolution;
         geometry_info.sigma=1.0;
-        flags=ParseGeometry(value,&geometry_info);
+        (void) ParseGeometry(value,&geometry_info);
         if (geometry_info.sigma != 0)
           next->y_resolution=geometry_info.rho/geometry_info.sigma;
+        if (strchr(value,',') != (char *) NULL)
+          next->y_resolution=geometry_info.rho+geometry_info.sigma/1000.0;
         (void) DeleteImageProperty(next,"exif:YResolution");
       }
     value=GetImageProperty(next,"tiff:ResolutionUnit");
@@ -644,7 +649,8 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
       value=GetImageProperty(next,"exif:ResolutionUnit");
     if (value != (char *) NULL)
       {
-        next->units=(ResolutionType) (StringToLong(value)-1);
+        next->units=(ResolutionType) ParseCommandOption(MagickResolutionOptions,
+          MagickFalse,value);
         (void) DeleteImageProperty(next,"exif:ResolutionUnit");
         (void) DeleteImageProperty(next,"tiff:ResolutionUnit");
       }
@@ -700,7 +706,7 @@ MagickExport Image *ReadImage(const ImageInfo *image_info,
                   Image
                     *size_image;
 
-                  flags=ParseRegionGeometry(next,read_info->extract,&geometry,
+                  (void) ParseRegionGeometry(next,read_info->extract,&geometry,
                     exception);
                   size_image=ResizeImage(next,geometry.width,geometry.height,
                     next->filter,next->blur,exception);
@@ -812,9 +818,11 @@ MagickExport Image *ReadImages(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",
       image_info->filename);
   assert(exception != (ExceptionInfo *) NULL);
-  (void) InterpretImageFilename(image_info,(Image *) NULL,image_info->filename,
-    (int) image_info->scene,filename);
-  if (LocaleCompare(filename,image_info->filename) != 0)
+  read_info=CloneImageInfo(image_info);
+  *read_info->magick='\0';
+  (void) InterpretImageFilename(read_info,(Image *) NULL,read_info->filename,
+    (int) read_info->scene,filename);
+  if (LocaleCompare(filename,read_info->filename) != 0)
     {
       ExceptionInfo
         *sans;
@@ -826,7 +834,6 @@ MagickExport Image *ReadImages(const ImageInfo *image_info,
       /*
         Images of the form image-%d.png[1-5].
       */
-      read_info=CloneImageInfo(image_info);
       sans=AcquireExceptionInfo();
       (void) SetImageInfo(read_info,0,sans);
       sans=DestroyExceptionInfo(sans);
@@ -850,7 +857,9 @@ MagickExport Image *ReadImages(const ImageInfo *image_info,
       read_info=DestroyImageInfo(read_info);
       return(images);
     }
-  return(ReadImage(image_info,exception));
+  image=ReadImage(read_info,exception);
+  read_info=DestroyImageInfo(read_info);
+  return(image);
 }
 
 /*
@@ -1001,9 +1010,9 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
   sans_exception=AcquireExceptionInfo();
   write_info=CloneImageInfo(image_info);
   (void) CopyMagickString(write_info->filename,image->filename,MaxTextExtent);
+  (void) SetImageInfo(write_info,1,sans_exception);
   if (*write_info->magick == '\0')
     (void) CopyMagickString(write_info->magick,image->magick,MaxTextExtent);
-  (void) SetImageInfo(write_info,1,sans_exception);
   if (LocaleCompare(write_info->magick,"clipmask") == 0)
     {
       if (image->clip_mask == (Image *) NULL)
@@ -1028,7 +1037,7 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
       ThrowBinaryException(PolicyError,"NotAuthorized",filename);
     }
   /*
-    Call appropriate image reader based on image type.
+    Call appropriate image writer based on image type.
   */
   magick_info=GetMagickInfo(write_info->magick,sans_exception);
   sans_exception=DestroyExceptionInfo(sans_exception);
@@ -1040,7 +1049,7 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
         if ((image_info->endian == UndefinedEndian) &&
             (GetMagickRawSupport(magick_info) != MagickFalse))
           {
-            size_t
+            unsigned long
               lsb_first;
 
             lsb_first=1;
@@ -1187,9 +1196,6 @@ MagickExport MagickBooleanType WriteImage(const ImageInfo *image_info,
             }
         }
     }
-  if (GetBlobError(image) != MagickFalse)
-    ThrowFileException(exception,FileOpenError,
-      "AnErrorHasOccurredWritingToFile",image->filename);
   if (temporary != MagickFalse)
     {
       /*
@@ -1286,20 +1292,28 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",images->filename);
   assert(exception != (ExceptionInfo *) NULL);
   write_info=CloneImageInfo(image_info);
+  *write_info->magick='\0';
   images=GetFirstImageInList(images);
   if (filename != (const char *) NULL)
     for (p=images; p != (Image *) NULL; p=GetNextImageInList(p))
       (void) CopyMagickString(p->filename,filename,MaxTextExtent);
   (void) CopyMagickString(write_info->filename,images->filename,MaxTextExtent);
-  if (*write_info->magick == '\0')
-    (void) CopyMagickString(write_info->magick,images->magick,MaxTextExtent);
   sans_exception=AcquireExceptionInfo();
   (void) SetImageInfo(write_info,(unsigned int) GetImageListLength(images),
     sans_exception);
   sans_exception=DestroyExceptionInfo(sans_exception);
+  if (*write_info->magick == '\0')
+    (void) CopyMagickString(write_info->magick,images->magick,MaxTextExtent);
   p=images;
   for ( ; GetNextImageInList(p) != (Image *) NULL; p=GetNextImageInList(p))
-    if (p->scene >= GetNextImageInList(p)->scene)
+  {
+    register Image
+      *next;
+
+    next=GetNextImageInList(p);
+    if (next == (Image *) NULL)
+      break;
+    if (p->scene >= next->scene)
       {
         register ssize_t
           i;
@@ -1312,6 +1326,7 @@ MagickExport MagickBooleanType WriteImages(const ImageInfo *image_info,
           p->scene=(size_t) i++;
         break;
       }
+  }
   /*
     Write images.
   */
