@@ -17,7 +17,7 @@
 %                                   July 1992                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -184,7 +184,6 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     accentuate,
     border,
     highlight,
-    interior,
     matte,
     shadow,
     trough;
@@ -211,9 +210,9 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   if ((frame_info->outer_bevel < 0) || (frame_info->inner_bevel < 0))
     ThrowImageException(OptionError,"FrameIsLessThanImageSize");
   bevel_width=(size_t) (frame_info->outer_bevel+frame_info->inner_bevel);
-  width=frame_info->width-frame_info->x-bevel_width;
-  height=frame_info->height-frame_info->y-bevel_width;
-  if ((width < image->columns) || (height < image->rows))
+  x=(ssize_t) frame_info->width-frame_info->x-bevel_width;
+  y=(ssize_t) frame_info->height-frame_info->y-bevel_width;
+  if ((x < (ssize_t) image->columns) || (y < (ssize_t) image->rows))
     ThrowImageException(OptionError,"FrameIsLessThanImageSize");
   /*
     Initialize framed image attributes.
@@ -243,9 +242,6 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   /*
     Initialize 3D effects color.
   */
-  GetMagickPixelPacket(frame_image,&interior);
-  SetMagickPixelPacket(frame_image,&image->border_color,(IndexPacket *) NULL,
-    &interior);
   GetMagickPixelPacket(frame_image,&matte);
   matte.colorspace=sRGBColorspace;
   SetMagickPixelPacket(frame_image,&image->matte_color,(IndexPacket *) NULL,
@@ -282,7 +278,6 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   trough.opacity=matte.opacity;
   if (image->colorspace == CMYKColorspace)
     {
-      ConvertRGBToCMYK(&interior);
       ConvertRGBToCMYK(&matte);
       ConvertRGBToCMYK(&border);
       ConvertRGBToCMYK(&accentuate);
@@ -299,13 +294,13 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   if (height != 0)
     {
       register IndexPacket
-        *restrict frame_indexes;
+        *magick_restrict frame_indexes;
 
       register ssize_t
         x;
 
       register PixelPacket
-        *restrict q;
+        *magick_restrict q;
 
       /*
         Draw top of ornamental border.
@@ -416,13 +411,13 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register IndexPacket
-      *restrict frame_indexes;
+      *magick_restrict frame_indexes;
 
     register ssize_t
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     /*
       Initialize scanline with matte color.
@@ -456,41 +451,32 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
       frame_indexes++;
     }
     /*
-      Set frame interior to interior color.
+      Set frame interior pixels.
     */
-    if ((image->compose != CopyCompositeOp) &&
-        ((image->compose != OverCompositeOp) || (image->matte != MagickFalse)))
-      for (x=0; x < (ssize_t) image->columns; x++)
-      {
-        SetPixelPacket(frame_image,&interior,q,frame_indexes);
-        q++;
-        frame_indexes++;
-      }
-    else
-      {
-        register const IndexPacket
-          *indexes;
+    {
+      register const IndexPacket
+        *indexes;
 
-        register const PixelPacket
-          *p;
+      register const PixelPacket
+        *p;
 
-        p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
-        if (p == (const PixelPacket *) NULL)
-          {
-            status=MagickFalse;
-            continue;
-          }
-        indexes=GetCacheViewVirtualIndexQueue(image_view);
-        (void) CopyMagickMemory(q,p,image->columns*sizeof(*p));
-        if ((image->colorspace == CMYKColorspace) &&
-            (frame_image->colorspace == CMYKColorspace))
-          {
-            (void) CopyMagickMemory(frame_indexes,indexes,image->columns*
-              sizeof(*indexes));
-            frame_indexes+=image->columns;
-          }
-        q+=image->columns;
-      }
+      p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+      if (p == (const PixelPacket *) NULL)
+        {
+          status=MagickFalse;
+          continue;
+        }
+      indexes=GetCacheViewVirtualIndexQueue(image_view);
+      (void) CopyMagickMemory(q,p,image->columns*sizeof(*p));
+      if ((image->colorspace == CMYKColorspace) &&
+          (frame_image->colorspace == CMYKColorspace))
+        {
+          (void) CopyMagickMemory(frame_indexes,indexes,image->columns*
+            sizeof(*indexes));
+          frame_indexes+=image->columns;
+        }
+      q+=image->columns;
+    }
     for (x=0; x < (ssize_t) frame_info->inner_bevel; x++)
     {
       SetPixelPacket(frame_image,&highlight,q,frame_indexes);
@@ -530,13 +516,13 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
   if (height != 0)
     {
       register IndexPacket
-        *restrict frame_indexes;
+        *magick_restrict frame_indexes;
 
       register ssize_t
         x;
 
       register PixelPacket
-        *restrict q;
+        *magick_restrict q;
 
       /*
         Draw bottom of ornamental border.
@@ -638,15 +624,12 @@ MagickExport Image *FrameImage(const Image *image,const FrameInfo *frame_info,
     }
   frame_view=DestroyCacheView(frame_view);
   image_view=DestroyCacheView(image_view);
-  if ((image->compose != CopyCompositeOp) &&
-      ((image->compose != OverCompositeOp) || (image->matte != MagickFalse)))
-    {
-      x=(ssize_t) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
-        frame_info->inner_bevel);
-      y=(ssize_t) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
-        frame_info->inner_bevel);
-      (void) CompositeImage(frame_image,image->compose,image,x,y);
-    }
+  x=(ssize_t) (frame_info->outer_bevel+(frame_info->x-bevel_width)+
+    frame_info->inner_bevel);
+  y=(ssize_t) (frame_info->outer_bevel+(frame_info->y-bevel_width)+
+    frame_info->inner_bevel);
+  if (status != MagickFalse)
+    status=CompositeImage(frame_image,image->compose,image,x,y);
   if (status == MagickFalse)
     frame_image=DestroyImage(frame_image);
   return(frame_image);
@@ -746,7 +729,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -820,7 +803,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
@@ -883,7 +866,7 @@ MagickExport MagickBooleanType RaiseImage(Image *image,
       x;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     if (status == MagickFalse)
       continue;
