@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -48,6 +48,7 @@
 #include "magick/cache.h"
 #include "magick/cache-private.h"
 #include "magick/cache-view.h"
+#include "magick/channel.h"
 #include "magick/client.h"
 #include "magick/color.h"
 #include "magick/color-private.h"
@@ -291,8 +292,8 @@ MagickExport Image *CannyEdgeImage(const Image *image,const double radius,
   kernel_info=AcquireKernelInfo(geometry);
   if (kernel_info == (KernelInfo *) NULL)
     ThrowImageException(ResourceLimitError,"MemoryAllocationFailed");
-  edge_image=MorphologyApply(image,DefaultChannels,ConvolveMorphology,1,
-    kernel_info,UndefinedCompositeOp,0.0,exception);
+  edge_image=MorphologyImageChannel(image,DefaultChannels,ConvolveMorphology,1,
+    kernel_info,exception);
   kernel_info=DestroyKernelInfo(kernel_info);
   if (edge_image == (Image *) NULL)
     return((Image *) NULL);
@@ -301,6 +302,7 @@ MagickExport Image *CannyEdgeImage(const Image *image,const double radius,
       edge_image=DestroyImage(edge_image);
       return((Image *) NULL);
     }
+  (void) SetImageAlphaChannel(edge_image,DeactivateAlphaChannel);
   /*
     Find the intensity gradient of the image.
   */
@@ -320,7 +322,7 @@ MagickExport Image *CannyEdgeImage(const Image *image,const double radius,
   for (y=0; y < (ssize_t) edge_image->rows; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       x;
@@ -344,7 +346,7 @@ MagickExport Image *CannyEdgeImage(const Image *image,const double radius,
         dy;
 
       register const PixelPacket
-        *restrict kernel_pixels;
+        *magick_restrict kernel_pixels;
 
       ssize_t
         v;
@@ -432,7 +434,7 @@ MagickExport Image *CannyEdgeImage(const Image *image,const double radius,
   for (y=0; y < (ssize_t) edge_image->rows; y++)
   {
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     register ssize_t
       x;
@@ -552,7 +554,7 @@ MagickExport Image *CannyEdgeImage(const Image *image,const double radius,
         pixel;
 
       register const PixelPacket
-        *restrict p;
+        *magick_restrict p;
 
       /*
         Edge if pixel gradient higher than upper threshold.
@@ -616,13 +618,6 @@ MagickExport Image *CannyEdgeImage(const Image *image,const double radius,
 %
 */
 
-static inline ssize_t MagickAbsoluteValue(const ssize_t x)
-{
-  if (x < 0)
-    return(-x);
-  return(x);
-}
-
 static inline double MagickLog10(const double x)
 {
 #define Log10Epsilon  (1.0e-11)
@@ -725,10 +720,10 @@ MagickExport ChannelFeatures *GetImageChannelFeatures(const Image *image,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       x;
@@ -900,10 +895,10 @@ MagickExport ChannelFeatures *GetImageChannelFeatures(const Image *image,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       x;
@@ -1786,6 +1781,69 @@ static inline double MagickRound(double x)
   return(ceil(x));
 }
 
+static Image *RenderHoughLines(const ImageInfo *image_info,const size_t columns,
+  const size_t rows,ExceptionInfo *exception)
+{
+#define BoundingBox  "viewbox"
+
+  DrawInfo
+    *draw_info;
+
+  Image
+    *image;
+
+  MagickBooleanType
+    status;
+
+  /*
+    Open image.
+  */
+  image=AcquireImage(image_info);
+  status=OpenBlob(image_info,image,ReadBinaryBlobMode,exception);
+  if (status == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
+  image->columns=columns;
+  image->rows=rows;
+  draw_info=CloneDrawInfo(image_info,(DrawInfo *) NULL);
+  draw_info->affine.sx=image->x_resolution == 0.0 ? 1.0 : image->x_resolution/
+    DefaultResolution;
+  draw_info->affine.sy=image->y_resolution == 0.0 ? 1.0 : image->y_resolution/
+    DefaultResolution;
+  image->columns=(size_t) (draw_info->affine.sx*image->columns);
+  image->rows=(size_t) (draw_info->affine.sy*image->rows);
+  status=SetImageExtent(image,image->columns,image->rows);
+  if (status == MagickFalse)
+    return(DestroyImageList(image));
+  if (SetImageBackgroundColor(image) == MagickFalse)
+    {
+      image=DestroyImageList(image);
+      return((Image *) NULL);
+    }
+  /*
+    Render drawing.
+  */
+  if (GetBlobStreamData(image) == (unsigned char *) NULL)
+    draw_info->primitive=FileToString(image->filename,~0UL,exception);
+  else
+    {
+      draw_info->primitive=(char *) AcquireMagickMemory((size_t)
+        GetBlobSize(image)+1);
+      if (draw_info->primitive != (char *) NULL)
+        {
+          (void) CopyMagickMemory(draw_info->primitive,GetBlobStreamData(image),
+            (size_t) GetBlobSize(image));
+          draw_info->primitive[GetBlobSize(image)]='\0';
+        }
+     }
+  (void) DrawImage(image,draw_info);
+  draw_info=DestroyDrawInfo(draw_info);
+  (void) CloseBlob(image);
+  return(GetFirstImageInList(image));
+}
+
 MagickExport Image *HoughLineImage(const Image *image,const size_t width,
   const size_t height,const size_t threshold,ExceptionInfo *exception)
 {
@@ -1866,7 +1924,7 @@ MagickExport Image *HoughLineImage(const Image *image,const size_t width,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       x;
@@ -1881,7 +1939,7 @@ MagickExport Image *HoughLineImage(const Image *image,const size_t width,
       }
     for (x=0; x < (ssize_t) image->columns; x++)
     {
-      if (GetPixelIntensity(image,p) > (QuantumRange/2))
+      if (GetPixelIntensity(image,p) > (QuantumRange/2.0))
         {
           register ssize_t
             i;
@@ -2034,7 +2092,7 @@ MagickExport Image *HoughLineImage(const Image *image,const size_t width,
   */
   image_info=AcquireImageInfo();
   image_info->background_color=image->background_color;
-  (void) FormatLocaleString(image_info->filename,MaxTextExtent,"mvg:%s",path);
+  (void) FormatLocaleString(image_info->filename,MaxTextExtent,"%s",path);
   artifact=GetImageArtifact(image,"background");
   if (artifact != (const char *) NULL)
     (void) SetImageOption(image_info,"background",artifact);
@@ -2047,7 +2105,7 @@ MagickExport Image *HoughLineImage(const Image *image,const size_t width,
   artifact=GetImageArtifact(image,"strokewidth");
   if (artifact != (const char *) NULL)
     (void) SetImageOption(image_info,"strokewidth",artifact);
-  lines_image=ReadImage(image_info,exception);
+  lines_image=RenderHoughLines(image_info,image->columns,image->rows,exception);
   artifact=GetImageArtifact(image,"hough-lines:accumulator");
   if ((lines_image != (Image *) NULL) &&
       (IsMagickTrue(artifact) != MagickFalse))
@@ -2158,13 +2216,13 @@ MagickExport Image *MeanShiftImage(const Image *image,const size_t width,
   for (y=0; y < (ssize_t) mean_image->rows; y++)
   {
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register PixelPacket
-      *restrict q;
+      *magick_restrict q;
 
     register ssize_t
       x;
