@@ -17,7 +17,7 @@
 %                               December 2002                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -46,6 +46,7 @@
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/hashmap.h"
+#include "magick/locale_.h"
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
 #include "magick/signature-private.h"
@@ -1425,7 +1426,6 @@ static MagickBooleanType IncreaseHashmapCapacity(HashmapInfo *hashmap_info)
     *entry;
 
   LinkedListInfo
-    *list_info,
     *map_info,
     **map;
 
@@ -1457,6 +1457,9 @@ static MagickBooleanType IncreaseHashmapCapacity(HashmapInfo *hashmap_info)
   */
   for (i=0; i < (ssize_t) hashmap_info->capacity; i++)
   {
+    LinkedListInfo
+      *list_info;
+
     list_info=hashmap_info->map[i];
     if (list_info == (LinkedListInfo *) NULL)
       continue;
