@@ -18,7 +18,7 @@
 %                                August 2009                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -537,8 +537,8 @@ MagickExport ColorPacket *GetImageHistogram(const Image *image,
   cube_info=ClassifyImageColors(image,exception);
   if (cube_info != (CubeInfo *) NULL)
     {
-      histogram=(ColorPacket *) AcquireQuantumMemory((size_t) cube_info->colors,
-        sizeof(*histogram));
+      histogram=(ColorPacket *) AcquireQuantumMemory((size_t)
+        cube_info->colors+1,sizeof(*histogram));
       if (histogram == (ColorPacket *) NULL)
         (void) ThrowMagickException(exception,GetMagickModule(),
           ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
@@ -614,6 +614,174 @@ static NodeInfo *GetNodeInfo(CubeInfo *cube_info,const size_t level)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%  I d e n t i f y P a l e t t e I m a g e                                    %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  IdentifyPaletteImage() returns MagickTrue if the image has 256 unique colors
+%  or less.
+%
+%  The format of the IdentifyPaletteImage method is:
+%
+%      MagickBooleanType IdentifyPaletteImage(const Image *image,
+%        ExceptionInfo *exception)
+%
+%  A description of each parameter follows.
+%
+%    o image: the image.
+%
+%    o exception: return any errors or warnings in this structure.
+%
+*/
+
+static MagickBooleanType CheckImageColors(const Image *image,
+  ExceptionInfo *exception,size_t max_colors)
+{
+  CacheView
+    *image_view;
+
+  CubeInfo
+    *cube_info;
+
+  MagickPixelPacket
+    pixel,
+    target;
+
+  register const IndexPacket
+    *indexes;
+
+  register const PixelPacket
+    *p;
+
+  register ssize_t
+    x;
+
+  register NodeInfo
+    *node_info;
+
+  register ssize_t
+    i;
+
+  size_t
+    id,
+    index,
+    level;
+
+  ssize_t
+    y;
+
+  if (image->storage_class == PseudoClass)
+    return((image->colors <= max_colors) ? MagickTrue : MagickFalse);
+  /*
+    Initialize color description tree.
+  */
+  cube_info=GetCubeInfo();
+  if (cube_info == (CubeInfo *) NULL)
+    {
+      (void) ThrowMagickException(exception,GetMagickModule(),
+        ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
+      return(MagickFalse);
+    }
+  GetMagickPixelPacket(image,&pixel);
+  GetMagickPixelPacket(image,&target);
+  image_view=AcquireVirtualCacheView(image,exception);
+  for (y=0; y < (ssize_t) image->rows; y++)
+  {
+    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
+    if (p == (const PixelPacket *) NULL)
+      break;
+    indexes=GetCacheViewVirtualIndexQueue(image_view);
+    for (x=0; x < (ssize_t) image->columns; x++)
+    {
+      /*
+        Start at the root and proceed level by level.
+      */
+      node_info=cube_info->root;
+      index=MaxTreeDepth-1;
+      for (level=1; level < MaxTreeDepth; level++)
+      {
+        SetMagickPixelPacket(image,p,indexes+x,&pixel);
+        id=ColorToNodeId(image,&pixel,index);
+        if (node_info->child[id] == (NodeInfo *) NULL)
+          {
+            node_info->child[id]=GetNodeInfo(cube_info,level);
+            if (node_info->child[id] == (NodeInfo *) NULL)
+              {
+                (void) ThrowMagickException(exception,GetMagickModule(),
+                  ResourceLimitError,"MemoryAllocationFailed","`%s'",
+                  image->filename);
+                break;
+              }
+          }
+        node_info=node_info->child[id];
+        index--;
+      }
+      if (level < MaxTreeDepth)
+        break;
+      for (i=0; i < (ssize_t) node_info->number_unique; i++)
+      {
+        SetMagickPixelPacket(image,&node_info->list[i].pixel,
+          &node_info->list[i].index,&target);
+        if (IsMagickColorEqual(&pixel,&target) != MagickFalse)
+          break;
+      }
+      if (i < (ssize_t) node_info->number_unique)
+        node_info->list[i].count++;
+      else
+        {
+          /*
+            Add this unique color to the color list.
+          */
+          if (node_info->number_unique == 0)
+            node_info->list=(ColorPacket *) AcquireMagickMemory(
+              sizeof(*node_info->list));
+          else
+            node_info->list=(ColorPacket *) ResizeQuantumMemory(node_info->list,
+              (size_t) (i+1),sizeof(*node_info->list));
+          if (node_info->list == (ColorPacket *) NULL)
+            {
+              (void) ThrowMagickException(exception,GetMagickModule(),
+                ResourceLimitError,"MemoryAllocationFailed","`%s'",
+                image->filename);
+              break;
+            }
+          node_info->list[i].pixel=(*p);
+          if ((image->colorspace == CMYKColorspace) ||
+              (image->storage_class == PseudoClass))
+            node_info->list[i].index=GetPixelIndex(indexes+x);
+          node_info->list[i].count=1;
+          node_info->number_unique++;
+          cube_info->colors++;
+          if (cube_info->colors > max_colors)
+            break;
+        }
+      p++;
+    }
+    if (x < (ssize_t) image->columns)
+      break;
+  }
+  image_view=DestroyCacheView(image_view);
+  cube_info=DestroyCubeInfo(image,cube_info);
+  return(y < (ssize_t) image->rows ? MagickFalse : MagickTrue);
+}
+
+MagickExport MagickBooleanType IdentifyPaletteImage(const Image *image,
+  ExceptionInfo *exception)
+{
+  assert(image != (Image *) NULL);
+  assert(image->signature == MagickSignature);
+  if (image->debug != MagickFalse)
+    (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
+  return(CheckImageColors(image,exception,256));
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %  I s H i s t o g r a m I m a g e                                            %
 %                                                                             %
 %                                                                             %
@@ -677,7 +845,8 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  if ((image->storage_class == PseudoClass) && (image->colors <= 256))
+  if ((image->storage_class == PseudoClass) &&
+      (image->colors <= MaximumUniqueColors))
     return(MagickTrue);
   if (image->storage_class == PseudoClass)
     return(MagickFalse);
@@ -803,138 +972,13 @@ MagickExport MagickBooleanType IsHistogramImage(const Image *image,
 MagickExport MagickBooleanType IsPaletteImage(const Image *image,
   ExceptionInfo *exception)
 {
-  CacheView
-    *image_view;
-
-  CubeInfo
-    *cube_info;
-
-  MagickPixelPacket
-    pixel,
-    target;
-
-  register const IndexPacket
-    *indexes;
-
-  register const PixelPacket
-    *p;
-
-  register ssize_t
-    x;
-
-  register NodeInfo
-    *node_info;
-
-  register ssize_t
-    i;
-
-  size_t
-    id,
-    index,
-    level;
-
-  ssize_t
-    y;
-
   assert(image != (Image *) NULL);
   assert(image->signature == MagickSignature);
   if (image->debug != MagickFalse)
     (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",image->filename);
-  if ((image->storage_class == PseudoClass) && (image->colors <= 256))
-    return(MagickTrue);
-  if (image->storage_class == PseudoClass)
+  if (image->storage_class != PseudoClass)
     return(MagickFalse);
-  /*
-    Initialize color description tree.
-  */
-  cube_info=GetCubeInfo();
-  if (cube_info == (CubeInfo *) NULL)
-    {
-      (void) ThrowMagickException(exception,GetMagickModule(),
-        ResourceLimitError,"MemoryAllocationFailed","`%s'",image->filename);
-      return(MagickFalse);
-    }
-  GetMagickPixelPacket(image,&pixel);
-  GetMagickPixelPacket(image,&target);
-  image_view=AcquireVirtualCacheView(image,exception);
-  for (y=0; y < (ssize_t) image->rows; y++)
-  {
-    p=GetCacheViewVirtualPixels(image_view,0,y,image->columns,1,exception);
-    if (p == (const PixelPacket *) NULL)
-      break;
-    indexes=GetCacheViewVirtualIndexQueue(image_view);
-    for (x=0; x < (ssize_t) image->columns; x++)
-    {
-      /*
-        Start at the root and proceed level by level.
-      */
-      node_info=cube_info->root;
-      index=MaxTreeDepth-1;
-      for (level=1; level < MaxTreeDepth; level++)
-      {
-        SetMagickPixelPacket(image,p,indexes+x,&pixel);
-        id=ColorToNodeId(image,&pixel,index);
-        if (node_info->child[id] == (NodeInfo *) NULL)
-          {
-            node_info->child[id]=GetNodeInfo(cube_info,level);
-            if (node_info->child[id] == (NodeInfo *) NULL)
-              {
-                (void) ThrowMagickException(exception,GetMagickModule(),
-                  ResourceLimitError,"MemoryAllocationFailed","`%s'",
-                  image->filename);
-                break;
-              }
-          }
-        node_info=node_info->child[id];
-        index--;
-      }
-      if (level < MaxTreeDepth)
-        break;
-      for (i=0; i < (ssize_t) node_info->number_unique; i++)
-      {
-        SetMagickPixelPacket(image,&node_info->list[i].pixel,
-          &node_info->list[i].index,&target);
-        if (IsMagickColorEqual(&pixel,&target) != MagickFalse)
-          break;
-      }
-      if (i < (ssize_t) node_info->number_unique)
-        node_info->list[i].count++;
-      else
-        {
-          /*
-            Add this unique color to the color list.
-          */
-          if (node_info->number_unique == 0)
-            node_info->list=(ColorPacket *) AcquireMagickMemory(
-              sizeof(*node_info->list));
-          else
-            node_info->list=(ColorPacket *) ResizeQuantumMemory(node_info->list,
-              (size_t) (i+1),sizeof(*node_info->list));
-          if (node_info->list == (ColorPacket *) NULL)
-            {
-              (void) ThrowMagickException(exception,GetMagickModule(),
-                ResourceLimitError,"MemoryAllocationFailed","`%s'",
-                image->filename);
-              break;
-            }
-          node_info->list[i].pixel=(*p);
-          if ((image->colorspace == CMYKColorspace) ||
-              (image->storage_class == PseudoClass))
-            node_info->list[i].index=GetPixelIndex(indexes+x);
-          node_info->list[i].count=1;
-          node_info->number_unique++;
-          cube_info->colors++;
-          if (cube_info->colors > 256)
-            break;
-        }
-      p++;
-    }
-    if (x < (ssize_t) image->columns)
-      break;
-  }
-  image_view=DestroyCacheView(image_view);
-  cube_info=DestroyCubeInfo(image,cube_info);
-  return(y < (ssize_t) image->rows ? MagickFalse : MagickTrue);
+  return((image->colors <= 256) ? MagickTrue : MagickFalse);
 }
 
 /*
@@ -1259,10 +1303,10 @@ static void UniqueColorsToImage(Image *unique_image,CacheView *unique_view,
         *p;
 
       register IndexPacket
-        *restrict indexes;
+        *magick_restrict indexes;
 
       register PixelPacket
-        *restrict q;
+        *magick_restrict q;
 
       status=MagickTrue;
       p=node_info->list;
@@ -1325,18 +1369,6 @@ MagickExport Image *UniqueImageColors(const Image *image,
   UniqueColorsToImage(unique_image,unique_view,cube_info,cube_info->root,
     exception);
   unique_view=DestroyCacheView(unique_view);
-  if (cube_info->colors < MaxColormapSize)
-    {
-      QuantizeInfo
-        *quantize_info;
-
-      quantize_info=AcquireQuantizeInfo((ImageInfo *) NULL);
-      quantize_info->number_colors=MaxColormapSize;
-      quantize_info->dither=MagickFalse;
-      quantize_info->tree_depth=8;
-      (void) QuantizeImage(quantize_info,unique_image);
-      quantize_info=DestroyQuantizeInfo(quantize_info);
-    }
   cube_info=DestroyCubeInfo(image,cube_info);
   return(unique_image);
 }
