@@ -17,7 +17,7 @@
 %                            September 1994                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -172,10 +172,10 @@ static ChannelStatistics *GetLocationStatistics(const Image *image,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const IndexPacket
-      *restrict indexes;
+      *magick_restrict indexes;
 
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       x;
@@ -351,22 +351,24 @@ static ssize_t PrintChannelLocations(FILE *file,const Image *image,
       {
         case RedChannel:
         {
-          match=fabs((double) p->red-target) < 0.5 ? MagickTrue : MagickFalse;
+          match=fabs((double) (p->red-target)) < 0.5 ? MagickTrue : MagickFalse;
           break;
         }
         case GreenChannel:
         {
-          match=fabs((double) p->green-target) < 0.5 ? MagickTrue : MagickFalse;
+          match=fabs((double) (p->green-target)) < 0.5 ? MagickTrue :
+            MagickFalse;
           break;
         }
         case BlueChannel:
         {
-          match=fabs((double) p->blue-target) < 0.5 ? MagickTrue : MagickFalse;
+          match=fabs((double) (p->blue-target)) < 0.5 ? MagickTrue :
+            MagickFalse;
           break;
         }
         case AlphaChannel:
         {
-          match=fabs((double) p->opacity-target) < 0.5 ? MagickTrue :
+          match=fabs((double) (p->opacity-target)) < 0.5 ? MagickTrue :
             MagickFalse;
           break;
         }
@@ -448,35 +450,39 @@ static ssize_t PrintChannelStatistics(FILE *file,const ChannelType channel,
 #define StatisticsFormat "    %s:\n      min: " QuantumFormat  \
   " (%g)\n      max: " QuantumFormat " (%g)\n"  \
   "      mean: %g (%g)\n      standard deviation: %g (%g)\n"  \
-  "      kurtosis: %g\n      skewness: %g\n"
+  "      kurtosis: %g\n      skewness: %g\n      entropy: %g\n"
 
   ssize_t
     n;
 
   if (channel == AlphaChannel)
     {
-      n=FormatLocaleFile(file,StatisticsFormat,name,ClampToQuantum(scale*
-        (QuantumRange-channel_statistics[channel].maxima)),
-        (QuantumRange-channel_statistics[channel].maxima)/(double) QuantumRange,
-        ClampToQuantum(scale*(QuantumRange-channel_statistics[channel].minima)),
-        (QuantumRange-channel_statistics[channel].minima)/(double) QuantumRange,
+      n=FormatLocaleFile(file,StatisticsFormat,name,ClampToQuantum(
+        (MagickRealType) (scale*(QuantumRange-
+        channel_statistics[channel].maxima))),(QuantumRange-
+        channel_statistics[channel].maxima)/(double) QuantumRange,
+        ClampToQuantum((MagickRealType) (scale*(QuantumRange-
+        channel_statistics[channel].minima))),(QuantumRange-
+        channel_statistics[channel].minima)/(double) QuantumRange,
         scale*(QuantumRange-channel_statistics[channel].mean),(QuantumRange-
         channel_statistics[channel].mean)/(double) QuantumRange,scale*
         channel_statistics[channel].standard_deviation,
         channel_statistics[channel].standard_deviation/(double) QuantumRange,
         channel_statistics[channel].kurtosis,
-        channel_statistics[channel].skewness);
+        channel_statistics[channel].skewness,
+        channel_statistics[channel].entropy);
       return(n);
     }
-  n=FormatLocaleFile(file,StatisticsFormat,name,ClampToQuantum(scale*
-    channel_statistics[channel].minima),channel_statistics[channel].minima/
-    (double) QuantumRange,ClampToQuantum(scale*
-    channel_statistics[channel].maxima),channel_statistics[channel].maxima/
-    (double) QuantumRange,scale*channel_statistics[channel].mean,
-    channel_statistics[channel].mean/(double) QuantumRange,scale*
-    channel_statistics[channel].standard_deviation,
+  n=FormatLocaleFile(file,StatisticsFormat,name,ClampToQuantum((MagickRealType)
+    (scale*channel_statistics[channel].minima)),
+    channel_statistics[channel].minima/(double) QuantumRange,ClampToQuantum(
+    (MagickRealType) (scale*channel_statistics[channel].maxima)),
+    channel_statistics[channel].maxima/(double) QuantumRange,scale*
+    channel_statistics[channel].mean,channel_statistics[channel].mean/(double)
+    QuantumRange,scale*channel_statistics[channel].standard_deviation,
     channel_statistics[channel].standard_deviation/(double) QuantumRange,
-    channel_statistics[channel].kurtosis,channel_statistics[channel].skewness);
+    channel_statistics[channel].kurtosis,channel_statistics[channel].skewness,
+    channel_statistics[channel].entropy);
   return(n);
 }
 
@@ -558,22 +564,24 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         max_locations;
 
       StatisticType
-        type;
+        statistic_type;
 
       /*
         Display minimum, maximum, or mean pixel locations.
       */
-      type=(StatisticType) ParseCommandOption(MagickStatisticOptions,
+      statistic_type=(StatisticType) ParseCommandOption(MagickStatisticOptions,
         MagickFalse,locate);
       limit=GetImageArtifact(image,"identify:limit");
       max_locations=0;
       if (limit != (const char *) NULL)
         max_locations=StringToUnsignedLong(limit);
-      channel_statistics=GetLocationStatistics(image,type,exception);
+      channel_statistics=GetLocationStatistics(image,statistic_type,exception);
       if (channel_statistics == (ChannelStatistics *) NULL)
         return(MagickFalse);
       colorspace=image->colorspace;
-      if (IsGrayImage(image,exception) != MagickFalse)
+      type=IdentifyImageType(image,exception);
+      if ((type == BilevelType) || (type == GrayscaleType) ||
+          (type == GrayscaleMatteType))
         colorspace=GRAYColorspace;
       (void) FormatLocaleFile(file,"  Channel %s locations:\n",locate);
       switch (colorspace)
@@ -581,36 +589,36 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         case RGBColorspace:
         default:
         {
-          (void) PrintChannelLocations(file,image,RedChannel,"Red",type,
-            max_locations,channel_statistics);
-          (void) PrintChannelLocations(file,image,GreenChannel,"Green",type,
-            max_locations,channel_statistics);
-          (void) PrintChannelLocations(file,image,BlueChannel,"Blue",type,
-            max_locations,channel_statistics);
+          (void) PrintChannelLocations(file,image,RedChannel,"Red",
+            statistic_type,max_locations,channel_statistics);
+          (void) PrintChannelLocations(file,image,GreenChannel,"Green",
+            statistic_type,max_locations,channel_statistics);
+          (void) PrintChannelLocations(file,image,BlueChannel,"Blue",
+            statistic_type,max_locations,channel_statistics);
           break;
         }
         case CMYKColorspace:
         {
-          (void) PrintChannelLocations(file,image,CyanChannel,"Cyan",type,
-            max_locations,channel_statistics);
-          (void) PrintChannelLocations(file,image,MagentaChannel,"Magenta",type,
-            max_locations,channel_statistics);
-          (void) PrintChannelLocations(file,image,YellowChannel,"Yellow",type,
-            max_locations,channel_statistics);
-          (void) PrintChannelLocations(file,image,BlackChannel,"Black",type,
-            max_locations,channel_statistics);
+          (void) PrintChannelLocations(file,image,CyanChannel,"Cyan",
+            statistic_type,max_locations,channel_statistics);
+          (void) PrintChannelLocations(file,image,MagentaChannel,"Magenta",
+            statistic_type,max_locations,channel_statistics);
+          (void) PrintChannelLocations(file,image,YellowChannel,"Yellow",
+            statistic_type,max_locations,channel_statistics);
+          (void) PrintChannelLocations(file,image,BlackChannel,"Black",
+            statistic_type,max_locations,channel_statistics);
           break;
         }
         case GRAYColorspace:
         {
-          (void) PrintChannelLocations(file,image,GrayChannel,"Gray",type,
-            max_locations,channel_statistics);
+          (void) PrintChannelLocations(file,image,GrayChannel,"Gray",
+            statistic_type,max_locations,channel_statistics);
           break;
         }
       }
       if (image->matte != MagickFalse)
-        (void) PrintChannelLocations(file,image,AlphaChannel,"Alpha",type,
-          max_locations,channel_statistics);
+        (void) PrintChannelLocations(file,image,AlphaChannel,"Alpha",
+          statistic_type,max_locations,channel_statistics);
       channel_statistics=(ChannelStatistics *) RelinquishMagickMemory(
         channel_statistics);
       exception=DestroyExceptionInfo(exception);
@@ -690,11 +698,15 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   /*
     Display verbose info about the image.
   */
+  colorspace=image->colorspace;
+  type=IdentifyImageType(image,exception);
+  if ((type == BilevelType) || (type == GrayscaleType) ||
+      (type == GrayscaleMatteType))
+    colorspace=GRAYColorspace;
   pixels=GetVirtualPixels(image,0,0,1,1,exception);
   exception=DestroyExceptionInfo(exception);
   ping=pixels == (const PixelPacket *) NULL ? MagickTrue : MagickFalse;
   exception=(&image->exception);
-  type=GetImageType(image,exception);
   (void) SignatureImage(image);
   (void) FormatLocaleFile(file,"Image: %s\n",image->filename);
   if (*image->magick_filename != '\0')
@@ -783,7 +795,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
         (void) FormatLocaleFile(file,"  Depth: %.20g/%.20g-bit\n",(double)
           image->depth,(double) depth);
       (void) FormatLocaleFile(file,"  Channel depth:\n");
-      if (IsGrayImage(image,exception) != MagickFalse)
+      if (SetImageGray(image,exception) != MagickFalse)
         colorspace=GRAYColorspace;
       switch (colorspace)
       {
@@ -822,14 +834,14 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
           channel_statistics[OpacityChannel].depth);
       scale=1.0;
       if (image->depth <= MAGICKCORE_QUANTUM_DEPTH)
-        scale=(double) (QuantumRange/((size_t) QuantumRange >> ((size_t)
-          MAGICKCORE_QUANTUM_DEPTH-image->depth)));
+        scale=(double) QuantumRange/((size_t) QuantumRange >> ((size_t)
+          MAGICKCORE_QUANTUM_DEPTH-image->depth));
     }
   if (channel_statistics != (ChannelStatistics *) NULL)
     {
       (void) FormatLocaleFile(file,"  Channel statistics:\n");
-      (void) FormatLocaleFile(file,"    Pixels: %.20g\n",
-        (double) image->columns*image->rows);
+      (void) FormatLocaleFile(file,"    Pixels: %.20g\n",(double)
+        image->columns*image->rows);
       switch (colorspace)
       {
         case RGBColorspace:
@@ -1053,7 +1065,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
             pixel;
 
           register PixelPacket
-            *restrict p;
+            *magick_restrict p;
 
           GetMagickPixelPacket(image,&pixel);
           p=image->colormap;
@@ -1334,6 +1346,7 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
                 case 80: tag="Byline"; break;
                 case 85: tag="Byline Title"; break;
                 case 90: tag="City"; break;
+                case 92: tag="Sub-Location"; break;
                 case 95: tag="Province State"; break;
                 case 100: tag="Country Code"; break;
                 case 101: tag="Country"; break;
@@ -1445,9 +1458,12 @@ MagickExport MagickBooleanType IdentifyImage(Image *image,FILE *file,
   if (strlen(format) > 1)
     format[strlen(format)-1]='\0';
   (void) FormatLocaleFile(file,"  Number pixels: %s\n",format);
-  (void) FormatMagickSize((MagickSizeType) ((double) image->columns*image->rows/
-    elapsed_time+0.5),MagickFalse,format);
-  (void) FormatLocaleFile(file,"  Pixels per second: %s\n",format);
+  if (elapsed_time > MagickEpsilon)
+    {
+      (void) FormatMagickSize((MagickSizeType) ((double) image->columns*
+        image->rows/elapsed_time+0.5),MagickFalse,format);
+      (void) FormatLocaleFile(file,"  Pixels per second: %s\n",format);
+    }
   (void) FormatLocaleFile(file,"  User time: %0.3fu\n",user_time);
   (void) FormatLocaleFile(file,"  Elapsed time: %lu:%02lu.%03lu\n",
     (unsigned long) (elapsed_time/60.0),(unsigned long) ceil(fmod(
