@@ -18,7 +18,7 @@
 %                             November 1998                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -36,8 +36,7 @@
 %
 %
 */
-
-
+
 /*
   Include declarations.
 */
@@ -64,6 +63,7 @@
 #include "magick/module.h"
 #include "magick/nt-base-private.h"
 #include "magick/nt-feature.h"
+#include "magick/opencl-private.h"
 #include "magick/random_.h"
 #include "magick/registry.h"
 #include "magick/resource_.h"
@@ -115,7 +115,8 @@ static SplayTreeInfo
   *magick_list = (SplayTreeInfo *) NULL;
 
 static volatile MagickBooleanType
-  instantiate_magickcore = MagickFalse;
+  instantiate_magickcore = MagickFalse,
+  magickcore_signal_in_progress = MagickFalse;
 
 /*
   Forward declarations.
@@ -851,12 +852,6 @@ static MagickBooleanType IsMagickTreeInstantiated(ExceptionInfo *exception)
           if (magick_list == (SplayTreeInfo *) NULL)
             ThrowFatalException(ResourceLimitFatalError,
               "MemoryAllocationFailed");
-          magick_info=SetMagickInfo("ephemeral");
-          magick_info->stealth=MagickTrue;
-          status=AddValueToSplayTree(magick_list,magick_info->name,magick_info);
-          if (status == MagickFalse)
-            ThrowFatalException(ResourceLimitFatalError,
-              "MemoryAllocationFailed");
           magick_info=SetMagickInfo("clipmask");
           magick_info->stealth=MagickTrue;
           status=AddValueToSplayTree(magick_list,magick_info->name,magick_info);
@@ -1161,11 +1156,10 @@ static SignalHandler *SetMagickSignalHandler(int signal_number,
 
 static void MagickSignalHandler(int signal_number)
 {
-#if !defined(MAGICKCORE_HAVE_SIGACTION)
-  (void) signal(signal_number,SIG_IGN);
-#endif
+  if (magickcore_signal_in_progress != MagickFalse)
+    (void) SetMagickSignalHandler(signal_number,signal_handlers[signal_number]);
+  magickcore_signal_in_progress=MagickTrue;
   AsynchronousResourceComponentTerminus();
-  (void) SetMagickSignalHandler(signal_number,signal_handlers[signal_number]);
 #if defined(SIGQUIT)
   if (signal_number == SIGQUIT)
     abort();
@@ -1195,15 +1189,15 @@ static void MagickSignalHandler(int signal_number)
 #else
 #if defined(SIGHUP)
   if (signal_number == SIGHUP)
-    exit(signal_number);
+    _exit(signal_number);
 #endif
-#if defined(SIGINT) && !defined(MAGICKCORE_WINDOWS_SUPPORT)
+#if defined(SIGINT)
   if (signal_number == SIGINT)
-    exit(signal_number);
+    _exit(signal_number);
 #endif
 #if defined(SIGTERM)
   if (signal_number == SIGTERM)
-    exit(signal_number);
+    _exit(signal_number);
 #endif
 #if defined(MAGICKCORE_HAVE_RAISE)
   if (signal_handlers[signal_number] != MagickSignalHandler)
@@ -1263,10 +1257,15 @@ MagickExport void MagickCoreGenesis(const char *path,
   /*
     Set client name and execution path.
   */
-  (void) GetExecutionPath(execution_path,MaxTextExtent);
+#if defined(MAGICKCORE_WINDOWS_SUPPORT)
+  if ((path != (const char *) NULL) && (IsPathAccessible(path) != MagickFalse))
+#else
   if ((path != (const char *) NULL) && (*path == *DirectorySeparator) &&
       (IsPathAccessible(path) != MagickFalse))
+#endif
     (void) CopyMagickString(execution_path,path,MaxTextExtent);
+  else
+    (void) GetExecutionPath(execution_path,MaxTextExtent);
   GetPathComponent(execution_path,TailPath,filename);
   (void) SetClientName(filename);
   GetPathComponent(execution_path,HeadPath,execution_path);
@@ -1292,7 +1291,7 @@ MagickExport void MagickCoreGenesis(const char *path,
       if (signal_handlers[SIGHUP] == (SignalHandler *) NULL)
         signal_handlers[SIGHUP]=RegisterMagickSignalHandler(SIGHUP);
 #endif
-#if defined(SIGINT) && !defined(MAGICKCORE_WINDOWS_SUPPORT)
+#if defined(SIGINT)
       if (signal_handlers[SIGINT] == (SignalHandler *) NULL)
         signal_handlers[SIGINT]=RegisterMagickSignalHandler(SIGINT);
 #endif
@@ -1373,9 +1372,12 @@ MagickExport void MagickCoreTerminus(void)
   AnnotateComponentTerminus();
   MimeComponentTerminus();
   TypeComponentTerminus();
+#if defined(MAGICKCORE_OPENCL_SUPPORT)
+  OpenCLTerminus();
+#endif
   ColorComponentTerminus();
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
-  NTGhostscriptUnLoadDLL();
+  NTWindowsTerminus();
 #endif
   MagicComponentTerminus();
   DelegateComponentTerminus();
@@ -1394,10 +1396,9 @@ MagickExport void MagickCoreTerminus(void)
   RandomComponentTerminus();
   LocaleComponentTerminus();
   LogComponentTerminus();
-  SemaphoreComponentTerminus();
   instantiate_magickcore=MagickFalse;
   UnlockMagickMutex();
-  DestroyMagickMutex();
+  SemaphoreComponentTerminus();
 }
 
 /*
@@ -1542,7 +1543,7 @@ MagickExport int SetMagickPrecision(const int precision)
       /*
         Precision reset, or it has not been set yet.
       */
-      magick_precision = MagickPrecision;
+      magick_precision=MagickPrecision;
       limit=GetEnvironmentValue("MAGICK_PRECISION");
       if (limit == (char *) NULL)
         limit=GetPolicyValue("precision");
