@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -15,14 +15,14 @@
 
   MagickCore private memory methods.
 */
-#ifndef _MAGICKCORE_MEMORY_PRIVATE_H
-#define _MAGICKCORE_MEMORY_PRIVATE_H
+#ifndef MAGICKCORE_MEMORY_PRIVATE_H
+#define MAGICKCORE_MEMORY_PRIVATE_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
 #endif
 
-#if defined(MAGICK_TARGET_CPU) && (MAGICK_TARGET_CPU == powerpc)
+#if defined(__powerpc__)
 #  define CACHE_LINE_SIZE 128
 #else
 #  define CACHE_LINE_SIZE 64
@@ -41,6 +41,9 @@ extern "C" {
 #define MagickAssumeAligned(address)  (address)
 #endif
 
+MagickExport MagickBooleanType
+  HeapOverflowSanityCheck(const size_t,const size_t) magick_alloc_sizes(1,2);
+
 #if defined(__cplusplus) || defined(c_plusplus)
 }
 #endif
