@@ -17,7 +17,7 @@
 %                                March 2000                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -397,6 +397,8 @@ static inline int MagickReadDirectory(DIR *directory,struct dirent *entry,
 MagickExport char **GetModuleList(const char *pattern,
   const MagickModuleType type,size_t *number_modules,ExceptionInfo *exception)
 {
+#define MaxModules  511
+
   char
     **modules,
     filename[MaxTextExtent],
@@ -443,7 +445,7 @@ MagickExport char **GetModuleList(const char *pattern,
   if (status == MagickFalse)
     return((char **) NULL);
   GetPathComponent(module_path,HeadPath,path);
-  max_entries=255;
+  max_entries=MaxModules;
   modules=(char **) AcquireQuantumMemory((size_t) max_entries+1UL,
     sizeof(*modules));
   if (modules == (char **) NULL)
@@ -544,6 +546,8 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
   (void) LogMagickEvent(TraceEvent,GetMagickModule(),"%s",filename);
   assert(path != (char *) NULL);
   assert(exception != (ExceptionInfo *) NULL);
+  if (strchr(filename,'/') != (char *) NULL)
+    return(MagickFalse);
   (void) CopyMagickString(path,filename,MaxTextExtent);
   module_path=(char *) NULL;
   switch (module_type)
@@ -776,9 +780,25 @@ static MagickBooleanType GetMagickModulePath(const char *filename,
     char
       *home;
 
-    home=GetEnvironmentValue("HOME");
+    home=GetEnvironmentValue("XDG_CONFIG_HOME");
+    if (home == (char *) NULL)
+      home=GetEnvironmentValue("LOCALAPPDATA");
+    if (home == (char *) NULL)
+      home=GetEnvironmentValue("APPDATA");
     if (home == (char *) NULL)
       home=GetEnvironmentValue("USERPROFILE");
+    if (home != (char *) NULL)
+      {
+        /*
+          Search $XDG_CONFIG_HOME/ImageMagick.
+        */
+        (void) FormatLocaleString(path,MaxTextExtent,"%s%sImageMagick%s%s",
+          home,DirectorySeparator,DirectorySeparator,filename);
+        home=DestroyString(home);
+        if (IsPathAccessible(path) != MagickFalse)
+          return(MagickTrue);
+      }
+    home=GetEnvironmentValue("HOME");
     if (home != (char *) NULL)
       {
         /*
@@ -1368,8 +1388,12 @@ MagickExport MagickBooleanType OpenModules(ExceptionInfo *exception)
   (void) GetMagickInfo((char *) NULL,exception);
   number_modules=0;
   modules=GetModuleList("*",MagickImageCoderModule,&number_modules,exception);
-  if (modules == (char **) NULL)
-    return(MagickFalse);
+  if ((modules == (char **) NULL) || (*modules == (char *) NULL))
+    {
+      if (modules != (char **) NULL)
+        modules=(char **) RelinquishMagickMemory(modules);
+      return(MagickFalse);
+    }
   for (i=0; i < (ssize_t) number_modules; i++)
     (void) OpenModule(modules[i],exception);
   /*
