@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 
   The ImageMagick progress monitor private methods.
 */
-#ifndef _MAGICK_MONITOR_PRIVATE_H
-#define _MAGICK_MONITOR_PRIVATE_H
+#ifndef MAGICKCORE_MONITOR_PRIVATE_H
+#define MAGICKCORE_MONITOR_PRIVATE_H
 
 #include "magick/image.h"
 
