@@ -17,7 +17,7 @@
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -67,8 +67,32 @@
   Define declarations.
 */
 #if !defined(MAP_FAILED)
-#define MAP_FAILED      ((void *) -1)
+#define MAP_FAILED      ((void *)(LONG_PTR) -1)
 #endif
+
+/*
+Typdef declarations.
+*/
+
+/*
+We need to make sure only one instance is created for each process and that
+is why we wrap the new/delete instance methods.
+
+From: http://www.ghostscript.com/doc/current/API.htm
+"The Win32 DLL gsdll32.dll can be used by multiple programs simultaneously,
+but only once within each process"
+*/
+typedef struct _NTGhostInfo
+{
+  void
+    (MagickDLLCall *delete_instance)(gs_main_instance *);
+
+  int
+    (MagickDLLCall *new_instance)(gs_main_instance **, void *);
+
+  MagickBooleanType
+    has_instance;
+} NTGhostInfo;
 
 /*
   Static declarations.
@@ -78,6 +102,9 @@ static char
   *lt_slsearchpath = (char *) NULL;
 #endif
 
+static NTGhostInfo
+  nt_ghost_info;
+
 static GhostInfo
   ghost_info;
 
@@ -85,7 +112,11 @@ static void
   *ghost_handle = (void *) NULL;
 
 static SemaphoreInfo
-  *ghost_semaphore = (SemaphoreInfo *) NULL;
+  *ghost_semaphore = (SemaphoreInfo *) NULL,
+  *winsock_semaphore = (SemaphoreInfo *) NULL;
+
+static WSADATA
+  *wsaData = (WSADATA*) NULL;
 
 struct
 {
@@ -109,6 +140,33 @@ extern "C" BOOL WINAPI
   DllMain(HINSTANCE handle,DWORD reason,LPVOID lpvReserved);
 #endif
 
+static void MagickDLLCall NTGhostscriptDeleteInstance(
+  gs_main_instance *instance)
+{
+  LockSemaphoreInfo(ghost_semaphore);
+  nt_ghost_info.delete_instance(instance);
+  nt_ghost_info.has_instance=MagickFalse;
+  UnlockSemaphoreInfo(ghost_semaphore);
+}
+
+static int MagickDLLCall NTGhostscriptNewInstance(gs_main_instance **pinstance,
+  void *caller_handle)
+{
+  int
+    status;
+
+  LockSemaphoreInfo(ghost_semaphore);
+  status=-1;
+  if (nt_ghost_info.has_instance == MagickFalse)
+    {
+      status=nt_ghost_info.new_instance(pinstance,caller_handle);
+      if (status >= 0)
+        nt_ghost_info.has_instance=MagickTrue;
+    }
+  UnlockSemaphoreInfo(ghost_semaphore);
+  return(status);
+}
+
 static inline char *create_utf8_string(const wchar_t *wideChar)
 {
   char
@@ -176,7 +234,7 @@ static inline char *create_utf8_string(const wchar_t *wideChar)
 %                   and DLL_PROCESS_DETACH.
 %
 */
-#if defined(_DLL) && defined( ProvideDllMain )
+#if defined(_DLL) && defined(ProvideDllMain)
 BOOL WINAPI DllMain(HINSTANCE handle,DWORD reason,LPVOID lpvReserved)
 {
   magick_unreferenced(lpvReserved);
@@ -194,6 +252,7 @@ BOOL WINAPI DllMain(HINSTANCE handle,DWORD reason,LPVOID lpvReserved)
       wchar_t
         *wide_path;
 
+      MagickCoreGenesis((const char*) NULL,MagickFalse);
       wide_path=(wchar_t *) AcquireQuantumMemory(MaxTextExtent,
         sizeof(*wide_path));
       if (wide_path == (wchar_t *) NULL)
@@ -211,7 +270,6 @@ BOOL WINAPI DllMain(HINSTANCE handle,DWORD reason,LPVOID lpvReserved)
                 module_path[count+1]='\0';
                 break;
               }
-          MagickCoreGenesis(module_path,MagickFalse);
           path=(char *) AcquireQuantumMemory(16UL*MaxTextExtent,sizeof(*path));
           if (path == (char *) NULL)
             {
@@ -746,7 +804,7 @@ MagickPrivate MagickBooleanType NTGetExecutionPath(char *path,
   wchar_t
     wide_path[MaxTextExtent];
 
-  (void) GetModuleFileNameW(0,wide_path,(DWORD) extent);
+  (void) GetModuleFileNameW((HMODULE) NULL,wide_path,(DWORD) extent);
   (void) WideCharToMultiByte(CP_UTF8,0,wide_path,-1,path,(int) extent,NULL,
     NULL);
   return(MagickTrue);
@@ -1098,7 +1156,7 @@ static int NTGhostscriptGetString(const char *name,BOOL *is_64_bit,
   */
   *value='\0';
   directory=(char *) NULL;
-  if (LocaleCompare(name, "GS_DLL") == 0)
+  if (LocaleCompare(name,"GS_DLL") == 0)
     {
       directory=GetEnvironmentValue("MAGICK_GHOSTSCRIPT_PATH");
       if (directory != (char *) NULL)
@@ -1107,6 +1165,7 @@ static int NTGhostscriptGetString(const char *name,BOOL *is_64_bit,
             directory,DirectorySeparator);
           if (IsPathAccessible(buffer) != MagickFalse)
             {
+              directory=DestroyString(directory);
               (void) CopyMagickString(value,buffer,length);
               if (is_64_bit != NULL)
                 *is_64_bit=FALSE;
@@ -1116,6 +1175,7 @@ static int NTGhostscriptGetString(const char *name,BOOL *is_64_bit,
             directory,DirectorySeparator);
           if (IsPathAccessible(buffer) != MagickFalse)
             {
+              directory=DestroyString(directory);
               (void) CopyMagickString(value,buffer,length);
               if (is_64_bit != NULL)
                 *is_64_bit=TRUE;
@@ -1261,16 +1321,29 @@ MagickPrivate int NTGhostscriptEXE(char *path,int length)
     is_64_bit_version = FALSE;
 
   (void) CopyMagickString(path,"gswin32c.exe",length);
-  if ((*program == '\0') &&
-      (NTGhostscriptGetString("GS_DLL",&is_64_bit_version,program,sizeof(program)) == FALSE))
-    return(FALSE);
-  p=strrchr(program,'\\');
-  if (p != (char *) NULL)
+  if (*program == '\0')
     {
-      p++;
-      *p='\0';
-      (void) ConcatenateMagickString(program,is_64_bit_version ?
-        "gswin64c.exe" : "gswin32c.exe",sizeof(program));
+      if (ghost_semaphore == (SemaphoreInfo *) NULL)
+        ActivateSemaphoreInfo(&ghost_semaphore);
+      LockSemaphoreInfo(ghost_semaphore);
+      if (*program == '\0')
+        {
+          if (NTGhostscriptGetString("GS_DLL",&is_64_bit_version,program,
+              sizeof(program)) == FALSE)
+            {
+              UnlockSemaphoreInfo(ghost_semaphore);
+              return(FALSE);
+            }
+          p=strrchr(program,'\\');
+          if (p != (char *) NULL)
+            {
+              p++;
+              *p='\0';
+              (void) ConcatenateMagickString(program,is_64_bit_version ?
+                "gswin64c.exe" : "gswin32c.exe",sizeof(program));
+            }
+        }
+      UnlockSemaphoreInfo(ghost_semaphore);
     }
   (void) CopyMagickString(path,program,length);
   return(TRUE);
@@ -1358,6 +1431,19 @@ MagickPrivate int NTGhostscriptFonts(char *path,int length)
 %      int NTGhostscriptLoadDLL(void)
 %
 */
+static inline int NTGhostscriptHasValidHandle()
+{
+  if ((nt_ghost_info.delete_instance == NULL) || (ghost_info.exit == NULL) ||
+    (ghost_info.init_with_args == NULL) ||
+    (nt_ghost_info.new_instance == NULL) ||
+    (ghost_info.run_string == NULL) || (ghost_info.set_stdio == NULL) ||
+    (ghost_info.revision == NULL))
+  {
+    return(FALSE);
+  }
+  return(TRUE);
+}
+
 MagickPrivate int NTGhostscriptLoadDLL(void)
 {
   char
@@ -1369,7 +1455,7 @@ MagickPrivate int NTGhostscriptLoadDLL(void)
   if (ghost_handle != (void *) NULL)
     {
       UnlockSemaphoreInfo(ghost_semaphore);
-      return(TRUE);
+      return(NTGhostscriptHasValidHandle());
     }
   if (NTGhostscriptDLL(path,sizeof(path)) == FALSE)
     {
@@ -1382,28 +1468,29 @@ MagickPrivate int NTGhostscriptLoadDLL(void)
       UnlockSemaphoreInfo(ghost_semaphore);
       return(FALSE);
     }
-  (void) ResetMagickMemory((void *) &ghost_info,0,sizeof(GhostInfo));
-  ghost_info.delete_instance=(void (MagickDLLCall *) (gs_main_instance *)) (
+  (void) ResetMagickMemory((void *) &nt_ghost_info,0,sizeof(NTGhostInfo));
+  nt_ghost_info.delete_instance=(void (MagickDLLCall *)(gs_main_instance *)) (
     lt_dlsym(ghost_handle,"gsapi_delete_instance"));
+  nt_ghost_info.new_instance=(int (MagickDLLCall *)(gs_main_instance **,
+    void *)) (lt_dlsym(ghost_handle,"gsapi_new_instance"));
+  nt_ghost_info.has_instance=MagickFalse;
+  (void) ResetMagickMemory((void *) &ghost_info,0,sizeof(GhostInfo));
+  ghost_info.delete_instance=NTGhostscriptDeleteInstance;
   ghost_info.exit=(int (MagickDLLCall *)(gs_main_instance*))
     lt_dlsym(ghost_handle,"gsapi_exit");
   ghost_info.init_with_args=(int (MagickDLLCall *)(gs_main_instance *,int,
     char **)) (lt_dlsym(ghost_handle,"gsapi_init_with_args"));
-  ghost_info.new_instance=(int (MagickDLLCall *)(gs_main_instance **,void *)) (
-    lt_dlsym(ghost_handle,"gsapi_new_instance"));
+  ghost_info.new_instance=NTGhostscriptNewInstance;
   ghost_info.run_string=(int (MagickDLLCall *)(gs_main_instance *,const char *,
     int,int *)) (lt_dlsym(ghost_handle,"gsapi_run_string"));
   ghost_info.set_stdio=(int (MagickDLLCall *)(gs_main_instance *,int(
     MagickDLLCall *)(void *,char *,int),int(MagickDLLCall *)(void *,
     const char *,int),int(MagickDLLCall *)(void *,const char *,int)))
     (lt_dlsym(ghost_handle,"gsapi_set_stdio"));
+  ghost_info.revision=(int (MagickDLLCall *)(gsapi_revision_t *,int)) (
+    lt_dlsym(ghost_handle,"gsapi_revision"));
   UnlockSemaphoreInfo(ghost_semaphore);
-  if ((ghost_info.delete_instance == NULL) || (ghost_info.exit == NULL) ||
-      (ghost_info.init_with_args == NULL) || (ghost_info.new_instance == NULL)
-      || (ghost_info.run_string == NULL) || (ghost_info.set_stdio == NULL)
-      )
-    return(FALSE);
-  return(TRUE);
+  return(NTGhostscriptHasValidHandle());
 }
 
 /*
@@ -1467,6 +1554,42 @@ MagickPrivate int NTInitializeLibrary(void)
 {
   return(0);
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   N T I n i t i a l i z e W i n s o c k                                     %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  NTInitializeWinsock() initializes Winsock.
+%
+%  The format of the NTInitializeWinsock method is:
+%
+%      void NTInitializeWinsock(void)
+%
+*/
+MagickPrivate void NTInitializeWinsock(MagickBooleanType use_lock)
+{
+  if (use_lock)
+    {
+      if (winsock_semaphore == (SemaphoreInfo *) NULL)
+        ActivateSemaphoreInfo(&winsock_semaphore);
+      LockSemaphoreInfo(winsock_semaphore);
+    }
+  if (wsaData == (WSADATA *) NULL)
+    {
+      wsaData=(WSADATA *) AcquireMagickMemory(sizeof(WSADATA));
+      if (WSAStartup(MAKEWORD(2,2),wsaData) != 0)
+        ThrowFatalException(CacheFatalError,"WSAStartup failed");
+    }
+  if (use_lock)
+    UnlockSemaphoreInfo(winsock_semaphore);
+}
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -1581,6 +1704,7 @@ MagickPrivate DIR *NTOpenDirectory(const char *path)
     length;
 
   wchar_t
+    directory_separator[MaxTextExtent],
     file_specification[MaxTextExtent];
 
   assert(path != (const char *) NULL);
@@ -1588,8 +1712,12 @@ MagickPrivate DIR *NTOpenDirectory(const char *path)
     MaxTextExtent);
   if (length == 0)
     return((DIR *) NULL);
-  if(wcsncat(file_specification,(const wchar_t*) DirectorySeparator,
-       MaxTextExtent-wcslen(file_specification)-1) == (wchar_t*)NULL)
+  length=MultiByteToWideChar(CP_UTF8,0,DirectorySeparator,-1,
+    directory_separator,strlen(DirectorySeparator)+1);
+  if (length == 0)
+    return((DIR *) NULL);
+  if (wcsncat(file_specification,directory_separator,
+        MaxTextExtent-wcslen(file_specification)-1) == (wchar_t *) NULL)
     return((DIR *) NULL);
   entry=(DIR *) AcquireMagickMemory(sizeof(DIR));
   if (entry != (DIR *) NULL)
@@ -1600,7 +1728,7 @@ MagickPrivate DIR *NTOpenDirectory(const char *path)
   if (entry->hSearch == INVALID_HANDLE_VALUE)
     {
       if(wcsncat(file_specification,L"*.*",
-        MaxTextExtent-wcslen(file_specification)-1) == (wchar_t*)NULL)
+        MaxTextExtent-wcslen(file_specification)-1) == (wchar_t*) NULL)
         {
           entry=(DIR *) RelinquishMagickMemory(entry);
           return((DIR *) NULL);
@@ -2261,7 +2389,7 @@ MagickPrivate int NTSystemCommand(const char *command,char *output)
     if (PeekNamedPipe(read_output,(LPVOID) NULL,0,(LPDWORD) NULL,&size,
           (LPDWORD) NULL))
       if ((size > 0) && (ReadFile(read_output,output,MaxTextExtent-1,
-          &bytes_read,NULL))) 
+          &bytes_read,NULL)))
         output[bytes_read]='\0';
   CleanupOutputHandles;
   return((int) child_status);
@@ -2324,10 +2452,10 @@ MagickPrivate ssize_t NTSystemConfiguration(int name)
       if (module == (LPFNDLLFUNC2) NULL)
         {
           MEMORYSTATUS
-            status;
+            global_status;
 
-          GlobalMemoryStatus(&status);
-          return((ssize_t) status.dwTotalPhys/system_info.dwPageSize/4);
+          GlobalMemoryStatus(&global_status);
+          return((ssize_t) global_status.dwTotalPhys/system_info.dwPageSize/4);
         }
       status.dwLength=sizeof(status);
       if (module(&status) == 0)
@@ -2409,7 +2537,7 @@ MagickPrivate int NTTruncateFile(int file,off_t length)
     low;
 
   file_handle=(HANDLE) _get_osfhandle(file);
-  if (file_handle == (HANDLE) -1L)
+  if (file_handle == INVALID_HANDLE_VALUE)
     return(-1);
   low=(long) (length & 0xffffffffUL);
   high=(long) ((((MagickOffsetType) length) >> 32) & 0xffffffffUL);
@@ -2578,11 +2706,19 @@ MagickPrivate void NTWarningHandler(const ExceptionType severity,
 %      void NTWindowsGenesis(void)
 %
 */
+
+static LONG WINAPI NTUncaughtException(EXCEPTION_POINTERS *info)
+{
+  AsynchronousResourceComponentTerminus();
+  return(EXCEPTION_CONTINUE_SEARCH);
+}
+
 MagickPrivate void NTWindowsGenesis(void)
 {
   char
     *mode;
 
+  SetUnhandledExceptionFilter(NTUncaughtException);
   mode=GetEnvironmentValue("MAGICK_ERRORMODE");
   if (mode != (char *) NULL)
     {
@@ -2596,10 +2732,44 @@ MagickPrivate void NTWindowsGenesis(void)
         debug;
 
       debug=_CrtSetDbgFlag(_CRTDBG_REPORT_FLAG);
-      debug|=_CRTDBG_CHECK_ALWAYS_DF | _CRTDBG_DELAY_FREE_MEM_DF | _CRTDBG_LEAK_CHECK_DF;
+      debug|=_CRTDBG_CHECK_ALWAYS_DF | _CRTDBG_DELAY_FREE_MEM_DF |
+        _CRTDBG_LEAK_CHECK_DF;
       (void) _CrtSetDbgFlag(debug);
       _ASSERTE(_CrtCheckMemory());
     }
 #endif
 }
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%   N T W i n d o w s T e r m i n u s                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  NTWindowsTerminus() terminates the MagickCore Windows environment.
+%
+%  The format of the NTWindowsTerminus method is:
+%
+%      void NTWindowsTerminus(void)
+%
+*/
+MagickPrivate void NTWindowsTerminus(void)
+{
+  NTGhostscriptUnLoadDLL();
+  if (winsock_semaphore == (SemaphoreInfo *) NULL)
+    ActivateSemaphoreInfo(&winsock_semaphore);
+  LockSemaphoreInfo(winsock_semaphore);
+  if (wsaData != (WSADATA *) NULL)
+    {
+      WSACleanup();
+      wsaData=(WSADATA *) RelinquishMagickMemory((void *) wsaData);
+    }
+  UnlockSemaphoreInfo(winsock_semaphore);
+  DestroySemaphoreInfo(&winsock_semaphore);
+}
 #endif
