@@ -18,7 +18,7 @@
 %                                December 1996                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -261,7 +261,7 @@ MagickExport MagickBooleanType NTIsMagickConflict(const char *magick)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   N T A c q u i r e T y pe C a c h e                                        %
+%   N T A c q u i r e T y p e C a c h e                                       %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -410,7 +410,7 @@ MagickExport MagickBooleanType NTAcquireTypeCache(SplayTreeInfo *type_cache,
         family_extent=value_name;
         for (q=value_name; *q != '\0'; )
         {
-            GetMagickToken(q,(const char **) &q,token);
+            GetNextToken(q,(const char **) &q,MaxTextExtent,token);
             if (*token == '\0')
               break;
 
@@ -525,8 +525,7 @@ MagickExport MagickBooleanType NTAcquireTypeCache(SplayTreeInfo *type_cache,
         type_info->family=ConstantString(buffer);
 
         list_entries++;
-        status=AddValueToSplayTree(type_cache,ConstantString(type_info->name),
-          type_info);
+        status=AddValueToSplayTree(type_cache,type_info->name,type_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",type_info->name);
