@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 
   MagickCore Windows NT utility methods.
 */
-#ifndef _MAGICKCORE_NT_FEATURE_H
-#define _MAGICKCORE_NT_FEATURE_H
+#ifndef MAGICKCORE_NT_FEATURE_H
+#define MAGICKCORE_NT_FEATURE_H
 
 #include "magick/exception.h"
 #include "magick/geometry.h"
