@@ -1,5 +1,5 @@
 /*
-Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
 dedicated to making software imaging solutions freely available.
 
 You may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@ limitations under the License.
 
 MagickCore OpenCL private methods.
 */
-#ifndef _MAGICKCORE_OPENCL_PRIVATE_H
-#define _MAGICKCORE_OPENCL_PRIVATE_H
+#ifndef MAGICKCORE_OPENCL_PRIVATE_H
+#define MAGICKCORE_OPENCL_PRIVATE_H
 
 /*
 Include declarations.
@@ -29,14 +29,13 @@ extern "C" {
 #endif
 
 #if !defined(MAGICKCORE_OPENCL_SUPPORT)
-  typedef void* cl_platform_id;
-  typedef void* cl_device_id;
-  typedef void* cl_context;
-  typedef void* cl_command_queue;
-  typedef void* cl_kernel;
+  typedef void* cl_event;
   typedef void* cl_mem;
-  typedef struct { unsigned char t[8]; } cl_device_type; /* 64-bit */
+  typedef void* cl_uint;
 #else
+
+#define MAX_COMMAND_QUEUES 16
+
 /*
  *
  * function pointer typedefs
@@ -159,8 +158,9 @@ typedef CL_API_ENTRY cl_int (CL_API_CALL *MAGICKpfn_clSetKernelArg)(
     size_t       arg_size,
     const void * arg_value) CL_API_SUFFIX__VERSION_1_0;
 
-/* Flush and Finish APIs */
-typedef CL_API_ENTRY cl_int (CL_API_CALL *MAGICKpfn_clFlush)(cl_command_queue command_queue) CL_API_SUFFIX__VERSION_1_0;
+typedef CL_API_ENTRY cl_int
+  (CL_API_CALL *MAGICKpfn_clFlush)(cl_command_queue command_queue)
+    CL_API_SUFFIX__VERSION_1_0;
 
 typedef CL_API_ENTRY cl_int (CL_API_CALL *MAGICKpfn_clFinish)(cl_command_queue command_queue) CL_API_SUFFIX__VERSION_1_0;
 
@@ -176,17 +176,6 @@ typedef CL_API_ENTRY cl_int (CL_API_CALL *MAGICKpfn_clEnqueueReadBuffer)(
     const cl_event *    event_wait_list,
     cl_event *          event) CL_API_SUFFIX__VERSION_1_0;
 
-typedef CL_API_ENTRY cl_int (CL_API_CALL *MAGICKpfn_clEnqueueWriteBuffer)(
-    cl_command_queue   command_queue, 
-    cl_mem             buffer, 
-    cl_bool            blocking_write, 
-    size_t             offset, 
-    size_t             cb, 
-    const void *       ptr, 
-    cl_uint            num_events_in_wait_list, 
-    const cl_event *   event_wait_list, 
-    cl_event *         event) CL_API_SUFFIX__VERSION_1_0;
-
 typedef CL_API_ENTRY void * (CL_API_CALL *MAGICKpfn_clEnqueueMapBuffer)(
     cl_command_queue command_queue,
     cl_mem           buffer,
@@ -218,6 +207,28 @@ typedef CL_API_ENTRY cl_int (CL_API_CALL *MAGICKpfn_clEnqueueNDRangeKernel)(
     const cl_event * event_wait_list,
     cl_event *       event) CL_API_SUFFIX__VERSION_1_0;
 
+typedef CL_API_ENTRY cl_int(CL_API_CALL *MAGICKpfn_clGetEventProfilingInfo)(
+    cl_event event,
+    cl_profiling_info param_name,
+    size_t param_value_size,
+    void *param_value,
+    size_t *param_value_size_ret) CL_API_SUFFIX__VERSION_1_0;
+
+typedef CL_API_ENTRY cl_int(CL_API_CALL *MAGICKpfn_clWaitForEvents)(
+    cl_uint num_events,
+    const cl_event *event_list) CL_API_SUFFIX__VERSION_1_0;
+
+typedef CL_API_ENTRY cl_int(CL_API_CALL *MAGICKpfn_clReleaseEvent)(
+    cl_event event) CL_API_SUFFIX__VERSION_1_0;
+
+typedef CL_API_ENTRY cl_int(CL_API_CALL *MAGICKpfn_clRetainEvent)(
+    cl_event event) CL_API_SUFFIX__VERSION_1_0;
+
+typedef CL_API_ENTRY cl_int(CL_API_CALL *MAGICKpfn_clSetEventCallback)(
+    cl_event event,cl_int command_exec_callback_type,
+    void (CL_CALLBACK *MAGICKpfn_notify)(cl_event,cl_int,void *),
+    void *user_data) CL_API_SUFFIX__VERSION_1_1;
+
 /*
  *
  * vendor dispatch table structure
@@ -233,31 +244,44 @@ struct MagickLibraryRec
 {
   void * base;
 
-  MAGICKpfn_clGetPlatformIDs                         clGetPlatformIDs;
-  MAGICKpfn_clGetPlatformInfo                        clGetPlatformInfo;
-  MAGICKpfn_clGetDeviceIDs                           clGetDeviceIDs;
-  MAGICKpfn_clGetDeviceInfo                          clGetDeviceInfo;
-  MAGICKpfn_clCreateContext                          clCreateContext;
-  MAGICKpfn_clCreateCommandQueue                     clCreateCommandQueue;
-  MAGICKpfn_clReleaseCommandQueue                    clReleaseCommandQueue;
-  MAGICKpfn_clCreateBuffer                           clCreateBuffer;
-  MAGICKpfn_clReleaseMemObject                       clReleaseMemObject;
-  MAGICKpfn_clCreateProgramWithSource                clCreateProgramWithSource;
-  MAGICKpfn_clCreateProgramWithBinary                clCreateProgramWithBinary;
-  MAGICKpfn_clReleaseProgram                         clReleaseProgram;
-  MAGICKpfn_clBuildProgram                           clBuildProgram;
-  MAGICKpfn_clGetProgramInfo                         clGetProgramInfo;
-  MAGICKpfn_clGetProgramBuildInfo                    clGetProgramBuildInfo;
-  MAGICKpfn_clCreateKernel                           clCreateKernel;
-  MAGICKpfn_clReleaseKernel                          clReleaseKernel;
-  MAGICKpfn_clSetKernelArg                           clSetKernelArg;
-  MAGICKpfn_clFlush                                  clFlush;
-  MAGICKpfn_clFinish                                 clFinish;
-  MAGICKpfn_clEnqueueReadBuffer                      clEnqueueReadBuffer;
-  MAGICKpfn_clEnqueueWriteBuffer                     clEnqueueWriteBuffer;
-  MAGICKpfn_clEnqueueMapBuffer                       clEnqueueMapBuffer;
-  MAGICKpfn_clEnqueueUnmapMemObject                  clEnqueueUnmapMemObject;
-  MAGICKpfn_clEnqueueNDRangeKernel                   clEnqueueNDRangeKernel;
+  MAGICKpfn_clGetPlatformIDs          clGetPlatformIDs;
+  MAGICKpfn_clGetPlatformInfo         clGetPlatformInfo;
+
+  MAGICKpfn_clGetDeviceIDs            clGetDeviceIDs;
+  MAGICKpfn_clGetDeviceInfo           clGetDeviceInfo;
+
+  MAGICKpfn_clCreateContext           clCreateContext;
+  MAGICKpfn_clReleaseContext          clReleaseContext;
+
+  MAGICKpfn_clCreateCommandQueue      clCreateCommandQueue;
+  MAGICKpfn_clReleaseCommandQueue     clReleaseCommandQueue;
+  MAGICKpfn_clFlush                   clFlush;
+  MAGICKpfn_clFinish                  clFinish;
+
+  MAGICKpfn_clCreateBuffer            clCreateBuffer;
+  MAGICKpfn_clReleaseMemObject        clReleaseMemObject;
+  MAGICKpfn_clCreateProgramWithSource clCreateProgramWithSource;
+  MAGICKpfn_clCreateProgramWithBinary clCreateProgramWithBinary;
+  MAGICKpfn_clReleaseProgram          clReleaseProgram;
+  MAGICKpfn_clBuildProgram            clBuildProgram;
+  MAGICKpfn_clGetProgramInfo          clGetProgramInfo;
+  MAGICKpfn_clGetProgramBuildInfo     clGetProgramBuildInfo;
+
+  MAGICKpfn_clCreateKernel            clCreateKernel;
+  MAGICKpfn_clReleaseKernel           clReleaseKernel;
+  MAGICKpfn_clSetKernelArg            clSetKernelArg;
+
+  MAGICKpfn_clEnqueueReadBuffer       clEnqueueReadBuffer;
+  MAGICKpfn_clEnqueueMapBuffer        clEnqueueMapBuffer;
+  MAGICKpfn_clEnqueueUnmapMemObject   clEnqueueUnmapMemObject;
+  MAGICKpfn_clEnqueueNDRangeKernel    clEnqueueNDRangeKernel;
+
+  MAGICKpfn_clGetEventProfilingInfo   clGetEventProfilingInfo;
+
+  MAGICKpfn_clWaitForEvents           clWaitForEvents;
+  MAGICKpfn_clReleaseEvent            clReleaseEvent;
+  MAGICKpfn_clRetainEvent             clRetainEvent;
+  MAGICKpfn_clSetEventCallback        clSetEventCallback;
 };
 
 struct _MagickCLEnv {
@@ -277,16 +301,13 @@ struct _MagickCLEnv {
 
   MagickBooleanType regenerateProfile;   /* re-run the microbenchmark in auto device selection mode */ 
 
-  /* Random number generator seeds */
-  unsigned int numGenerators;
-  float randNormalize;
-  cl_mem seeds;
-  SemaphoreInfo* seedsLock;
-
   SemaphoreInfo* lock;
+
+  cl_command_queue commandQueues[MAX_COMMAND_QUEUES];
+  ssize_t commandQueuesPos;
+  SemaphoreInfo* commandQueuesLock;
 };
 
-#endif
 
 #if defined(MAGICKCORE_HDRI_SUPPORT)
 #define CLOptions "-cl-single-precision-constant -cl-mad-enable -DMAGICKCORE_HDRI_SUPPORT=1 "\
@@ -325,54 +346,68 @@ struct _MagickCLEnv {
 #define CLCharQuantumScale 72340172838076673.0f
 #endif
 
-extern MagickPrivate cl_context 
+typedef enum {
+  AddNoiseKernel,
+  BlurRowKernel,
+  BlurColumnKernel,
+  CompositeKernel,
+  ComputeFunctionKernel,
+  ContrastKernel,
+  ContrastStretchKernel,
+  ConvolveKernel,
+  EqualizeKernel,
+  GrayScaleKernel,
+  HistogramKernel,
+  HullPass1Kernel,
+  HullPass2Kernel,
+  LocalContrastBlurRowKernel,
+  LocalContrastBlurApplyColumnKernel,
+  ModulateKernel,
+  MotionBlurKernel,
+  RadialBlurKernel,
+  RandomNumberGeneratorKernel,
+  ResizeHorizontalKernel,
+  ResizeVerticalKernel,
+  UnsharpMaskBlurColumnKernel,
+  UnsharpMaskKernel,
+  WaveletDenoiseKernel,
+  KERNEL_COUNT
+} ProfiledKernels;
+
+extern MagickPrivate cl_context
   GetOpenCLContext(MagickCLEnv);
 
-extern MagickPrivate cl_kernel 
+extern MagickPrivate cl_kernel
   AcquireOpenCLKernel(MagickCLEnv, MagickOpenCLProgram, const char*);
 
-extern MagickPrivate cl_command_queue 
+extern MagickPrivate cl_command_queue
   AcquireOpenCLCommandQueue(MagickCLEnv);
 
-extern MagickPrivate MagickBooleanType 
+extern MagickPrivate MagickBooleanType
   OpenCLThrowMagickException(ExceptionInfo *,
     const char *,const char *,const size_t,
     const ExceptionType,const char *,const char *,...),
+  RecordProfileData(MagickCLEnv,ProfiledKernels,cl_event),
+  RelinquishMagickOpenCLEnv(MagickCLEnv),
   RelinquishOpenCLCommandQueue(MagickCLEnv, cl_command_queue),
   RelinquishOpenCLKernel(MagickCLEnv, cl_kernel);
 
-extern MagickPrivate unsigned long 
+extern MagickPrivate MagickCLEnv
+  AcquireMagickOpenCLEnv(),
+  SetDefaultOpenCLEnv(MagickCLEnv);
+
+extern MagickPrivate unsigned long
   GetOpenCLDeviceLocalMemorySize(MagickCLEnv),
   GetOpenCLDeviceMaxMemAllocSize(MagickCLEnv);
 
-extern MagickPrivate const char* 
+extern MagickPrivate const char*
   GetOpenCLCachedFilesDirectory();
 
 extern MagickPrivate void
-  UnlockRandSeedBuffer(MagickCLEnv),
   OpenCLLog(const char*);
 
-extern MagickPrivate cl_mem 
-  GetAndLockRandSeedBuffer(MagickCLEnv);
-
-extern MagickPrivate unsigned int 
-  GetNumRandGenerators(MagickCLEnv);
-
-extern MagickPrivate float 
-  GetRandNormalize(MagickCLEnv clEnv);
-
-typedef struct _AccelerateTimer {
-  long long _freq;	
-  long long _clocks;
-  long long _start;
-} AccelerateTimer;
-
-
-void startAccelerateTimer(AccelerateTimer* timer);
-void stopAccelerateTimer(AccelerateTimer* timer);
-void resetAccelerateTimer(AccelerateTimer* timer);
-void initAccelerateTimer(AccelerateTimer* timer);
-double readAccelerateTimer(AccelerateTimer* timer);
+extern MagickPrivate void
+  OpenCLTerminus();
 
 /* #define OPENCLLOG_ENABLED 1 */
 static inline void OpenCLLogException(const char* function, 
@@ -392,7 +427,7 @@ static inline void OpenCLLogException(const char* function,
   magick_unreferenced(exception);
 #endif
 }
-
+#endif
 
 #if defined(__cplusplus) || defined(c_plusplus)
 }
