@@ -16,7 +16,7 @@
 %                              December 2001                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -37,8 +37,7 @@
 %
 %
 */
-
-
+
 /*
   Include declarations.
 */
@@ -51,6 +50,7 @@
 #include "magick/studio.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/image-private.h"
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
 #include "magick/random_.h"
@@ -69,8 +69,7 @@
 #define RandomFiletype  "random"
 #define RandomProtocolMajorVersion  1
 #define RandomProtocolMinorVersion  0
-
-
+
 /*
   Typedef declarations.
 */
@@ -108,8 +107,7 @@ struct _RandomInfo
   size_t
     signature;
 };
-
-
+
 /*
   External declarations.
 */
@@ -122,8 +120,7 @@ struct _RandomInfo
 extern char
   **environ;
 #endif
-
-
+
 /*
   Global declarations.
 */
@@ -135,15 +132,13 @@ static unsigned long
 
 static MagickBooleanType
   gather_true_random = MagickFalse;
-
-
+
 /*
   Forward declarations.
 */
 static StringInfo
   *GenerateEntropicChaos(RandomInfo *);
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -162,14 +157,6 @@ static StringInfo
 %      RandomInfo *AcquireRandomInfo(void)
 %
 */
-
-static inline size_t MagickMin(const size_t x,const size_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 MagickExport RandomInfo *AcquireRandomInfo(void)
 {
   const StringInfo
@@ -258,8 +245,7 @@ MagickExport RandomInfo *AcquireRandomInfo(void)
   random_info->seed[3]=0xc3a4171aUL;
   return(random_info);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -303,8 +289,7 @@ MagickExport RandomInfo *DestroyRandomInfo(RandomInfo *random_info)
   random_info=(RandomInfo *) RelinquishMagickMemory(random_info);
   return(random_info);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -393,6 +378,17 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
   SetStringInfoLength(chaos,sizeof(tid));
   SetStringInfoDatum(chaos,(unsigned char *) &tid);
   ConcatenateStringInfo(entropy,chaos);
+#if defined(MAGICKCORE_HAVE_SYSCONF) && defined(_SC_PHYS_PAGES)
+  {
+    ssize_t
+      pages;
+
+    pages=(ssize_t) sysconf(_SC_PHYS_PAGES);
+    SetStringInfoLength(chaos,sizeof(pages));
+    SetStringInfoDatum(chaos,(unsigned char *) &pages);
+    ConcatenateStringInfo(entropy,chaos);
+  }
+#endif
 #if defined(MAGICKCORE_HAVE_GETRUSAGE) && defined(RUSAGE_SELF)
   {
     struct rusage
@@ -463,11 +459,16 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
 
     (void) GetPathTemplate(path);
     file=mkstemp(path);
-#if defined(__OS2__)
-    setmode(file,O_BINARY);
-#endif
     if (file != -1)
-      (void) close(file);
+      {
+#if defined(MAGICKCORE_HAVE_FCHMOD)
+        (void) fchmod(file,0600);
+#endif
+#if defined(__OS2__)
+        setmode(file,O_BINARY);
+#endif
+        (void) close(file);
+      }
     (void) remove_utf8(path);
     SetStringInfoLength(chaos,strlen(path));
     SetStringInfoDatum(chaos,(unsigned char *) path);
@@ -583,8 +584,7 @@ static StringInfo *GenerateEntropicChaos(RandomInfo *random_info)
   UnlockSemaphoreInfo(random_info->semaphore);
   return(entropy);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -628,8 +628,63 @@ MagickExport double GetPseudoRandomValue(RandomInfo *random_info)
   } while (seed[0] == ~0UL);
   return(random_info->normalize*seed[0]);
 }
-
-
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   G e t R a n d o m I n f o N o r m a l i z e                               %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetRandomInfoNormalize() returns the random normalize value.
+%
+%  The format of the GetRandomInfoNormalize method is:
+%
+%      double GetRandomInfoNormalize(const RandomInfo *random_info)
+%
+%  A description of each parameter follows:
+%
+%    o random_info: the random info.
+%
+*/
+MagickPrivate double GetRandomInfoNormalize(const RandomInfo *random_info)
+{
+  assert(random_info != (const RandomInfo *) NULL);
+  return(random_info->normalize);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
++   G e t R a n d o m I n f o S e e d                                         %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetRandomInfoSeed() returns the random seed.
+%
+%  The format of the GetRandomInfoSeed method is:
+%
+%      unsigned long *GetRandomInfoSeed(RandomInfo *random_info)
+%
+%  A description of each parameter follows:
+%
+%    o random_info: the random info.
+%
+*/
+MagickPrivate unsigned long *GetRandomInfoSeed(RandomInfo *random_info)
+{
+  assert(random_info != (RandomInfo *) NULL);
+  return(random_info->seed);
+}
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -665,8 +720,7 @@ MagickExport StringInfo *GetRandomKey(RandomInfo *random_info,
   SetRandomKey(random_info,length,GetStringInfoDatum(key));
   return(key);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -692,8 +746,7 @@ MagickExport unsigned long GetRandomSecretKey(const RandomInfo *random_info)
 {
   return(random_info->secret_key);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -727,8 +780,7 @@ MagickExport double GetRandomValue(RandomInfo *random_info)
   } while (key == range);
   return((double) key/range);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -753,8 +805,7 @@ MagickExport MagickBooleanType RandomComponentGenesis(void)
     random_semaphore=AllocateSemaphoreInfo();
   return(MagickTrue);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -779,8 +830,7 @@ MagickExport void RandomComponentTerminus(void)
     ActivateSemaphoreInfo(&random_semaphore);
   DestroySemaphoreInfo(&random_semaphore);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -881,8 +931,7 @@ MagickExport void SetRandomKey(RandomInfo *random_info,const size_t length,
     }
   UnlockSemaphoreInfo(random_info->semaphore);
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -915,8 +964,7 @@ MagickExport void SetRandomSecretKey(const unsigned long key)
 {
   secret_key=key;
 }
-
-
+
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %                                                                             %
@@ -945,16 +993,3 @@ MagickExport void SetRandomTrueRandom(const MagickBooleanType true_random)
 {
   gather_true_random=true_random;
 }
-
-MagickPrivate unsigned long* GetRandomInfoSeed(RandomInfo *random_info)
-{
-  assert(random_info != (RandomInfo *) NULL);
-  return random_info->seed;
-}
-
-MagickPrivate double GetRandomInfoNormalize(RandomInfo *random_info)
-{
-  assert(random_info != (RandomInfo *) NULL);
-  return random_info->normalize;
-}
-
