@@ -17,7 +17,7 @@
 %                                  July 1992                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -34,7 +34,7 @@
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
 %  The XShearImage() and YShearImage() methods are based on the paper "A Fast
-%  Algorithm for General Raster Rotatation" by Alan W. Paeth, Graphics
+%  Algorithm for General Raster Rotation" by Alan W. Paeth, Graphics
 %  Interface '86 (Vancouver).  ShearRotateImage() is adapted from a similar
 %  method based on the Paeth paper written by Michael Halle of the Spatial
 %  Imaging Group, MIT Media Lab.
@@ -78,6 +78,7 @@
 #include "magick/string-private.h"
 #include "magick/thread-private.h"
 #include "magick/threshold.h"
+#include "magick/token.h"
 #include "magick/transform.h"
 
 /*
@@ -198,9 +199,7 @@ static MagickBooleanType CropToFitImage(Image **image,
 %  artifact "deskew:angle".
 %
 %  If the artifact "deskew:auto-crop" is given the image will be automatically
-%  cropped of the excess background.  The value is the border width of all
-%  pixels around the edge that will be used to determine an average border
-%  color for the automatic trim.
+%  cropped of the excess background.
 %
 %  The format of the DeskewImage method is:
 %
@@ -391,7 +390,7 @@ static MagickBooleanType RadonTransform(const Image *image,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       i,
@@ -448,7 +447,7 @@ static MagickBooleanType RadonTransform(const Image *image,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       i,
@@ -529,7 +528,7 @@ static void GetImageBackgroundColor(Image *image,const ssize_t offset,
   for (y=0; y < (ssize_t) image->rows; y++)
   {
     register const PixelPacket
-      *restrict p;
+      *magick_restrict p;
 
     register ssize_t
       x;
@@ -647,7 +646,7 @@ MagickExport Image *DeskewImage(const Image *image,const double threshold,
   affine_matrix.tx=0.0;
   affine_matrix.ty=0.0;
   artifact=GetImageArtifact(image,"deskew:auto-crop");
-  if (artifact == (const char *) NULL)
+  if (IsMagickTrue(artifact) == MagickFalse)
     {
       deskew_image=AffineTransformImage(clone_image,&affine_matrix,exception);
       clone_image=DestroyImage(clone_image);
@@ -786,16 +785,16 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
             sync;
 
           register const IndexPacket
-            *restrict indexes;
+            *magick_restrict indexes;
 
           register const PixelPacket
-            *restrict p;
+            *magick_restrict p;
 
           register IndexPacket
-            *restrict rotate_indexes;
+            *magick_restrict rotate_indexes;
 
           register PixelPacket
-            *restrict q;
+            *magick_restrict q;
 
           register ssize_t
             y;
@@ -821,7 +820,7 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
           for (y=0; y < (ssize_t) width; y++)
           {
             register const PixelPacket
-              *restrict tile_pixels;
+              *magick_restrict tile_pixels;
 
             register ssize_t
               x;
@@ -847,7 +846,7 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
                 (rotate_indexes != (IndexPacket *) NULL))
               {
                 register const IndexPacket
-                  *restrict tile_indexes;
+                  *magick_restrict tile_indexes;
 
                 tile_indexes=indexes+(height-1)*width+y;
                 for (x=0; x < (ssize_t) height; x++)
@@ -898,16 +897,16 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
           sync;
 
         register const IndexPacket
-          *restrict indexes;
+          *magick_restrict indexes;
 
         register const PixelPacket
-          *restrict p;
+          *magick_restrict p;
 
         register IndexPacket
-          *restrict rotate_indexes;
+          *magick_restrict rotate_indexes;
 
         register PixelPacket
-          *restrict q;
+          *magick_restrict q;
 
         register ssize_t
           x;
@@ -987,16 +986,16 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
             sync;
 
           register const IndexPacket
-            *restrict indexes;
+            *magick_restrict indexes;
 
           register const PixelPacket
-            *restrict p;
+            *magick_restrict p;
 
           register IndexPacket
-            *restrict rotate_indexes;
+            *magick_restrict rotate_indexes;
 
           register PixelPacket
-            *restrict q;
+            *magick_restrict q;
 
           register ssize_t
             y;
@@ -1022,7 +1021,7 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
           for (y=0; y < (ssize_t) width; y++)
           {
             register const PixelPacket
-              *restrict tile_pixels;
+              *magick_restrict tile_pixels;
 
             register ssize_t
               x;
@@ -1047,7 +1046,7 @@ MagickExport Image *IntegralRotateImage(const Image *image,size_t rotations,
                 (rotate_indexes != (IndexPacket *) NULL))
               {
                 register const IndexPacket
-                  *restrict tile_indexes;
+                  *magick_restrict tile_indexes;
 
                 tile_indexes=indexes+(width-1)-y;
                 for (x=0; x < (ssize_t) height; x++)
@@ -1189,12 +1188,12 @@ static MagickBooleanType XShearImage(Image *image,const MagickRealType degrees,
       displacement;
 
     register IndexPacket
-      *restrict indexes,
-      *restrict shear_indexes;
+      *magick_restrict indexes,
+      *magick_restrict shear_indexes;
 
     register PixelPacket
-      *restrict p,
-      *restrict q;
+      *magick_restrict p,
+      *magick_restrict q;
 
     register ssize_t
       i;
@@ -1413,15 +1412,15 @@ static MagickBooleanType YShearImage(Image *image,const MagickRealType degrees,
       displacement;
 
     register IndexPacket
-      *restrict indexes,
-      *restrict shear_indexes;
+      *magick_restrict indexes,
+      *magick_restrict shear_indexes;
 
     register ssize_t
       i;
 
     register PixelPacket
-      *restrict p,
-      *restrict q;
+      *magick_restrict p,
+      *magick_restrict q;
 
     ShearDirection
       direction;
@@ -1661,6 +1660,7 @@ MagickExport Image *ShearImage(const Image *image,const double x_shear,
     }
   status=CropToFitImage(&shear_image,shear.x,shear.y,(MagickRealType)
     image->columns,(MagickRealType) image->rows,MagickFalse,exception);
+  shear_image->matte=image->matte;
   shear_image->compose=image->compose;
   shear_image->page.width=0;
   shear_image->page.height=0;
@@ -1817,6 +1817,7 @@ MagickExport Image *ShearRotateImage(const Image *image,const double degrees,
     }
   status=CropToFitImage(&rotate_image,shear.x,shear.y,(MagickRealType) width,
     (MagickRealType) height,MagickTrue,exception);
+  rotate_image->matte=image->matte;
   rotate_image->compose=image->compose;
   rotate_image->page.width=0;
   rotate_image->page.height=0;
