@@ -17,7 +17,7 @@
 %                                 March 2000                                  %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -211,6 +211,9 @@ MagickExport void RegisterStaticModules(void)
   (void) RegisterFAXImage();
   (void) RegisterFDImage();
   (void) RegisterFITSImage();
+#if defined(MAGICKCORE_FLIF_DELEGATE)
+  (void) RegisterFLIFImage();
+#endif
 #if defined(MAGICKCORE_FPX_DELEGATE)
   (void) RegisterFPXImage();
 #endif
@@ -399,6 +402,9 @@ MagickExport void UnregisterStaticModules(void)
   UnregisterFAXImage();
   UnregisterFDImage();
   UnregisterFITSImage();
+#if defined(MAGICKCORE_FLIF_DELEGATE)
+  UnregisterFLIFImage();
+#endif
 #if defined(MAGICKCORE_FPX_DELEGATE)
   UnregisterFPXImage();
 #endif
