@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -51,11 +51,8 @@
 /*
   Define declarations.
 */
-#if defined(macintosh)
-#define CLK_TCK  CLOCKS_PER_SEC
-#endif
-#if !defined(CLK_TCK)
-#define CLK_TCK  sysconf(_SC_CLK_TCK)
+#if !defined(CLOCKS_PER_SEC)
+#define CLOCKS_PER_SEC  100
 #endif
 
 /*
@@ -191,16 +188,35 @@ MagickExport TimerInfo *DestroyTimerInfo(TimerInfo *timer_info)
 */
 static double ElapsedTime(void)
 {
-#if defined(MAGICKCORE_HAVE_TIMES)
+#if defined(HAVE_CLOCK_GETTIME)
+#define NANOSECONDS_PER_SECOND  1000000000.0
+#if defined(CLOCK_HIGHRES)
+#  define CLOCK_ID CLOCK_HIGHRES
+#elif defined(CLOCK_MONOTONIC_RAW)
+#  define CLOCK_ID CLOCK_MONOTONIC_RAW
+#elif defined(CLOCK_MONOTONIC_PRECISE)
+#  define CLOCK_ID CLOCK_MONOTONIC_PRECISE
+#elif defined(CLOCK_MONOTONIC)
+#  define CLOCK_ID CLOCK_MONOTONIC
+#else
+#  define CLOCK_ID CLOCK_REALTIME
+#endif
+
+  struct timespec 
+    timer;
+
+  (void) clock_gettime(CLOCK_ID,&timer);
+  return((double) timer.tv_sec+timer.tv_nsec/NANOSECONDS_PER_SECOND);
+#elif defined(MAGICKCORE_HAVE_TIMES) && defined(MAGICKCORE_HAVE_SYSCONF)
   struct tms
     timer;
 
-  return((double) times(&timer)/CLK_TCK);
+  return((double) times(&timer)/sysconf(_SC_CLK_TCK));
 #else
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
   return(NTElapsedTime());
 #else
-  return((double) clock()/CLK_TCK);
+  return((double) clock()/CLOCKS_PER_SEC);
 #endif
 #endif
 }
@@ -445,17 +461,17 @@ static void StopTimer(TimerInfo *time_info)
 */
 static double UserTime(void)
 {
-#if defined(MAGICKCORE_HAVE_TIMES)
+#if defined(MAGICKCORE_HAVE_TIMES) && defined(MAGICKCORE_HAVE_SYSCONF)
   struct tms
     timer;
 
   (void) times(&timer);
-  return((double) (timer.tms_utime+timer.tms_stime)/CLK_TCK);
+  return((double) (timer.tms_utime+timer.tms_stime)/sysconf(_SC_CLK_TCK));
 #else
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
   return(NTUserTime());
 #else
-  return((double) clock()/CLK_TCK);
+  return((double) clock()/CLOCKS_PER_SEC);
 #endif
 #endif
 }
