@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -140,21 +140,22 @@ MagickExport TokenInfo *DestroyTokenInfo(TokenInfo *token_info)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   G e t M a g i c k T o k e n                                               %
++   G e t N e x t T o k e n                                                   %
 %                                                                             %
 %                                                                             %
 %                                                                             %
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
 %
-%  GetMagickToken() gets a token from the token stream.  A token is defined as
+%  GetNextToken() gets a token from the token stream.  A token is defined as
 %  a sequence of characters delimited by whitespace (e.g. clip-path), a
 %  sequence delimited with quotes (.e.g "Quote me"), or a sequence enclosed in
-%  parenthesis (e.g. rgb(0,0,0)).  GetMagickToken() also recognizes these
+%  parenthesis (e.g. rgb(0,0,0)).  GetNextToken() also recognizes these
 %  separator characters: ':', '=', ',', and ';'.
 %
-%  The format of the GetMagickToken method is:
+%  The format of the GetNextToken method is:
 %
-%      void GetMagickToken(const char *start,const char **end,char *token)
+%      void GetNextToken(const char *start,const char **end,
+%        const size_t extent,char *token)
 %
 %  A description of each parameter follows:
 %
@@ -162,10 +163,13 @@ MagickExport TokenInfo *DestroyTokenInfo(TokenInfo *token_info)
 %
 %    o end: point to the end of the token sequence.
 %
+%    o extent: maximum extent of the token.
+%
 %    o token: copy the token to this buffer.
 %
 */
-MagickExport void GetMagickToken(const char *start,const char **end,char *token)
+MagickExport void GetNextToken(const char *start,const char **end,
+  const size_t extent,char *token)
 {
   double
     value;
@@ -212,15 +216,18 @@ MagickExport void GetMagickToken(const char *start,const char **end,char *token)
               p++;
               break;
             }
-        token[i++]=(*p);
+        if (i < (ssize_t) (extent-1))
+          token[i++]=(*p);
       }
       break;
     }
     case '/':
     {
-      token[i++]=(*p++);
-      if ((*p == '>') || (*p == '/'))
+      if (i < (ssize_t) (extent-1))
         token[i++]=(*p++);
+      if ((*p == '>') || (*p == '/'))
+        if (i < (ssize_t) (extent-1))
+          token[i++]=(*p++);
       break;
     }
     default:
@@ -233,15 +240,18 @@ MagickExport void GetMagickToken(const char *start,const char **end,char *token)
       if ((p != q) && (*p != ','))
         {
           for ( ; (p < q) && (*p != ','); p++)
-            token[i++]=(*p);
+            if (i < (ssize_t) (extent-1))
+              token[i++]=(*p);
           if (*p == '%')
-            token[i++]=(*p++);
+            if (i < (ssize_t) (extent-1))
+              token[i++]=(*p++);
           break;
         }
       if ((*p != '\0') && (isalpha((int) ((unsigned char) *p)) == 0) &&
           (*p != *DirectorySeparator) && (*p != '#') && (*p != '<'))
         {
-          token[i++]=(*p++);
+          if (i < (ssize_t) (extent-1))
+            token[i++]=(*p++);
           break;
         }
       for ( ; *p != '\0'; p++)
@@ -251,13 +261,15 @@ MagickExport void GetMagickToken(const char *start,const char **end,char *token)
           break;
         if ((i > 0) && (*p == '<'))
           break;
-        token[i++]=(*p);
+        if (i < (ssize_t) (extent-1))
+          token[i++]=(*p);
         if (*p == '>')
           break;
         if (*p == '(')
           for (p++; *p != '\0'; p++)
           {
-            token[i++]=(*p);
+            if (i < (ssize_t) (extent-1))
+              token[i++]=(*p);
             if ((*p == ')') && (*(p-1) != '\\'))
               break;
           }
@@ -491,38 +503,37 @@ MagickExport MagickBooleanType GlobExpression(const char *expression,
               done=MagickTrue;
               break;
             }
+          if (match != MagickFalse)
+            {
+              expression=p;
+              while ((GetUTFCode(pattern) != '}') &&
+                     (GetUTFCode(pattern) != 0))
+              {
+                pattern+=GetUTFOctets(pattern);
+                if (GetUTFCode(pattern) == '\\')
+                  {
+                    pattern+=GetUTFOctets(pattern);
+                    if (GetUTFCode(pattern) == '}')
+                      pattern+=GetUTFOctets(pattern);
+                  }
+              }
+            }
           else
-            if (match != MagickFalse)
+            {
+              while ((GetUTFCode(pattern) != '}') &&
+                     (GetUTFCode(pattern) != ',') &&
+                     (GetUTFCode(pattern) != 0))
               {
-                expression=p;
-                while ((GetUTFCode(pattern) != '}') &&
-                       (GetUTFCode(pattern) != 0))
-                {
-                  pattern+=GetUTFOctets(pattern);
-                  if (GetUTFCode(pattern) == '\\')
-                    {
+                pattern+=GetUTFOctets(pattern);
+                if (GetUTFCode(pattern) == '\\')
+                  {
+                    pattern+=GetUTFOctets(pattern);
+                    if ((GetUTFCode(pattern) == '}') ||
+                        (GetUTFCode(pattern) == ','))
                       pattern+=GetUTFOctets(pattern);
-                      if (GetUTFCode(pattern) == '}')
-                        pattern+=GetUTFOctets(pattern);
-                    }
-                }
-              }
-            else
-              {
-                while ((GetUTFCode(pattern) != '}') &&
-                       (GetUTFCode(pattern) != ',') &&
-                       (GetUTFCode(pattern) != 0))
-                {
-                  pattern+=GetUTFOctets(pattern);
-                  if (GetUTFCode(pattern) == '\\')
-                    {
-                      pattern+=GetUTFOctets(pattern);
-                      if ((GetUTFCode(pattern) == '}') ||
-                          (GetUTFCode(pattern) == ','))
-                        pattern+=GetUTFOctets(pattern);
-                    }
-                }
+                  }
               }
+            }
             if (GetUTFCode(pattern) != 0)
               pattern+=GetUTFOctets(pattern);
           }
