@@ -17,7 +17,7 @@
 %                                 May 2001                                    %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -47,6 +47,7 @@
 #include "magick/exception.h"
 #include "magick/exception-private.h"
 #include "magick/hashmap.h"
+#include "magick/image-private.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
 #include "magick/nt-base-private.h"
@@ -190,11 +191,11 @@ static SplayTreeInfo *AcquireTypeCache(const char *filename,
     status;
 
   SplayTreeInfo
-    *type_cache;
+    *cache;
 
-  type_cache=NewSplayTree(CompareSplayTreeString,(void *(*)(void *)) NULL,
+  cache=NewSplayTree(CompareSplayTreeString,(void *(*)(void *)) NULL,
     DestroyTypeNode);
-  if (type_cache == (SplayTreeInfo *) NULL)
+  if (cache == (SplayTreeInfo *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   status=MagickTrue;
 #if !defined(MAGICKCORE_ZERO_CONFIGURATION_SUPPORT)
@@ -215,8 +216,8 @@ static SplayTreeInfo *AcquireTypeCache(const char *filename,
     while (option != (const StringInfo *) NULL)
     {
       (void) CopyMagickString(path,GetStringInfoPath(option),MaxTextExtent);
-      status&=LoadTypeCache(type_cache,(const char *)
-        GetStringInfoDatum(option),GetStringInfoPath(option),0,exception);
+      status&=LoadTypeCache(cache,(const char *) GetStringInfoDatum(option),
+        GetStringInfoPath(option),0,exception);
       option=(const StringInfo *) GetNextValueInLinkedList(options);
     }
     options=DestroyConfigureOptions(options);
@@ -234,16 +235,16 @@ static SplayTreeInfo *AcquireTypeCache(const char *filename,
         option=FileToString(path,~0UL,exception);
         if (option != (void *) NULL)
           {
-            status&=LoadTypeCache(type_cache,option,path,0,exception);
+            status&=LoadTypeCache(cache,option,path,0,exception);
             option=DestroyString(option);
           }
         font_path=DestroyString(font_path);
       }
   }
 #endif
-  if (GetNumberOfNodesInSplayTree(type_cache) == 0)
-    status&=LoadTypeCache(type_cache,TypeMap,"built-in",0,exception);
-  return(type_cache);
+  if (GetNumberOfNodesInSplayTree(cache) == 0)
+    status&=LoadTypeCache(cache,TypeMap,"built-in",0,exception);
+  return(cache);
 }
 
 /*
@@ -329,20 +330,6 @@ MagickExport const TypeInfo *GetTypeInfo(const char *name,
 %
 */
 
-static inline size_t MagickMax(const size_t x,const size_t y)
-{
-  if (x > y)
-    return(x);
-  return(y);
-}
-
-static inline size_t MagickMin(const size_t x,const size_t y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   const StyleType style,const StretchType stretch,const size_t weight,
   ExceptionInfo *exception)
@@ -380,6 +367,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
     };
 
   size_t
+    font_weight,
     max_score,
     score;
 
@@ -389,6 +377,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
   (void) GetTypeInfo("*",exception);
   if (type_cache == (SplayTreeInfo *) NULL)
     return((TypeInfo *) NULL);
+  font_weight=weight == 0 ? 400 : weight;
   LockSemaphoreInfo(type_semaphore);
   ResetSplayTreeIterator(type_cache);
   type_info=(const TypeInfo *) NULL;
@@ -426,7 +415,7 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
         p=(const TypeInfo *) GetNextValueInSplayTree(type_cache);
         continue;
       }
-    if ((weight != 0) && (p->weight != weight))
+    if (p->weight != font_weight)
       {
         p=(const TypeInfo *) GetNextValueInSplayTree(type_cache);
         continue;
@@ -473,11 +462,8 @@ MagickExport const TypeInfo *GetTypeInfoByFamily(const char *family,
       if (((style == ItalicStyle) || (style == ObliqueStyle)) &&
           ((p->style == ItalicStyle) || (p->style == ObliqueStyle)))
         score+=25;
-    if (weight == 0)
-      score+=16;
-    else
-      score+=(16*(800-((ssize_t) MagickMax(MagickMin(weight,900),p->weight)-
-        (ssize_t) MagickMin(MagickMin(weight,900),p->weight))))/800;
+    score+=(16*(800-((ssize_t) MagickMax(MagickMin(font_weight,900),p->weight)-
+      (ssize_t) MagickMin(MagickMin(font_weight,900),p->weight))))/800;
     if ((stretch == UndefinedStretch) || (stretch == AnyStretch))
       score+=8;
     else
@@ -1015,7 +1001,7 @@ MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
 %                                                                             %
 %                                                                             %
 %                                                                             %
-+   L o a d T y p e L i s t                                                   %
++   L o a d T y p e C a c h e                                                 %
 %                                                                             %
 %                                                                             %
 %                                                                             %
@@ -1026,9 +1012,8 @@ MagickExport MagickBooleanType ListTypeInfo(FILE *file,ExceptionInfo *exception)
 %
 %  The format of the LoadTypeCache method is:
 %
-%      MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
-%        const char *xml,const char *filename,const size_t depth,
-%        ExceptionInfo *exception)
+%      MagickBooleanType LoadTypeCache(SplayTreeInfo *cache,const char *xml,
+%        const char *filename,const size_t depth,ExceptionInfo *exception)
 %
 %  A description of each parameter follows:
 %
@@ -1078,9 +1063,8 @@ static inline MagickBooleanType SetTypeNodePath(const char *filename,
   return(MagickTrue);
 }
 
-static MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
-  const char *xml,const char *filename,const size_t depth,
-  ExceptionInfo *exception)
+static MagickBooleanType LoadTypeCache(SplayTreeInfo *cache,const char *xml,
+  const char *filename,const size_t depth,ExceptionInfo *exception)
 {
   char
     font_path[MaxTextExtent],
@@ -1093,6 +1077,9 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
   MagickStatusType
     status;
 
+  size_t
+    extent;
+
   TypeInfo
     *type_info;
 
@@ -1106,6 +1093,7 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
   status=MagickTrue;
   type_info=(TypeInfo *) NULL;
   token=AcquireString(xml);
+  extent=strlen(token)+MaxTextExtent;
 #if defined(MAGICKCORE_WINDOWS_SUPPORT)
   /*
     Determine the Ghostscript font path.
@@ -1119,7 +1107,7 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
     /*
       Interpret XML.
     */
-    GetMagickToken(q,&q,token);
+    GetNextToken(q,&q,extent,token);
     if (*token == '\0')
       break;
     (void) CopyMagickString(keyword,token,MaxTextExtent);
@@ -1129,7 +1117,7 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
           Doctype element.
         */
         while ((LocaleNCompare(q,"]>",2) != 0) && (*q != '\0'))
-          GetMagickToken(q,&q,token);
+          GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleNCompare(keyword,"<!--",4) == 0)
@@ -1138,7 +1126,7 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
           Comment element.
         */
         while ((LocaleNCompare(q,"->",2) != 0) && (*q != '\0'))
-          GetMagickToken(q,&q,token);
+          GetNextToken(q,&q,extent,token);
         continue;
       }
     if (LocaleCompare(keyword,"<include") == 0)
@@ -1149,10 +1137,10 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
         while (((*token != '/') && (*(token+1) != '>')) && (*q != '\0'))
         {
           (void) CopyMagickString(keyword,token,MaxTextExtent);
-          GetMagickToken(q,&q,token);
+          GetNextToken(q,&q,extent,token);
           if (*token != '=')
             continue;
-          GetMagickToken(q,&q,token);
+          GetNextToken(q,&q,extent,token);
           if (LocaleCompare(keyword,"file") == 0)
             {
               if (depth > 200)
@@ -1181,7 +1169,7 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
                   sans_exception=DestroyExceptionInfo(sans_exception);
                   if (xml != (char *) NULL)
                     {
-                      status&=LoadTypeCache(type_cache,xml,path,depth+1,
+                      status&=LoadTypeCache(cache,xml,path,depth+1,
                         exception);
                       xml=(char *) RelinquishMagickMemory(xml);
                     }
@@ -1205,20 +1193,21 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
       }
     if (type_info == (TypeInfo *) NULL)
       continue;
-    if (LocaleCompare(keyword,"/>") == 0)
+    if ((LocaleCompare(keyword,"/>") == 0) ||
+        (LocaleCompare(keyword,"</policy>") == 0))
       {
-        status=AddValueToSplayTree(type_cache,type_info->name,type_info);
+        status=AddValueToSplayTree(cache,type_info->name,type_info);
         if (status == MagickFalse)
           (void) ThrowMagickException(exception,GetMagickModule(),
             ResourceLimitError,"MemoryAllocationFailed","`%s'",type_info->name);
         type_info=(TypeInfo *) NULL;
         continue;
       }
-    GetMagickToken(q,(const char **) NULL,token);
+    GetNextToken(q,(const char **) NULL,extent,token);
     if (*token != '=')
       continue;
-    GetMagickToken(q,&q,token);
-    GetMagickToken(q,&q,token);
+    GetNextToken(q,&q,extent,token);
+    GetNextToken(q,&q,extent,token);
     switch (*keyword)
     {
       case 'E':
@@ -1322,11 +1311,13 @@ static MagickBooleanType LoadTypeCache(SplayTreeInfo *type_cache,
       {
         if (LocaleCompare((char *) keyword,"weight") == 0)
           {
-            type_info->weight=StringToUnsignedLong(token);
-            if (LocaleCompare(token,"bold") == 0)
-              type_info->weight=700;
-            if (LocaleCompare(token,"normal") == 0)
-              type_info->weight=400;
+            ssize_t
+              weight;
+
+            weight=ParseCommandOption(MagickWeightOptions,MagickFalse,token);
+            if (weight == -1)
+              weight=StringToUnsignedLong(token);
+            type_info->weight=(size_t) weight;
             break;
           }
         break;
