@@ -17,7 +17,7 @@
 %                              January 1993                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2016 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -1438,7 +1438,7 @@ MagickExport MagickBooleanType IsPathAccessible(const char *path)
 
   if ((path == (const char *) NULL) || (*path == '\0'))
     return(MagickFalse);
-  if (strcmp(path,"-") == 0)
+  if (LocaleCompare(path,"-") == 0)
     return(MagickTrue);
   status=GetPathAttributes(path,&attributes);
   if (status == MagickFalse)
@@ -1548,14 +1548,12 @@ static int FileCompare(const void *x,const void *y)
 static inline int MagickReadDirectory(DIR *directory,struct dirent *entry,
   struct dirent **result)
 {
-#if defined(MAGICKCORE_HAVE_READDIR_R)
-  return(readdir_r(directory,entry,result));
-#else
-  (void) entry;
   errno=0;
-  *result=readdir(directory);
-  return(errno);
-#endif
+  entry=readdir(directory);
+  *result=entry;
+  if ((entry == (struct dirent *) NULL) && (errno != 0))
+    return(-1);
+  return(0);
 }
 
 MagickExport char **ListFiles(const char *directory,const char *pattern,
@@ -1815,6 +1813,7 @@ MagickPrivate MagickBooleanType ShredFile(const char *path)
       /*
         Don't shred the file, just remove it.
       */
+      passes=DestroyString(passes);
       status=remove_utf8(path);
       if (status == -1)
         {
@@ -1830,6 +1829,7 @@ MagickPrivate MagickBooleanType ShredFile(const char *path)
       /*
         Don't shred the file, just remove it.
       */
+      passes=DestroyString(passes);
       status=remove_utf8(path);
       if (status == -1)
         (void) LogMagickEvent(ExceptionEvent,GetMagickModule(),
@@ -1881,7 +1881,8 @@ MagickPrivate MagickBooleanType ShredFile(const char *path)
   }
   status=close(file);
   status=remove_utf8(path);
-  if (status == -1)
-    return(MagickFalse);
-  return(i < (ssize_t) StringToInteger(passes) ? MagickFalse : MagickTrue);
+  if (status != -1)
+    status=StringToInteger(passes);
+  passes=DestroyString(passes);
+  return((status == -1 || i < (ssize_t) status) ? MagickFalse : MagickTrue);
 }
