@@ -17,7 +17,7 @@
 %                               September 2002                                %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -42,6 +42,7 @@
 #include "magick/exception-private.h"
 #include "magick/hashmap.h"
 #include "magick/locale_.h"
+#include "magick/memory_.h"
 #include "magick/option.h"
 #include "magick/string_.h"
 #include "magick/utility.h"
@@ -120,15 +121,18 @@ MagickExport const char *GetMagickDelegates(void)
 #if defined(MAGICKCORE_FFTW_DELEGATE)
   "fftw "
 #endif
+#if defined(MAGICKCORE_FLIF_DELEGATE)
+  "flif "
+#endif
 #if defined(MAGICKCORE_FONTCONFIG_DELEGATE)
   "fontconfig "
 #endif
-#if defined(MAGICKCORE_FREETYPE_DELEGATE)
-  "freetype "
-#endif
 #if defined(MAGICKCORE_FPX_DELEGATE)
   "fpx "
 #endif
+#if defined(MAGICKCORE_FREETYPE_DELEGATE)
+  "freetype "
+#endif
 #if defined(MAGICKCORE_GS_DELEGATE)
   "gslib "
 #endif
@@ -217,21 +221,30 @@ MagickExport const char *GetMagickDelegates(void)
 */
 MagickExport const char *GetMagickFeatures(void)
 {
-  return "DPC"
-#if defined(MAGICKCORE_HDRI_SUPPORT)
-  " HDRI"
-#endif
-#if defined(MAGICKCORE_BUILD_MODULES) || defined(_DLL)
-  " Modules"
-#endif
-#if defined(MAGICKCORE_OPENCL_SUPPORT)
-  " OpenCL"
-#endif
-#if defined(MAGICKCORE_OPENMP_SUPPORT)
-  " OpenMP"
+  return ""
+#if defined(MAGICKCORE_CIPHER_SUPPORT)
+  "Cipher "
 #endif
 #if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_DEBUG)
-  " Debug"
+  "Debug "
+#endif
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) || defined(MAGICKCORE_HAVE_SOCKET) && defined(MAGICKCORE_THREAD_SUPPORT)
+  "DPC "
+#endif
+#if defined(MAGICKCORE_HDRI_SUPPORT)
+  "HDRI "
+#endif
+#if defined(MAGICKCORE_BUILD_MODULES) || defined(_DLL)
+  "Modules "
+#endif
+#if defined(MAGICKCORE_OPENCL_SUPPORT)
+  "OpenCL "
+#endif
+#if defined(MAGICKCORE_OPENMP_SUPPORT)
+  "OpenMP "
+#endif
+#if defined(ZERO_CONFIGURATION_SUPPORT)
+  "Zero-configuration "
 #endif
   ;
 }
@@ -279,9 +292,13 @@ MagickExport char *GetMagickHomeURL(void)
     (void) FormatLocaleString(path,MaxTextExtent,"%s%s%s",element,
       DirectorySeparator,MagickURLFilename);
     if (IsPathAccessible(path) != MagickFalse)
-      return(ConstantString(path));
+      {
+        paths=DestroyLinkedList(paths,RelinquishMagickMemory);
+        return(ConstantString(path));
+      }
     element=(const char *) GetNextValueInLinkedList(paths);
   }
+  paths=DestroyLinkedList(paths,RelinquishMagickMemory);
   return(ConstantString(MagickHomeURL));
 }
 
@@ -290,6 +307,29 @@ MagickExport char *GetMagickHomeURL(void)
 %                                                                             %
 %                                                                             %
 %                                                                             %
+%   G e t M a g i c k L i c e n s e                                           %
+%                                                                             %
+%                                                                             %
+%                                                                             %
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%
+%  GetMagickLicense() returns the ImageMagick API license as a string.
+%
+%  The format of the GetMagickLicense method is:
+%
+%      const char *GetMagickLicense(void)
+%
+*/
+MagickExport const char *GetMagickLicense(void)
+{
+  return(MagickAuthoritativeLicense);
+}
+
+/*
+%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
+%                                                                             %
+%                                                                             %
+%                                                                             %
 %   G e t M a g i c k P a c k a g e N a m e                                   %
 %                                                                             %
 %                                                                             %
@@ -553,6 +593,11 @@ MagickExport void ListMagickVersion(FILE *file)
   (void) FormatLocaleFile(file,"Version: %s\n",
     GetMagickVersion((size_t *) NULL));
   (void) FormatLocaleFile(file,"Copyright: %s\n",GetMagickCopyright());
+  (void) FormatLocaleFile(file,"License: %s\n",GetMagickLicense());
+#if defined(MAGICKCORE_WINDOWS_SUPPORT) && defined(_MSC_FULL_VER)
+  (void) FormatLocaleFile(file,"Visual C++: %d\n",_MSC_FULL_VER);
+#endif
   (void) FormatLocaleFile(file,"Features: %s\n",GetMagickFeatures());
-  (void) FormatLocaleFile(file,"Delegates: %s\n\n",GetMagickDelegates());
+  (void) FormatLocaleFile(file,"Delegates (built-in): %s\n",
+    GetMagickDelegates());
 }
