@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 
   MagickCore VMS compatibility methods.
 */
-#ifndef _MAGICKCORE_VMS_H
-#define _MAGICKCORE_VMS_H
+#ifndef MAGICKCORE_VMS_H
+#define MAGICKCORE_VMS_H
 
 #if defined(__DECC) || defined(__DECCXX)
 #  include <lib$routines.h>
