@@ -23,7 +23,7 @@
 %                               December 2004                                 %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -52,6 +52,7 @@
 #include "magick/blob-private.h"
 #include "magick/exception.h"
 #include "magick/exception-private.h"
+#include "magick/image-private.h"
 #include "magick/log.h"
 #include "magick/memory_.h"
 #include "magick/semaphore.h"
@@ -595,14 +596,6 @@ MagickExport XMLTreeInfo *DestroyXMLTree(XMLTreeInfo *xml_info)
 %
 */
 
-static inline MagickSizeType MagickMin(const MagickSizeType x,
-  const MagickSizeType y)
-{
-  if (x < y)
-    return(x);
-  return(y);
-}
-
 MagickPrivate char *FileToXML(const char *filename,const size_t extent)
 {
   char
@@ -649,9 +642,8 @@ MagickPrivate char *FileToXML(const char *filename,const size_t extent)
       */
       offset=(MagickOffsetType) lseek(file,0,SEEK_SET);
       quantum=(size_t) MagickMaxBufferExtent;
-      if ((fstat(file,&file_stats) == 0) && (file_stats.st_size != 0))
-        quantum=(size_t) MagickMin((MagickSizeType) file_stats.st_size,
-          MagickMaxBufferExtent);
+      if ((fstat(file,&file_stats) == 0) && (file_stats.st_size > 0))
+        quantum=(size_t) MagickMin(file_stats.st_size,MagickMaxBufferExtent);
       xml=(char *) AcquireQuantumMemory(quantum,sizeof(*xml));
       for (i=0; xml != (char *) NULL; i+=count)
       {
@@ -684,7 +676,7 @@ MagickPrivate char *FileToXML(const char *filename,const size_t extent)
       xml[length]='\0';
       return(xml);
     }
-  length=(size_t) MagickMin((MagickSizeType) offset,extent);
+  length=(size_t) MagickMin(offset,(MagickOffsetType) extent);
   xml=(char *) NULL;
   if (~length >= (MaxTextExtent-1))
     xml=(char *) AcquireQuantumMemory(length+MaxTextExtent,sizeof(*xml));
@@ -704,8 +696,7 @@ MagickPrivate char *FileToXML(const char *filename,const size_t extent)
       (void) lseek(file,0,SEEK_SET);
       for (i=0; i < length; i+=count)
       {
-        count=read(file,xml+i,(size_t) MagickMin(length-i,(MagickSizeType)
-          SSIZE_MAX));
+        count=read(file,xml+i,(size_t) MagickMin(length-i,SSIZE_MAX));
         if (count <= 0)
           {
             count=0;
@@ -1315,7 +1306,7 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
   ssize_t
     j;
 
-  utf8=(char *) AcquireQuantumMemory(*length,sizeof(*utf8));
+  utf8=(char *) AcquireQuantumMemory(*length+1,sizeof(*utf8));
   if (utf8 == (char *) NULL)
     return((char *) NULL);
   encoding=(*content == '\xFE') ? 1 : (*content == '\xFF') ? 0 : -1;
@@ -1325,6 +1316,7 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
         Already UTF-8.
       */
       (void) CopyMagickMemory(utf8,content,*length*sizeof(*utf8));
+      utf8[*length]='\0';
       return(utf8);
     }
   j=0;
@@ -1369,7 +1361,10 @@ static char *ConvertUTF16ToUTF8(const char *content,size_t *length)
     }
   }
   *length=(size_t) j;
-  return((char *) ResizeQuantumMemory(utf8,*length,sizeof(*utf8)));
+  utf8=(char *) ResizeQuantumMemory(utf8,*length,sizeof(*utf8));
+  if (utf8 != (char *) NULL)
+    utf8[*length]='\0';
+  return(utf8);
 }
 
 static char *ParseEntities(char *xml,char **entities,int state)
@@ -1521,12 +1516,16 @@ static char *ParseEntities(char *xml,char **entities,int state)
   }
   if (state == '*')
     {
+     
       /*
         Normalize spaces for non-CDATA attributes.
       */
       for (xml=p; *xml != '\0'; xml++)
       {
-        i=(ssize_t) strspn(xml," ");
+        char
+          accept[] = " ";
+
+        i=(ssize_t) strspn(xml,accept);
         if (i != 0)
           (void) CopyMagickMemory(xml,xml+i,strlen(xml+i)+1);
         while ((*xml != '\0') && (*xml != ' '))
@@ -1672,7 +1671,7 @@ static void ParseProcessingInstructions(XMLTreeRoot *root,char *xml,
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   root->processing_instructions[i][j+2]=(char *) ResizeQuantumMemory(
     root->processing_instructions[i][j+1],(size_t) (j+1),
-    sizeof(*root->processing_instructions));
+    sizeof(***root->processing_instructions));
   if (root->processing_instructions[i][j+2] == (char *) NULL)
     ThrowFatalException(ResourceLimitFatalError,"MemoryAllocationFailed");
   (void) CopyMagickString(root->processing_instructions[i][j+2]+j-1,
@@ -1854,7 +1853,7 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                     ThrowFatalException(ResourceLimitFatalError,
                       "MemoryAllocationFailed");
                   root->attributes[i]=(char **) AcquireQuantumMemory(2,
-                    sizeof(*root->attributes));
+                    sizeof(**root->attributes));
                   if (root->attributes[i] == (char **) NULL)
                     ThrowFatalException(ResourceLimitFatalError,
                       "MemoryAllocationFailed");
@@ -1864,7 +1863,7 @@ static MagickBooleanType ParseInternalDoctype(XMLTreeRoot *root,char *xml,
                 }
               for (j=1; root->attributes[i][j] != (char *) NULL; j+=3) ;
               root->attributes[i]=(char **) ResizeQuantumMemory(
-                root->attributes[i],(size_t) (j+4),sizeof(*root->attributes));
+                root->attributes[i],(size_t) (j+4),sizeof(**root->attributes));
               if (root->attributes[i] == (char **) NULL)
                 ThrowFatalException(ResourceLimitFatalError,
                   "MemoryAllocationFailed");
@@ -1921,7 +1920,7 @@ static const char
   {
     "rdf:Bag",
     "rdf:Seq",
-    (const char *)NULL
+    (const char *) NULL
   };
 
 static inline MagickBooleanType IsSkipTag(const char *tag)
@@ -2141,7 +2140,10 @@ MagickExport XMLTreeInfo *NewXMLTree(const char *xml,ExceptionInfo *exception)
                 if ((ignore_depth == 0) && (IsSkipTag(tag) == MagickFalse))
                   ParseOpenTag(root,tag,attributes);
                 else
-                  ignore_depth++;
+                  {
+                    ignore_depth++;
+                    (void) DestroyXMLTreeAttributes(attributes);
+                  }
                 *p=c;
               }
             else
@@ -2780,9 +2782,7 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
   root=(XMLTreeRoot *) xml_info;
   while (root->root.parent != (XMLTreeInfo *) NULL)
     root=(XMLTreeRoot *) root->root.parent;
-  parent=(XMLTreeInfo *) NULL;
-  if (xml_info != (XMLTreeInfo *) NULL)
-    parent=xml_info->parent;
+  parent=xml_info->parent;
   if (parent == (XMLTreeInfo *) NULL)
     for (i=0; root->processing_instructions[i] != (char **) NULL; i++)
     {
@@ -2811,9 +2811,7 @@ MagickExport char *XMLTreeInfoToXML(XMLTreeInfo *xml_info)
         p=root->processing_instructions[i][j];
       }
     }
-  ordered=(XMLTreeInfo *) NULL;
-  if (xml_info != (XMLTreeInfo *) NULL)
-    ordered=xml_info->ordered;
+  ordered=xml_info->ordered;
   xml_info->parent=(XMLTreeInfo *) NULL;
   xml_info->ordered=(XMLTreeInfo *) NULL;
   xml=XMLTreeTagToXML(xml_info,&xml,&length,&extent,0,root->attributes);
