@@ -17,7 +17,7 @@
 %                                 July 1992                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -379,6 +379,9 @@ static void RelinquishCompositeOptions(CompositeOptions *composite_options)
   if (composite_options->compose_args != (char *) NULL)
     composite_options->compose_args=(char *)
       RelinquishMagickMemory(composite_options->compose_args);
+  if (composite_options->geometry != (char *) NULL)
+    composite_options->geometry=(char *)
+      RelinquishMagickMemory(composite_options->geometry);
 }
 
 WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
@@ -1656,6 +1659,7 @@ WandExport MagickBooleanType CompositeImageCommand(ImageInfo *image_info,
         *text;
 
       text=InterpretImageProperties(image_info,images,format);
+      InheritException(exception,&image->exception);
       if (text == (char *) NULL)
         ThrowCompositeException(ResourceLimitError,"MemoryAllocationFailed",
           GetExceptionMessage(errno));
