@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 
   MagickWand composite command-line method.
 */
-#ifndef _MAGICKWAND_COMPOSITE_H
-#define _MAGICKWAND_COMPOSITE_H
+#ifndef MAGICKWAND_COMPOSITE_H
+#define MAGICKWAND_COMPOSITE_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
