@@ -17,7 +17,7 @@
 %                            September 1994                                   %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -331,6 +331,7 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
           continue;
         AppendImageStack(images);
         FinalizeImageSettings(image_info,image,MagickFalse);
+        count=0;
         for ( ; image != (Image *) NULL; image=GetNextImageInList(image))
         {
           if (image->scene == 0)
@@ -346,13 +347,12 @@ WandExport MagickBooleanType IdentifyImageCommand(ImageInfo *image_info,
                 *text;
 
               text=InterpretImageProperties(image_info,image,format);
+              InheritException(exception,&image->exception);
               if (text == (char *) NULL)
                 ThrowIdentifyException(ResourceLimitError,
                   "MemoryAllocationFailed",GetExceptionMessage(errno));
               (void) ConcatenateString(&(*metadata),text);
               text=DestroyString(text);
-              if (GlobExpression(format,"*%n*",MagickFalse) != MagickFalse)
-                break;
             }
         }
         RemoveAllImageStack();
