@@ -17,7 +17,7 @@
 %                              July 1992                                      %
 %                                                                             %
 %                                                                             %
-%  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization      %
+%  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization      %
 %  dedicated to making software imaging solutions freely available.           %
 %                                                                             %
 %  You may not use this file except in compliance with the License.  You may  %
@@ -44,8 +44,9 @@
 #include "wand/studio.h"
 #include "wand/MagickWand.h"
 #include "wand/mogrify-private.h"
-#include "magick/xwindow-private.h"
 #include "magick/string-private.h"
+#include "magick/image-private.h"
+#include "magick/xwindow-private.h"
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -209,13 +210,6 @@ static MagickBooleanType ImportUsage(void)
   return(MagickFalse);
 }
 
-static inline ssize_t MagickMax(const ssize_t x,const ssize_t y)
-{
-  if (x > y)
-    return(x);
-  return(y);
-}
-
 WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
   int argc,char **argv,char **wand_unused(metadata),ExceptionInfo *exception)
 {
@@ -460,7 +454,7 @@ WandExport MagickBooleanType ImportImageCommand(ImageInfo *image_info,
             MaxTextExtent);
         for (scene=0; scene < (size_t) MagickMax(snapshots,1); scene++)
         {
-          (void) sleep(resource_info.pause);
+          MagickDelay(1000*resource_info.pause);
           images=XImportImage(image_info,&ximage_info);
           status&=(images != (Image *) NULL) &&
             (exception->severity < ErrorException);
