@@ -1,5 +1,5 @@
 /*
-  Copyright 1999-2014 ImageMagick Studio LLC, a non-profit organization
+  Copyright 1999-2017 ImageMagick Studio LLC, a non-profit organization
   dedicated to making software imaging solutions freely available.
   
   You may not use this file except in compliance with the License.
@@ -15,8 +15,8 @@
 
   MagickWand montage command-line method.
 */
-#ifndef _MAGICKWAND_MONTAGE_H
-#define _MAGICKWAND_MONTAGE_H
+#ifndef MAGICKWAND_MONTAGE_H
+#define MAGICKWAND_MONTAGE_H
 
 #if defined(__cplusplus) || defined(c_plusplus)
 extern "C" {
